@@ -23,14 +23,14 @@ THE SOFTWARE.
 #ifndef HIP_INCLUDE_HIP_AMD_DETAIL_WARP_FUNCTIONS_H
 #define HIP_INCLUDE_HIP_AMD_DETAIL_WARP_FUNCTIONS_H
 
-#if defined(__clang__)
-#pragma clang diagnostic push
-#pragma clang diagnostic ignored "-Wreserved-identifier"
-#pragma clang diagnostic ignored "-Wreserved-macro-identifier"
-#pragma clang diagnostic ignored "-Wsign-conversion"
-#pragma clang diagnostic ignored "-Wold-style-cast"
-#pragma clang diagnostic ignored "-Wc++98-compat"
-#pragma clang diagnostic ignored "-Wc++98-compat-pedantic"
+#if !defined(__HIPCC_RTC__)
+#include "device_library_decls.h"  // ockl warp functions
+#endif // !defined(__HIPCC_RTC__)
+
+#if defined(__has_attribute) && __has_attribute(maybe_undef)
+#define MAYBE_UNDEF __attribute__((maybe_undef))
+#else
+#define MAYBE_UNDEF
 #endif
 
 __device__ static inline unsigned __hip_ds_bpermute(int index, unsigned src) {
@@ -83,32 +83,85 @@ __device__ static inline int __hip_move_dpp_N(int src) {
                                     bound_ctrl);
 }
 
-static constexpr int warpSize = __AMDGCN_WAVEFRONT_SIZE;
+#if defined(__SPIRV__)
+    inline __device__ const struct final {
+        __device__
+        __attribute__((always_inline, const))
+        operator int() const noexcept {
+            return __builtin_amdgcn_wavefrontsize();
+        }
+    } warpSize{};
+#else
+    __device__
+    static constexpr int warpSize = __AMDGCN_WAVEFRONT_SIZE;
+#endif
+
+// warp vote function __all __any __ballot
+__device__
+inline
+int __all(int predicate) {
+    return __ockl_wfall_i32(predicate);
+}
 
 __device__
 inline
-int __shfl(int var, int src_lane, int width = warpSize) {
+int __any(int predicate) {
+    return __ockl_wfany_i32(predicate);
+}
+
+__device__
+inline
+unsigned long long int __ballot(int predicate) {
+    return __builtin_amdgcn_ballot_w64(predicate);
+}
+
+__device__
+inline
+unsigned long long int __ballot64(int predicate) {
+    return __ballot(predicate);
+}
+
+// See amd_warp_sync_functions.h for an explanation of this preprocessor flag.
+#ifdef HIP_ENABLE_WARP_SYNC_BUILTINS
+// Since threads in a wave do not make independent progress, __activemask()
+// always returns the exact active mask, i.e, all active threads in the wave.
+__device__
+inline
+unsigned long long __activemask() {
+  return __ballot(true);
+}
+#endif // HIP_ENABLE_WARP_SYNC_BUILTINS
+
+__device__ static inline unsigned int __lane_id() {
+    if (warpSize == 32) return __builtin_amdgcn_mbcnt_lo(-1, 0);
+    return  __builtin_amdgcn_mbcnt_hi(
+        -1, __builtin_amdgcn_mbcnt_lo(-1, 0));
+}
+
+__device__
+inline
+int __shfl(MAYBE_UNDEF int var, int src_lane, int width = warpSize) {
     int self = __lane_id();
     int index = (src_lane & (width - 1)) + (self & ~(width-1));
     return __builtin_amdgcn_ds_bpermute(index<<2, var);
 }
 __device__
 inline
-unsigned int __shfl(unsigned int var, int src_lane, int width = warpSize) {
+unsigned int __shfl(MAYBE_UNDEF unsigned int var, int src_lane, int width = warpSize) {
      union { int i; unsigned u; float f; } tmp; tmp.u = var;
     tmp.i = __shfl(tmp.i, src_lane, width);
     return tmp.u;
 }
 __device__
 inline
-float __shfl(float var, int src_lane, int width = warpSize) {
+float __shfl(MAYBE_UNDEF float var, int src_lane, int width = warpSize) {
     union { int i; unsigned u; float f; } tmp; tmp.f = var;
     tmp.i = __shfl(tmp.i, src_lane, width);
     return tmp.f;
 }
 __device__
 inline
-double __shfl(double var, int src_lane, int width = warpSize) {
+double __shfl(MAYBE_UNDEF double var, int src_lane, int width = warpSize) {
     static_assert(sizeof(double) == 2 * sizeof(int), "");
     static_assert(sizeof(double) == sizeof(uint64_t), "");
 
@@ -122,7 +175,7 @@ double __shfl(double var, int src_lane, int width = warpSize) {
 }
 __device__
 inline
-long __shfl(long var, int src_lane, int width = warpSize)
+long __shfl(MAYBE_UNDEF long var, int src_lane, int width = warpSize)
 {
     #ifndef _MSC_VER
     static_assert(sizeof(long) == 2 * sizeof(int), "");
@@ -142,7 +195,7 @@ long __shfl(long var, int src_lane, int width = warpSize)
 }
 __device__
 inline
-unsigned long __shfl(unsigned long var, int src_lane, int width = warpSize) {
+unsigned long __shfl(MAYBE_UNDEF unsigned long var, int src_lane, int width = warpSize) {
     #ifndef _MSC_VER
     static_assert(sizeof(unsigned long) == 2 * sizeof(unsigned int), "");
     static_assert(sizeof(unsigned long) == sizeof(uint64_t), "");
@@ -161,7 +214,7 @@ unsigned long __shfl(unsigned long var, int src_lane, int width = warpSize) {
 }
 __device__
 inline
-long long __shfl(long long var, int src_lane, int width = warpSize)
+long long __shfl(MAYBE_UNDEF long long var, int src_lane, int width = warpSize)
 {
     static_assert(sizeof(long long) == 2 * sizeof(int), "");
     static_assert(sizeof(long long) == sizeof(uint64_t), "");
@@ -176,7 +229,7 @@ long long __shfl(long long var, int src_lane, int width = warpSize)
 }
 __device__
 inline
-unsigned long long __shfl(unsigned long long var, int src_lane, int width = warpSize) {
+unsigned long long __shfl(MAYBE_UNDEF unsigned long long var, int src_lane, int width = warpSize) {
     static_assert(sizeof(unsigned long long) == 2 * sizeof(unsigned int), "");
     static_assert(sizeof(unsigned long long) == sizeof(uint64_t), "");
 
@@ -191,7 +244,7 @@ unsigned long long __shfl(unsigned long long var, int src_lane, int width = warp
 
 __device__
 inline
-int __shfl_up(int var, unsigned int lane_delta, int width = warpSize) {
+int __shfl_up(MAYBE_UNDEF int var, unsigned int lane_delta, int width = warpSize) {
     int self = __lane_id();
     int index = self - lane_delta;
     index = (index < (self & ~(width-1)))?self:index;
@@ -199,21 +252,21 @@ int __shfl_up(int var, unsigned int lane_delta, int width = warpSize) {
 }
 __device__
 inline
-unsigned int __shfl_up(unsigned int var, unsigned int lane_delta, int width = warpSize) {
+unsigned int __shfl_up(MAYBE_UNDEF unsigned int var, unsigned int lane_delta, int width = warpSize) {
     union { int i; unsigned u; float f; } tmp; tmp.u = var;
     tmp.i = __shfl_up(tmp.i, lane_delta, width);
     return tmp.u;
 }
 __device__
 inline
-float __shfl_up(float var, unsigned int lane_delta, int width = warpSize) {
+float __shfl_up(MAYBE_UNDEF float var, unsigned int lane_delta, int width = warpSize) {
     union { int i; unsigned u; float f; } tmp; tmp.f = var;
     tmp.i = __shfl_up(tmp.i, lane_delta, width);
     return tmp.f;
 }
 __device__
 inline
-double __shfl_up(double var, unsigned int lane_delta, int width = warpSize) {
+double __shfl_up(MAYBE_UNDEF double var, unsigned int lane_delta, int width = warpSize) {
     static_assert(sizeof(double) == 2 * sizeof(int), "");
     static_assert(sizeof(double) == sizeof(uint64_t), "");
 
@@ -227,7 +280,7 @@ double __shfl_up(double var, unsigned int lane_delta, int width = warpSize) {
 }
 __device__
 inline
-long __shfl_up(long var, unsigned int lane_delta, int width = warpSize)
+long __shfl_up(MAYBE_UNDEF long var, unsigned int lane_delta, int width = warpSize)
 {
     #ifndef _MSC_VER
     static_assert(sizeof(long) == 2 * sizeof(int), "");
@@ -248,7 +301,7 @@ long __shfl_up(long var, unsigned int lane_delta, int width = warpSize)
 
 __device__
 inline
-unsigned long __shfl_up(unsigned long var, unsigned int lane_delta, int width = warpSize)
+unsigned long __shfl_up(MAYBE_UNDEF unsigned long var, unsigned int lane_delta, int width = warpSize)
 {
     #ifndef _MSC_VER
     static_assert(sizeof(unsigned long) == 2 * sizeof(unsigned int), "");
@@ -269,7 +322,7 @@ unsigned long __shfl_up(unsigned long var, unsigned int lane_delta, int width =
 
 __device__
 inline
-long long __shfl_up(long long var, unsigned int lane_delta, int width = warpSize)
+long long __shfl_up(MAYBE_UNDEF long long var, unsigned int lane_delta, int width = warpSize)
 {
     static_assert(sizeof(long long) == 2 * sizeof(int), "");
     static_assert(sizeof(long long) == sizeof(uint64_t), "");
@@ -283,7 +336,7 @@ long long __shfl_up(long long var, unsigned int lane_delta, int width = warpSize
 
 __device__
 inline
-unsigned long long __shfl_up(unsigned long long var, unsigned int lane_delta, int width = warpSize)
+unsigned long long __shfl_up(MAYBE_UNDEF unsigned long long var, unsigned int lane_delta, int width = warpSize)
 {
     static_assert(sizeof(unsigned long long) == 2 * sizeof(unsigned int), "");
     static_assert(sizeof(unsigned long long) == sizeof(uint64_t), "");
@@ -297,7 +350,7 @@ unsigned long long __shfl_up(unsigned long long var, unsigned int lane_delta, in
 
 __device__
 inline
-int __shfl_down(int var, unsigned int lane_delta, int width = warpSize) {
+int __shfl_down(MAYBE_UNDEF int var, unsigned int lane_delta, int width = warpSize) {
     int self = __lane_id();
     int index = self + lane_delta;
     index = (int)((self&(width-1))+lane_delta) >= width?self:index;
@@ -305,21 +358,21 @@ int __shfl_down(int var, unsigned int lane_delta, int width = warpSize) {
 }
 __device__
 inline
-unsigned int __shfl_down(unsigned int var, unsigned int lane_delta, int width = warpSize) {
+unsigned int __shfl_down(MAYBE_UNDEF unsigned int var, unsigned int lane_delta, int width = warpSize) {
     union { int i; unsigned u; float f; } tmp; tmp.u = var;
     tmp.i = __shfl_down(tmp.i, lane_delta, width);
     return tmp.u;
 }
 __device__
 inline
-float __shfl_down(float var, unsigned int lane_delta, int width = warpSize) {
+float __shfl_down(MAYBE_UNDEF float var, unsigned int lane_delta, int width = warpSize) {
     union { int i; unsigned u; float f; } tmp; tmp.f = var;
     tmp.i = __shfl_down(tmp.i, lane_delta, width);
     return tmp.f;
 }
 __device__
 inline
-double __shfl_down(double var, unsigned int lane_delta, int width = warpSize) {
+double __shfl_down(MAYBE_UNDEF double var, unsigned int lane_delta, int width = warpSize) {
     static_assert(sizeof(double) == 2 * sizeof(int), "");
     static_assert(sizeof(double) == sizeof(uint64_t), "");
 
@@ -333,7 +386,7 @@ double __shfl_down(double var, unsigned int lane_delta, int width = warpSize) {
 }
 __device__
 inline
-long __shfl_down(long var, unsigned int lane_delta, int width = warpSize)
+long __shfl_down(MAYBE_UNDEF long var, unsigned int lane_delta, int width = warpSize)
 {
     #ifndef _MSC_VER
     static_assert(sizeof(long) == 2 * sizeof(int), "");
@@ -353,7 +406,7 @@ long __shfl_down(long var, unsigned int lane_delta, int width = warpSize)
 }
 __device__
 inline
-unsigned long __shfl_down(unsigned long var, unsigned int lane_delta, int width = warpSize)
+unsigned long __shfl_down(MAYBE_UNDEF unsigned long var, unsigned int lane_delta, int width = warpSize)
 {
     #ifndef _MSC_VER
     static_assert(sizeof(unsigned long) == 2 * sizeof(unsigned int), "");
@@ -373,7 +426,7 @@ unsigned long __shfl_down(unsigned long var, unsigned int lane_delta, int width
 }
 __device__
 inline
-long long __shfl_down(long long var, unsigned int lane_delta, int width = warpSize)
+long long __shfl_down(MAYBE_UNDEF long long var, unsigned int lane_delta, int width = warpSize)
 {
     static_assert(sizeof(long long) == 2 * sizeof(int), "");
     static_assert(sizeof(long long) == sizeof(uint64_t), "");
@@ -386,7 +439,7 @@ long long __shfl_down(long long var, unsigned int lane_delta, int width = warpSi
 }
 __device__
 inline
-unsigned long long __shfl_down(unsigned long long var, unsigned int lane_delta, int width = warpSize)
+unsigned long long __shfl_down(MAYBE_UNDEF unsigned long long var, unsigned int lane_delta, int width = warpSize)
 {
     static_assert(sizeof(unsigned long long) == 2 * sizeof(unsigned int), "");
     static_assert(sizeof(unsigned long long) == sizeof(uint64_t), "");
@@ -400,7 +453,7 @@ unsigned long long __shfl_down(unsigned long long var, unsigned int lane_delta,
 
 __device__
 inline
-int __shfl_xor(int var, int lane_mask, int width = warpSize) {
+int __shfl_xor(MAYBE_UNDEF int var, int lane_mask, int width = warpSize) {
     int self = __lane_id();
     int index = self^lane_mask;
     index = index >= ((self+width)&~(width-1))?self:index;
@@ -408,21 +461,21 @@ int __shfl_xor(int var, int lane_mask, int width = warpSize) {
 }
 __device__
 inline
-unsigned int __shfl_xor(unsigned int var, int lane_mask, int width = warpSize) {
+unsigned int __shfl_xor(MAYBE_UNDEF unsigned int var, int lane_mask, int width = warpSize) {
     union { int i; unsigned u; float f; } tmp; tmp.u = var;
     tmp.i = __shfl_xor(tmp.i, lane_mask, width);
     return tmp.u;
 }
 __device__
 inline
-float __shfl_xor(float var, int lane_mask, int width = warpSize) {
+float __shfl_xor(MAYBE_UNDEF float var, int lane_mask, int width = warpSize) {
     union { int i; unsigned u; float f; } tmp; tmp.f = var;
     tmp.i = __shfl_xor(tmp.i, lane_mask, width);
     return tmp.f;
 }
 __device__
 inline
-double __shfl_xor(double var, int lane_mask, int width = warpSize) {
+double __shfl_xor(MAYBE_UNDEF double var, int lane_mask, int width = warpSize) {
     static_assert(sizeof(double) == 2 * sizeof(int), "");
     static_assert(sizeof(double) == sizeof(uint64_t), "");
 
@@ -436,7 +489,7 @@ double __shfl_xor(double var, int lane_mask, int width = warpSize) {
 }
 __device__
 inline
-long __shfl_xor(long var, int lane_mask, int width = warpSize)
+long __shfl_xor(MAYBE_UNDEF long var, int lane_mask, int width = warpSize)
 {
     #ifndef _MSC_VER
     static_assert(sizeof(long) == 2 * sizeof(int), "");
@@ -456,7 +509,7 @@ long __shfl_xor(long var, int lane_mask, int width = warpSize)
 }
 __device__
 inline
-unsigned long __shfl_xor(unsigned long var, int lane_mask, int width = warpSize)
+unsigned long __shfl_xor(MAYBE_UNDEF unsigned long var, int lane_mask, int width = warpSize)
 {
     #ifndef _MSC_VER
     static_assert(sizeof(unsigned long) == 2 * sizeof(unsigned int), "");
@@ -476,7 +529,7 @@ unsigned long __shfl_xor(unsigned long var, int lane_mask, int width = warpSize)
 }
 __device__
 inline
-long long __shfl_xor(long long var, int lane_mask, int width = warpSize)
+long long __shfl_xor(MAYBE_UNDEF long long var, int lane_mask, int width = warpSize)
 {
     static_assert(sizeof(long long) == 2 * sizeof(int), "");
     static_assert(sizeof(long long) == sizeof(uint64_t), "");
@@ -489,7 +542,7 @@ long long __shfl_xor(long long var, int lane_mask, int width = warpSize)
 }
 __device__
 inline
-unsigned long long __shfl_xor(unsigned long long var, int lane_mask, int width = warpSize)
+unsigned long long __shfl_xor(MAYBE_UNDEF unsigned long long var, int lane_mask, int width = warpSize)
 {
     static_assert(sizeof(unsigned long long) == 2 * sizeof(unsigned int), "");
     static_assert(sizeof(unsigned long long) == sizeof(uint64_t), "");
@@ -501,8 +554,4 @@ unsigned long long __shfl_xor(unsigned long long var, int lane_mask, int width =
     return tmp1;
 }
 
-#if defined(__clang__)
-#pragma clang diagnostic pop
-#endif
-
 #endif
