@@ -116,6 +116,16 @@ cl_mem_object_type getCLMemObjectType(const unsigned int hipWidth,
   return CL_MEM_OBJECT_ALLOCATION_FAILURE;
 }
 
+inline cl_mem_object_type getCLMemObjectType(const hipArray* arr) {
+  const cl_mem dstMemObj = reinterpret_cast<const cl_mem>(arr->data);
+  const amd::Image* dstImage = as_amd(dstMemObj)->asImage();
+  return dstImage ? dstImage->getType() : CL_MEM_OBJECT_ALLOCATION_FAILURE;
+}
+
+inline bool isLayered1D(const hipArray* arr) {
+  return CL_MEM_OBJECT_IMAGE1D_ARRAY == getCLMemObjectType(arr);
+}
+
 inline
 cl_addressing_mode getCLAddressingMode(const hipTextureAddressMode hipAddressMode) {
   switch (hipAddressMode) {
@@ -594,6 +604,7 @@ std::pair<hipMemoryType, hipMemoryType> getMemoryType(const hipMemcpyKind kind)
     case hipMemcpyDeviceToHost:
       return {hipMemoryTypeDevice, hipMemoryTypeHost};
     case hipMemcpyDeviceToDevice:
+    case hipMemcpyDeviceToDeviceNoCU:
       return {hipMemoryTypeDevice, hipMemoryTypeDevice};
     case hipMemcpyDefault:
       return {hipMemoryTypeUnified, hipMemoryTypeUnified};
@@ -615,7 +626,8 @@ HIP_MEMCPY3D getDrvMemcpy3DDesc(const hip_Memcpy2D& desc2D) {
   desc3D.srcHost = desc2D.srcHost;
   desc3D.srcDevice = desc2D.srcDevice;
   desc3D.srcArray = desc2D.srcArray;
-  desc3D.srcPitch = desc2D.srcPitch;
+  desc3D.srcPitch = desc2D.srcPitch ? desc2D.srcPitch
+                                    : (desc2D.srcXInBytes + desc2D.WidthInBytes);
   desc3D.srcHeight = 0;
 
   desc3D.dstXInBytes = desc2D.dstXInBytes;
@@ -626,7 +638,8 @@ HIP_MEMCPY3D getDrvMemcpy3DDesc(const hip_Memcpy2D& desc2D) {
   desc3D.dstHost = desc2D.dstHost;
   desc3D.dstDevice = desc2D.dstDevice;
   desc3D.dstArray = desc2D.dstArray;
-  desc3D.dstPitch = desc2D.dstPitch;
+  desc3D.dstPitch = desc2D.dstPitch ? desc2D.dstPitch
+                                    : (desc2D.dstXInBytes + desc2D.WidthInBytes);
   desc3D.dstHeight = 0;
 
   desc3D.WidthInBytes = desc2D.WidthInBytes;
@@ -654,11 +667,15 @@ HIP_MEMCPY3D getDrvMemcpy3DDesc(const hipMemcpy3DParms& desc) {
   descDrv.dstZ = desc.dstPos.z;
   descDrv.dstLOD = 0;
 
+  bool is1DArraySrc = false;
+  bool is1DArrayDst = false;
+
   if (desc.srcArray != nullptr) {
     descDrv.srcMemoryType = hipMemoryTypeArray;
     descDrv.srcArray = desc.srcArray;
     // When reffering to array memory, hipPos::x is in elements.
     descDrv.srcXInBytes *= getElementSize(desc.srcArray);
+    is1DArraySrc = isLayered1D(descDrv.srcArray);
   }
 
   if (desc.srcPtr.ptr != nullptr) {
@@ -674,6 +691,7 @@ HIP_MEMCPY3D getDrvMemcpy3DDesc(const hipMemcpy3DParms& desc) {
     descDrv.dstArray = desc.dstArray;
     // When reffering to array memory, hipPos::x is in elements.
     descDrv.dstXInBytes *= getElementSize(desc.dstArray);
+    is1DArrayDst = isLayered1D(descDrv.dstArray);
   }
 
   if (desc.dstPtr.ptr != nullptr) {
@@ -693,6 +711,25 @@ HIP_MEMCPY3D getDrvMemcpy3DDesc(const hipMemcpy3DParms& desc) {
     descDrv.WidthInBytes *= getElementSize(desc.dstArray);
   }
 
+  // The following will happen in
+  // buffer to/from layered 1D, layered 1D to/from layered 1D
+  size_t t = 0;
+  if (is1DArraySrc) {
+    t = descDrv.srcY;
+    descDrv.srcY = descDrv.srcZ;
+    descDrv.srcZ = t;
+  }
+  if (is1DArrayDst) {
+    t = descDrv.dstY;
+    descDrv.dstY = descDrv.dstZ;
+    descDrv.dstZ = t;
+  }
+
+  if (is1DArraySrc || is1DArrayDst) {
+    t = descDrv.Height;
+    descDrv.Height = descDrv.Depth;
+    descDrv.Depth = t;
+  }
   return descDrv;
 }
 
