@@ -86,6 +86,9 @@ inline std::ostream& operator<<(std::ostream& os, const hipMemcpyKind& s) {
     case hipMemcpyDefault:
       os << "hipMemcpyDefault";
       break;
+    case hipMemcpyDeviceToDeviceNoCU:
+      os << "hipMemcpyDeviceToDeviceNoCU";
+      break;
     default:
       os << "hipMemcpyDefault";
   };
@@ -343,7 +346,9 @@ inline std::ostream& operator<<(std::ostream& os, const hiprtcResult& s) {
 }
 
 inline std::ostream& operator<<(std::ostream& os, const hipJitOption& s) {
-  switch (s) {
+  hiprtcJIT_option option = (hiprtcJIT_option)((int)s);
+
+  switch (option) {
     case HIPRTC_JIT_MAX_REGISTERS:
       os << "HIPRTC_JIT_MAX_REGISTERS";
       break;
@@ -479,6 +484,84 @@ inline std::ostream& operator<<(std::ostream& os, const hipDataType& s) {
     case HIP_C_64F:
       os << "HIP_C_64F";
       break;
+    case HIP_R_8I:
+      os << "HIP_R_8I";
+      break;
+    case HIP_C_8I:
+      os << "HIP_C_8I";
+      break;
+    case HIP_R_8U:
+      os << "HIP_R_8U";
+      break;
+    case HIP_C_8U:
+      os << "HIP_C_8U";
+      break;
+    case HIP_R_32I:
+      os << "HIP_R_32I";
+      break;
+    case HIP_C_32I:
+      os << "HIP_C_32I";
+      break;
+    case HIP_R_32U:
+      os << "HIP_R_32U";
+      break;
+    case HIP_C_32U:
+      os << "HIP_C_32U";
+      break;
+    case HIP_R_16BF:
+      os << "HIP_R_16BF";
+      break;
+    case HIP_C_16BF:
+      os << "HIP_C_16BF";
+      break;
+    case HIP_R_4I:
+      os << "HIP_R_4I";
+      break;
+    case HIP_C_4I:
+      os << "HIP_C_4I";
+      break;
+    case HIP_R_4U:
+      os << "HIP_R_4U";
+      break;
+    case HIP_C_4U:
+      os << "HIP_C_4U";
+      break;
+    case HIP_R_16I:
+      os << "HIP_R_16I";
+      break;
+    case HIP_C_16I:
+      os << "HIP_C_16I";
+      break;
+    case HIP_R_16U:
+      os << "HIP_R_16U";
+      break;
+    case HIP_C_16U:
+      os << "HIP_C_16U";
+      break;
+    case HIP_R_64I:
+      os << "HIP_R_64I";
+      break;
+    case HIP_C_64I:
+      os << "HIP_C_64I";
+      break;
+    case HIP_R_64U:
+      os << "HIP_R_64U";
+      break;
+    case HIP_C_64U:
+      os << "HIP_C_64U";
+      break;
+    case HIP_R_8F_E4M3:
+      os << "HIP_R_8F_E4M3";
+      break;
+    case HIP_R_8F_E5M2:
+      os << "HIP_R_8F_E5M2";
+      break;
+    case HIP_R_8F_E4M3_FNUZ:
+      os << "HIP_R_8F_E4M3_FNUZ";
+      break;
+    case HIP_R_8F_E5M2_FNUZ:
+      os << "HIP_R_8F_E5M2_FNUZ";
+      break;
     default:
       os << "HIP_R_16F";
   };
@@ -683,41 +766,6 @@ inline std::ostream& operator<<(std::ostream& os, const hipResourceDesc* s) {
   return os;
 }
 
-
-inline std::ostream& operator<<(std::ostream& os, const hipArray& s) {
-  os << '{'
-  << s.data
-  << ','
-  << s.desc
-  << ','
-  << s.type
-  << ','
-  << s.width
-  << ','
-  << s.height
-  << ','
-  << s.depth
-  << ','
-  << s.Format
-  << ','
-  << s.NumChannels
-  << ','
-  << s.isDrv
-  << ','
-  << s.textureType
-  << '}';
-  return os;
-}
-
-inline std::ostream& operator<<(std::ostream& os, const hipArray* s) {
-  if (s) {
-    os << *s;
-  } else {
-    os << "nullptr";
-  }
-  return os;
-}
-
 inline std::ostream& operator<<(std::ostream& os, const textureReference& s) {
   os << '{'
   << s.normalized
