@@ -24,7 +24,7 @@
 #include "vdi_common.hpp"
 #include "hip_prof_api.h"
 #include "trace_helper.h"
-#include "utils/debug.hpp"
+#include "rocclr/utils/debug.hpp"
 #include "hip_formatting.hpp"
 #include "hip_graph_capture.hpp"
 
@@ -51,7 +51,44 @@
 /*! IHIP IPC MEMORY Structure */
 #define IHIP_IPC_MEM_HANDLE_SIZE   32
 #define IHIP_IPC_MEM_RESERVED_SIZE LP64_SWITCH(20,12)
+namespace hip{
+  extern std::once_flag g_ihipInitialized;
+}
+typedef struct hipArray {
+    void* data;  // FIXME: generalize this
+    struct hipChannelFormatDesc desc;
+    unsigned int type;
+    unsigned int width;
+    unsigned int height;
+    unsigned int depth;
+    enum hipArray_Format Format;
+    unsigned int NumChannels;
+    bool isDrv;
+    unsigned int textureType;
+    unsigned int flags;
+}hipArray;
 
+namespace hip {
+enum MemcpyType {
+  /// Memcpy from host to host
+  hipHostToHost,
+  /// Memcpy from host to device
+  hipWriteBuffer,
+  /// Memcpy from device to host
+  hipReadBuffer,
+  /// Memcpy from device A to device A
+  /// Memcpy from pinned host buffer to device/device to pinned host buffer
+  hipCopyBuffer,
+  /// Memcpy from device A to device A, user forced to SDMA
+  hipCopyBufferSDMA,
+  /// Memcpy from device A to device B
+  hipCopyBufferP2P,
+};
+struct Graph;
+struct GraphNode;
+struct GraphExec;
+struct UserObject;
+class Stream;
 typedef struct ihipIpcMemHandle_st {
   char ipc_handle[IHIP_IPC_MEM_HANDLE_SIZE];  ///< ipc memory handle on ROCr
   size_t psize;
@@ -70,77 +107,93 @@ typedef struct ihipIpcEventHandle_st {
 }ihipIpcEventHandle_t;
 
 const char* ihipGetErrorName(hipError_t hip_error);
+}
 
-extern std::once_flag g_ihipInitialized;
 #define HIP_INIT(noReturn)                                                                         \
   {                                                                                                \
     bool status = true;                                                                            \
-    std::call_once(g_ihipInitialized, hip::init, &status);                                         \
-    if (!status  && !noReturn) {                                                                   \
+    std::call_once(hip::g_ihipInitialized, hip::init, &status);                                    \
+    if (!status && !noReturn) {                                                                    \
       HIP_RETURN(hipErrorInvalidDevice);                                                           \
     }                                                                                              \
-    if (hip::tls.device_ == nullptr && g_devices.size() > 0) {                                     \
-      hip::tls.device_ = g_devices[0];                                                             \
-      amd::Os::setPreferredNumaNode(g_devices[0]->devices()[0]->getPreferredNumaNode());           \
+    if (hip::tls.device_ == nullptr && hip::g_devices.size() > 0) {                                \
+      hip::tls.device_ = hip::g_devices[0];                                                        \
+      amd::Os::setPreferredNumaNode(hip::g_devices[0]->devices()[0]->getPreferredNumaNode());      \
     }                                                                                              \
   }
 
 #define HIP_INIT_VOID()                                                                            \
   {                                                                                                \
     bool status = true;                                                                            \
-    std::call_once(g_ihipInitialized, hip::init, &status);                                         \
-    if (hip::tls.device_ == nullptr && g_devices.size() > 0) {                                     \
-      hip::tls.device_ = g_devices[0];                                                             \
-      amd::Os::setPreferredNumaNode(g_devices[0]->devices()[0]->getPreferredNumaNode());           \
+    std::call_once(hip::g_ihipInitialized, hip::init, &status);                                    \
+    if (hip::tls.device_ == nullptr && hip::g_devices.size() > 0) {                                \
+      hip::tls.device_ = hip::g_devices[0];                                                        \
+      amd::Os::setPreferredNumaNode(hip::g_devices[0]->devices()[0]->getPreferredNumaNode());      \
     }                                                                                              \
   }
 
 
 #define HIP_API_PRINT(...)                                          \
-  uint64_t startTimeUs=0;                                           \
+  uint64_t startTimeUs = 0;                                         \
   HIPPrintDuration(amd::LOG_INFO, amd::LOG_API, &startTimeUs,       \
                   "%s %s ( %s ) %s", KGRN,                          \
                   __func__, ToString( __VA_ARGS__ ).c_str(), KNRM);
 
 #define HIP_ERROR_PRINT(err, ...)                                                  \
   ClPrint(amd::LOG_INFO, amd::LOG_API, "%s: Returned %s : %s",                     \
-          __func__, ihipGetErrorName(err), ToString( __VA_ARGS__ ).c_str());
+          __func__, hip::ihipGetErrorName(err), ToString( __VA_ARGS__ ).c_str());
 
-#define HIP_INIT_API_INTERNAL(noReturn, cid, ...)            \
-  amd::Thread* thread = amd::Thread::current();              \
-  if (!VDI_CHECK_THREAD(thread)) {                           \
-    ClPrint(amd::LOG_NONE, amd::LOG_ALWAYS, "An internal error has occurred."   \
-      " This may be due to insufficient memory.");                              \
-    if (!noReturn) {                                         \
-      return hipErrorOutOfMemory;                            \
-    }                                                        \
-  }                                                          \
-  HIP_INIT(noReturn)                                         \
-  HIP_API_PRINT(__VA_ARGS__)                                 \
+#define HIP_INIT_API_INTERNAL(noReturn, cid, ...)                                                  \
+  amd::Thread* thread = amd::Thread::current();                                                    \
+  if (!VDI_CHECK_THREAD(thread)) {                                                                 \
+    ClPrint(amd::LOG_NONE, amd::LOG_ALWAYS,                                                        \
+            "An internal error has occurred."                                                      \
+            " This may be due to insufficient memory.");                                           \
+    if (!noReturn) {                                                                               \
+      return hipErrorOutOfMemory;                                                                  \
+    }                                                                                              \
+  }                                                                                                \
+  HIP_INIT(noReturn)                                                                               \
+  HIP_API_PRINT(__VA_ARGS__)                                                                       \
   HIP_CB_SPAWNER_OBJECT(cid);
 
 // This macro should be called at the beginning of every HIP API.
 #define HIP_INIT_API(cid, ...)                                                                     \
   HIP_INIT_API_INTERNAL(0, cid, __VA_ARGS__)                                                       \
-  if (g_devices.size() == 0) {                                                                     \
+  if (hip::g_devices.size() == 0) {                                                                \
     HIP_RETURN(hipErrorNoDevice);                                                                  \
   }
 
-#define HIP_INIT_API_NO_RETURN(cid, ...)                     \
+#define HIP_INIT_API_NO_RETURN(cid, ...)                                                           \
   HIP_INIT_API_INTERNAL(1, cid, __VA_ARGS__)
 
-#define HIP_RETURN_DURATION(ret, ...)                        \
-  hip::tls.last_error_ = ret;                                \
-  HIPPrintDuration(amd::LOG_INFO, amd::LOG_API, &startTimeUs,                      \
-                   "%s: Returned %s : %s",                                         \
-                   __func__, ihipGetErrorName(hip::tls.last_error_),               \
-                   ToString( __VA_ARGS__ ).c_str());                               \
-  return hip::tls.last_error_;
+#define HIP_RETURN_DURATION(ret, ...)                                                              \
+  hip::tls.last_command_error_ = ret;                                                              \
+  if (DEBUG_HIP_7_PREVIEW & amd::CHANGE_HIP_GET_LAST_ERROR) {                                      \
+    if (hip::tls.last_command_error_ != hipSuccess &&                                              \
+           hip::tls.last_command_error_ != hipErrorNotReady) {                                     \
+      hip::tls.last_error_ = hip::tls.last_command_error_;                                         \
+    }                                                                                              \
+  } else {                                                                                         \
+    hip::tls.last_error_ = hip::tls.last_command_error_;                                           \
+  }                                                                                                \
+  HIPPrintDuration(amd::LOG_INFO, amd::LOG_API, &startTimeUs, "%s: Returned %s : %s", __func__,    \
+                   hip::ihipGetErrorName(hip::tls.last_command_error_),                            \
+                   ToString(__VA_ARGS__).c_str());                                                 \
+  return hip::tls.last_command_error_;
 
-#define HIP_RETURN(ret, ...)                      \
-  hip::tls.last_error_ = ret;                         \
-  HIP_ERROR_PRINT(hip::tls.last_error_, __VA_ARGS__)  \
-  return hip::tls.last_error_;
+#define HIP_RETURN(ret, ...)                                                                       \
+  hip::tls.last_command_error_ = ret;                                                              \
+  if (DEBUG_HIP_7_PREVIEW & amd::CHANGE_HIP_GET_LAST_ERROR) {                                      \
+    if (hip::tls.last_command_error_ != hipSuccess &&                                              \
+           hip::tls.last_command_error_ != hipErrorNotReady) {                                     \
+      hip::tls.last_error_ = hip::tls.last_command_error_;                                         \
+    }                                                                                              \
+  } else {                                                                                         \
+    hip::tls.last_error_ = hip::tls.last_command_error_;                                           \
+  }                                                                                                \
+  HIP_ERROR_PRINT(hip::tls.last_command_error_, __VA_ARGS__)                                       \
+  return hip::tls.last_command_error_;
 
 #define HIP_RETURN_ONFAIL(func)          \
   do {                                   \
@@ -160,37 +213,66 @@ extern std::once_flag g_ihipInitialized;
     }                                    \
   } while (0);
 
+// During stream capture some actions, such as a call to hipMalloc, may be unsafe and prohibited
+// during capture. It is allowed only in relaxed mode.
 #define CHECK_STREAM_CAPTURE_SUPPORTED()                                                           \
   if (hip::tls.stream_capture_mode_ == hipStreamCaptureModeThreadLocal) {                          \
     if (hip::tls.capture_streams_.size() != 0) {                                                   \
+      for (auto stream : hip::tls.capture_streams_) {                                              \
+        stream->SetCaptureStatus(hipStreamCaptureStatusInvalidated);                               \
+      }                                                                                            \
       HIP_RETURN(hipErrorStreamCaptureUnsupported);                                                \
     }                                                                                              \
   } else if (hip::tls.stream_capture_mode_ == hipStreamCaptureModeGlobal) {                        \
     if (hip::tls.capture_streams_.size() != 0) {                                                   \
+      for (auto stream : hip::tls.capture_streams_) {                                              \
+        stream->SetCaptureStatus(hipStreamCaptureStatusInvalidated);                               \
+      }                                                                                            \
       HIP_RETURN(hipErrorStreamCaptureUnsupported);                                                \
     }                                                                                              \
     if (g_captureStreams.size() != 0) {                                                            \
+      for (auto stream : g_captureStreams) {                                                       \
+        stream->SetCaptureStatus(hipStreamCaptureStatusInvalidated);                               \
+      }                                                                                            \
       HIP_RETURN(hipErrorStreamCaptureUnsupported);                                                \
     }                                                                                              \
   }
 
-// Sync APIs cannot be called when stream capture is active
+// Device sync is not supported during capture
+#define CHECK_SUPPORTED_DURING_CAPTURE()                                                           \
+  if (!g_allCapturingStreams.empty()) {                                                            \
+    for (auto stream : g_allCapturingStreams) {                                                    \
+      stream->SetCaptureStatus(hipStreamCaptureStatusInvalidated);                                 \
+    }                                                                                              \
+    return hipErrorStreamCaptureUnsupported;                                                       \
+  }
+
+// Sync APIs like hipMemset, hipMemcpy etc.. cannot be called when stream capture is active
+// for all capture modes hipStreamCaptureModeGlobal, hipStreamCaptureModeThreadLocal and
+// hipStreamCaptureModeRelaxed
 #define CHECK_STREAM_CAPTURING()                                                                   \
-  if (!g_captureStreams.empty()) {                                                                 \
+  if (!g_allCapturingStreams.empty()) {                                                            \
+    for (auto stream : g_allCapturingStreams) {                                                    \
+      stream->SetCaptureStatus(hipStreamCaptureStatusInvalidated);                                 \
+    }                                                                                              \
     return hipErrorStreamCaptureImplicit;                                                          \
   }
 
 #define STREAM_CAPTURE(name, stream, ...)                                                          \
-  getStreamPerThread(stream);                                                                      \
-  if (stream != nullptr &&                                                                         \
+  hip::getStreamPerThread(stream);                                                                 \
+  if (stream != nullptr && stream != hipStreamLegacy &&                                            \
       reinterpret_cast<hip::Stream*>(stream)->GetCaptureStatus() ==                                \
           hipStreamCaptureStatusActive) {                                                          \
-    hipError_t status = capture##name(stream, ##__VA_ARGS__);                                      \
+    hipError_t status = hip::capture##name(stream, ##__VA_ARGS__);                                 \
     return status;                                                                                 \
+  } else if (stream != nullptr && stream != hipStreamLegacy &&                                     \
+             reinterpret_cast<hip::Stream*>(stream)->GetCaptureStatus() ==                         \
+                 hipStreamCaptureStatusInvalidated) {                                              \
+    return hipErrorStreamCaptureInvalidated;                                                       \
   }
 
 #define PER_THREAD_DEFAULT_STREAM(stream)                                                         \
-  if (stream == nullptr) {                                                                        \
+  if (stream == nullptr || stream == hipStreamLegacy) {                                           \
     stream = getPerThreadDefaultStream();                                                         \
   }
 
@@ -207,6 +289,7 @@ struct ihipExec_t {
   std::vector<char> arguments_;
 };
 
+namespace hip {
 class stream_per_thread {
 private:
   std::vector<hipStream_t> m_streams;
@@ -216,11 +299,12 @@ public:
   void operator=(const stream_per_thread& ) = delete;
   ~stream_per_thread();
   hipStream_t get();
+  void clear_spt();
 };
 
-namespace hip {
   class Device;
   class MemoryPool;
+  class Event;
   class Stream : public amd::HostQueue {
   public:
     enum Priority : int { High = -1, Normal = 0, Low = 1 };
@@ -238,7 +322,7 @@ namespace hip {
     /// Current capture status of the stream
     hipStreamCaptureStatus captureStatus_;
     /// Graph that is constructed with capture
-    hipGraph_t pCaptureGraph_;
+    hip::Graph* pCaptureGraph_;
     /// Based on mode stream capture places restrictions on API calls that can be made within or
     /// concurrently
     hipStreamCaptureMode captureMode_{hipStreamCaptureModeGlobal};
@@ -247,9 +331,9 @@ namespace hip {
     /// dependencies
     hipStream_t parentStream_ = nullptr;
     /// Last graph node captured in the stream
-    std::vector<hipGraphNode_t> lastCapturedNodes_;
+    std::vector<hip::GraphNode*> lastCapturedNodes_;
     /// dependencies removed via API hipStreamUpdateCaptureDependencies
-    std::vector<hipGraphNode_t> removedDependencies_;
+    std::vector<hip::GraphNode*> removedDependencies_;
     /// Derived streams/Paralell branches from the origin stream
     std::vector<hipStream_t> parallelCaptureStreams_;
     /// Capture events
@@ -285,16 +369,12 @@ namespace hip {
     /// Returns the CU mask for the current stream
     const std::vector<uint32_t> GetCUMask() const { return cuMask_; }
 
-    /// Sync all streams
-    static void SyncAllStreams(int deviceId);
-
     /// Check whether any blocking stream running
     static bool StreamCaptureBlocking();
 
-    /// Destroy all streams on a given device
-    static void destroyAllStreams(int deviceId);
+    static void Destroy(hip::Stream* stream, bool forceDestroy = false);
 
-    static void Destroy(hip::Stream* stream);
+    virtual bool terminate();
 
     /// Check Stream Capture status to make sure it is done
     static bool StreamCaptureOngoing(hipStream_t hStream);
@@ -307,20 +387,20 @@ namespace hip {
     bool IsOriginStream() const { return originStream_; }
     void SetOriginStream() { originStream_ = true; }
     /// Returns captured graph
-    hipGraph_t GetCaptureGraph() const { return pCaptureGraph_; }
+    hip::Graph* GetCaptureGraph() const { return pCaptureGraph_; }
     /// Returns last captured graph node
-    const std::vector<hipGraphNode_t>& GetLastCapturedNodes() const { return lastCapturedNodes_; }
+    const std::vector<hip::GraphNode*>& GetLastCapturedNodes() const { return lastCapturedNodes_; }
     /// Set last captured graph node
-    void SetLastCapturedNode(hipGraphNode_t graphNode) {
+    void SetLastCapturedNode(hip::GraphNode* graphNode) {
       lastCapturedNodes_.clear();
       lastCapturedNodes_.push_back(graphNode);
     }
     /// returns updated dependencies removed
-    const std::vector<hipGraphNode_t>& GetRemovedDependencies() {
+    const std::vector<hip::GraphNode*>& GetRemovedDependencies() {
       return removedDependencies_;
     }
     /// Append captured node via the wait event cross stream
-    void AddCrossCapturedNode(std::vector<hipGraphNode_t> graphNodes, bool replace = false) {
+    void AddCrossCapturedNode(std::vector<hip::GraphNode*> graphNodes, bool replace = false) {
       // replace dependencies as per flag hipStreamSetCaptureDependencies
       if (replace == true) {
         for (auto node : lastCapturedNodes_) {
@@ -329,14 +409,19 @@ namespace hip {
         lastCapturedNodes_.clear();
       }
       for (auto node : graphNodes) {
-        lastCapturedNodes_.push_back(node);
+        if (std::find(lastCapturedNodes_.begin(), lastCapturedNodes_.end(), node) ==
+            lastCapturedNodes_.end()) {
+          lastCapturedNodes_.push_back(node);
+        }
       }
     }
     /// Set graph that is being captured
-    void SetCaptureGraph(hipGraph_t pGraph) {
+    void SetCaptureGraph(hip::Graph* pGraph) {
       pCaptureGraph_ = pGraph;
       captureStatus_ = hipStreamCaptureStatusActive;
     }
+    /// Reset graph to nullptr when capture is invalidated, but keep the status
+    void ResetCaptureGraph() { pCaptureGraph_ = nullptr; }
     void SetCaptureId() {
       // ID is generated in Begin Capture i.e.. when capture status is active
       captureID_ = GenerateCaptureID();
@@ -392,7 +477,6 @@ namespace hip {
         parallelCaptureStreams_.erase(it);
       }
     }
-    static bool existsActiveStreamForDevice(hip::Device* device);
 
     /// The stream should be destroyed via release() rather than delete
     private:
@@ -400,8 +484,12 @@ namespace hip {
   };
 
   /// HIP Device class
-  class Device {
-    amd::Monitor lock_{"Device lock", true};
+  class Device : public amd::ReferenceCountedObject {
+    // Device lock
+    amd::Monitor lock_{true};
+    // Guards device stream set
+    std::shared_mutex streamSetLock;
+    std::unordered_set<hip::Stream*> streamSet;
     /// ROCclr context
     amd::Context* context_;
     /// Device's ID
@@ -464,13 +552,18 @@ namespace hip {
     void setFlags(unsigned int flags) { flags_ = flags; }
     void Reset();
 
-   hip::Stream* NullStream();
-   Stream* GetNullStream();
+    hip::Stream* NullStream(bool wait = true);
+    Stream* GetNullStream() const {return null_stream_;};
+
+    void SetActiveStatus() {
+      isActive_ = true;
+    }
 
     bool GetActiveStatus() {
       amd::ScopedLock lock(lock_);
+      /// Either stream is active or device is active
       if (isActive_) return true;
-      if (Stream::existsActiveStreamForDevice(this)) {
+      if (existsActiveStreamForDevice()) {
         isActive_ = true;
         return true;
       }
@@ -498,14 +591,35 @@ namespace hip {
     void RemoveMemoryPool(MemoryPool* pool);
 
     /// Free memory from the device
-    bool FreeMemory(amd::Memory* memory, Stream* stream);
+    bool FreeMemory(amd::Memory* memory, Stream* stream, Event* event = nullptr);
 
     /// Release freed memory from all pools on the current device
-    void ReleaseFreedMemory(Stream* stream);
+    void ReleaseFreedMemory();
 
     /// Removes a destroyed stream from the safe list of memory pools
     void RemoveStreamFromPools(Stream* stream);
 
+    /// Add safe streams into the memppools for reuse
+    void AddSafeStream(Stream* event_stream, Stream* wait_stream);
+
+    /// Returns true if memory pool is valid on this device
+    bool IsMemoryPoolValid(MemoryPool* pool);
+    void AddStream(Stream* stream);
+
+    void RemoveStream(Stream* stream);
+
+    bool StreamExists(Stream* stream);
+
+    void destroyAllStreams();
+
+    void SyncAllStreams( bool cpu_wait = true, bool wait_blocking_streams_only = false);
+
+    bool StreamCaptureBlocking();
+
+    bool existsActiveStreamForDevice();
+  /// Wait all active streams on the blocking queue. The method enqueues a wait command and
+  /// doesn't stall the current thread
+    void WaitActiveStreams(hip::Stream* blocking_stream, bool wait_null_stream = false);
   };
 
   /// Thread Local Storage Variables Aggregator Class
@@ -513,7 +627,7 @@ namespace hip {
   public:
     Device* device_;
     std::stack<Device*> ctxt_stack_;
-    hipError_t last_error_;
+    hipError_t last_error_, last_command_error_;
     std::vector<hip::Stream*> capture_streams_;
     hipStreamCaptureMode stream_capture_mode_;
     std::stack<ihipExec_t> exec_stack_;
@@ -521,6 +635,7 @@ namespace hip {
 
     TlsAggregator(): device_(nullptr),
       last_error_(hipSuccess),
+      last_command_error_(hipSuccess),
       stream_capture_mode_(hipStreamCaptureModeGlobal) {
     }
     ~TlsAggregator() {
@@ -540,11 +655,11 @@ namespace hip {
   /// Get ROCclr queue associated with hipStream
   /// Note: This follows the CUDA spec to sync with default streams
   ///       and Blocking streams
-  extern hip::Stream* getStream(hipStream_t stream);
+  extern hip::Stream* getStream(hipStream_t stream, bool wait = true);
   /// Get default stream associated with the ROCclr context
   extern hip::Stream* getNullStream(amd::Context&);
   /// Get default stream of the thread
-  extern hip::Stream* getNullStream();
+  extern hip::Stream* getNullStream(bool wait = true);
   /// Get device ID associated with the ROCclr context
   int getDeviceID(amd::Context& ctx);
   /// Check if stream is valid
@@ -552,40 +667,38 @@ namespace hip {
   extern bool isValid(hipEvent_t event);
   extern amd::Monitor hipArraySetLock;
   extern std::unordered_set<hipArray*> hipArraySet;
-}; // namespace hip
 
-extern void WaitThenDecrementSignal(hipStream_t stream, hipError_t status, void* user_data);
+  extern void WaitThenDecrementSignal(hipStream_t stream, hipError_t status, void* user_data);
 
-/// Wait all active streams on the blocking queue. The method enqueues a wait command and
-/// doesn't stall the current thread
-extern void iHipWaitActiveStreams(hip::Stream* blocking_stream, bool wait_null_stream = false);
+  extern std::vector<hip::Device*> g_devices;
+  extern hipError_t ihipDeviceGetCount(int* count);
+  extern int ihipGetDevice();
 
-extern std::vector<hip::Device*> g_devices;
-extern hipError_t ihipDeviceGetCount(int* count);
-extern int ihipGetDevice();
+  extern hipError_t ihipMalloc(void** ptr, size_t sizeBytes, unsigned int flags);
+  extern hipError_t ihipHostMalloc(void** ptr, size_t sizeBytes, unsigned int flags);
+  extern amd::Memory* getMemoryObject(const void* ptr, size_t& offset, size_t size = 0);
+  extern amd::Memory* getMemoryObjectWithOffset(const void* ptr, const size_t size = 0);
+  extern void getStreamPerThread(hipStream_t& stream);
+  extern hipStream_t getPerThreadDefaultStream();
+  extern hipError_t ihipUnbindTexture(textureReference* texRef);
+  extern hipError_t ihipHostRegister(void* hostPtr, size_t sizeBytes, unsigned int flags);
+  extern hipError_t ihipHostUnregister(void* hostPtr);
+  extern hipError_t ihipGetDeviceProperties(hipDeviceProp_t* props, hipDevice_t device);
 
-extern hipError_t ihipMalloc(void** ptr, size_t sizeBytes, unsigned int flags);
-extern amd::Memory* getMemoryObject(const void* ptr, size_t& offset, size_t size = 0);
-extern amd::Memory* getMemoryObjectWithOffset(const void* ptr, const size_t size = 0);
-extern void getStreamPerThread(hipStream_t& stream);
-extern hipStream_t getPerThreadDefaultStream();
-extern hipError_t ihipUnbindTexture(textureReference* texRef);
-extern hipError_t ihipHostRegister(void* hostPtr, size_t sizeBytes, unsigned int flags);
-extern hipError_t ihipHostUnregister(void* hostPtr);
-extern hipError_t ihipGetDeviceProperties(hipDeviceProp_t* props, hipDevice_t device);
+  extern hipError_t ihipDeviceGet(hipDevice_t* device, int deviceId);
+  extern hipError_t ihipStreamOperation(hipStream_t stream, cl_command_type cmdType, void* ptr,
+                                        uint64_t value, uint64_t mask, unsigned int flags,
+                                        size_t sizeBytes);
+  hipError_t ihipMemcpy(void* dst, const void* src, size_t sizeBytes, hipMemcpyKind kind,
+                        hip::Stream& stream, bool isHostAsync = false, bool isGPUAsync = true);
+  constexpr bool kOptionChangeable = true;
+  constexpr bool kNewDevProg = false;
 
-extern hipError_t ihipDeviceGet(hipDevice_t* device, int deviceId);
-extern hipError_t ihipStreamOperation(hipStream_t stream, cl_command_type cmdType, void* ptr,
-                                      uint64_t value, uint64_t mask, unsigned int flags, size_t sizeBytes);
-hipError_t ihipMemcpy(void* dst, const void* src, size_t sizeBytes, hipMemcpyKind kind,
-                      hip::Stream& stream, bool isHostAsync = false, bool isGPUAsync = true);
-constexpr bool kOptionChangeable = true;
-constexpr bool kNewDevProg = false;
+  constexpr bool kMarkerDisableFlush = true;  //!< Avoids command batch flush in ROCclr
 
-constexpr bool kMarkerDisableFlush = true;   //!< Avoids command batch flush in ROCclr
-
-extern std::vector<hip::Stream*> g_captureStreams;
-extern amd::Monitor g_captureStreamsLock;
-extern amd::Monitor g_streamSetLock;
-extern std::unordered_set<hip::Stream*> g_allCapturingStreams;
-#endif // HIP_SRC_HIP_INTERNAL_H
+  extern std::vector<hip::Stream*> g_captureStreams;
+  extern amd::Monitor g_captureStreamsLock;
+  extern amd::Monitor g_streamSetLock;
+  extern std::unordered_set<hip::Stream*> g_allCapturingStreams;
+} // namespace hip
+#endif  // HIP_SRC_HIP_INTERNAL_H
