@@ -1,4 +1,4 @@
-/* Copyright (c) 2015 - 2021 Advanced Micro Devices, Inc.
+/* Copyright (c) 2015 - 2024 Advanced Micro Devices, Inc.
 
  Permission is hereby granted, free of charge, to any person obtaining a copy
  of this software and associated documentation files (the "Software"), to deal
@@ -26,6 +26,9 @@
 #include "platform/program.hpp"
 #include "hip_event.hpp"
 #include "hip_platform.hpp"
+#include "hip_comgr_helper.hpp"
+
+namespace hip {
 
 hipError_t ihipModuleLoadData(hipModule_t* module, const void* mmap_ptr, size_t mmap_size);
 
@@ -80,7 +83,7 @@ hipError_t hipModuleGetFunction(hipFunction_t* hfunc, hipModule_t hmod, const ch
   }
 
   if (hipSuccess != PlatformState::instance().getDynFunc(hfunc, hmod, name)) {
-    LogPrintfError("Cannot find the function: %s for module: 0x%x \n", name, hmod);
+    LogPrintfError("Cannot find the function: %s for module: 0x%x", name, hmod);
     HIP_RETURN(hipErrorNotFound);
   }
 
@@ -91,10 +94,6 @@ hipError_t hipModuleGetGlobal(hipDeviceptr_t* dptr, size_t* bytes, hipModule_t h
                               const char* name) {
   HIP_INIT_API(hipModuleGetGlobal, dptr, bytes, hmod, name);
 
-  if (dptr == nullptr || bytes == nullptr) {
-    // If either is nullptr, ignore it
-    HIP_RETURN(hipSuccess);
-  }
   if ((dptr == nullptr && bytes == nullptr) || name == nullptr || strlen(name) == 0) {
     HIP_RETURN(hipErrorInvalidValue);
   }
@@ -103,7 +102,7 @@ hipError_t hipModuleGetGlobal(hipDeviceptr_t* dptr, size_t* bytes, hipModule_t h
   }
   /* Get address and size for the global symbol */
   if (hipSuccess != PlatformState::instance().getDynGlobalVar(name, hmod, dptr, bytes)) {
-    LogPrintfError("Cannot find global Var: %s for module: 0x%x at device: %d \n", name, hmod,
+    LogPrintfError("Cannot find global Var: %s for module: 0x%x at device: %d", name, hmod,
                    ihipGetDevice());
     HIP_RETURN(hipErrorNotFound);
   }
@@ -114,7 +113,7 @@ hipError_t hipModuleGetGlobal(hipDeviceptr_t* dptr, size_t* bytes, hipModule_t h
 hipError_t hipFuncGetAttribute(int* value, hipFunction_attribute attrib, hipFunction_t hfunc) {
   HIP_INIT_API(hipFuncGetAttribute, value, attrib, hfunc);
 
-  if ((value == nullptr)) {
+  if (value == nullptr) {
     HIP_RETURN(hipErrorInvalidValue);
   }
 
@@ -142,7 +141,7 @@ hipError_t hipFuncGetAttribute(int* value, hipFunction_attribute attrib, hipFunc
       *value = static_cast<int>(wrkGrpInfo->size_);
       break;
     case HIP_FUNC_ATTRIBUTE_CONST_SIZE_BYTES:
-      *value = 0;
+      *value = static_cast<int>(wrkGrpInfo->constMemSize_ - 1);
       break;
     case HIP_FUNC_ATTRIBUTE_LOCAL_SIZE_BYTES:
       *value = static_cast<int>(wrkGrpInfo->privateMemSize_);
@@ -151,10 +150,9 @@ hipError_t hipFuncGetAttribute(int* value, hipFunction_attribute attrib, hipFunc
       *value = static_cast<int>(wrkGrpInfo->usedVGPRs_);
       break;
     case HIP_FUNC_ATTRIBUTE_PTX_VERSION:
-      *value = 30;  // Defaults to 3.0 as HCC
-      break;
     case HIP_FUNC_ATTRIBUTE_BINARY_VERSION:
-      *value = static_cast<int>(kernel->signature().version());
+      *value = hip::getCurrentDevice()->devices()[0]->isa().versionMajor() * 10 +
+               hip::getCurrentDevice()->devices()[0]->isa().versionMinor();
       break;
     case HIP_FUNC_ATTRIBUTE_CACHE_MODE_CA:
       *value = 0;
@@ -183,7 +181,49 @@ hipError_t hipFuncGetAttributes(hipFuncAttributes* attr, const void* func) {
 hipError_t hipFuncSetAttribute(const void* func, hipFuncAttribute attr, int value) {
   HIP_INIT_API(hipFuncSetAttribute, func, attr, value);
 
-  // No way to set function attribute yet.
+  if (func == nullptr)  {
+    HIP_RETURN(hipErrorInvalidDeviceFunction);
+  }
+  if (attr < 0 || attr > hipFuncAttributeMax) {
+    HIP_RETURN(hipErrorInvalidValue);
+  }
+
+  hipFunction_t h_func = nullptr;
+  const hip::DeviceFunc* function = nullptr;
+
+  hipError_t err = PlatformState::instance().getStatFunc(&h_func, func, ihipGetDevice());
+  if (h_func == nullptr) {
+    if (PlatformState::instance().isValidDynFunc((func))) {
+      function = reinterpret_cast<const hip::DeviceFunc*>(func);
+    } else {
+      HIP_RETURN(hipErrorInvalidDeviceFunction);
+    }
+  } else {
+    function = reinterpret_cast<const hip::DeviceFunc*>(h_func);
+  }
+
+  amd::Kernel* kernel = function->kernel();
+
+  if (kernel == nullptr) {
+    HIP_RETURN(hipErrorInvalidDeviceFunction);
+  }
+  device::Kernel* d_kernel =
+                 (device::Kernel*)(kernel->getDeviceKernel(
+                  *(hip::getCurrentDevice()->devices()[0])));
+
+  if (attr == hipFuncAttributeMaxDynamicSharedMemorySize) {
+    if ((value < 0) || (value > (d_kernel->workGroupInfo()->availableLDSSize_ -
+                                 d_kernel->workGroupInfo()->localMemSize_))) {
+      HIP_RETURN(hipErrorInvalidValue);
+    }
+    d_kernel->workGroupInfo()->maxDynamicSharedSizeBytes_ = value;
+  }
+
+  if (attr == hipFuncAttributePreferredSharedMemoryCarveout) {
+    if (value < -1 || value > 100)  {
+      HIP_RETURN(hipErrorInvalidValue);
+    }
+  }
 
   HIP_RETURN(hipSuccess);
 }
@@ -191,7 +231,12 @@ hipError_t hipFuncSetAttribute(const void* func, hipFuncAttribute attr, int valu
 hipError_t hipFuncSetCacheConfig(const void* func, hipFuncCache_t cacheConfig) {
   HIP_INIT_API(hipFuncSetCacheConfig, cacheConfig);
 
-  // No way to set cache config yet.
+  if (func == nullptr) { HIP_RETURN(hipErrorInvalidDeviceFunction); }
+  if (cacheConfig != hipFuncCachePreferNone && cacheConfig != hipFuncCachePreferShared &&
+      cacheConfig != hipFuncCachePreferL1 && cacheConfig != hipFuncCachePreferEqual) {
+    HIP_RETURN(hipErrorInvalidValue);
+  }
+  // No way to set cache config yet
 
   HIP_RETURN(hipSuccess);
 }
@@ -199,7 +244,13 @@ hipError_t hipFuncSetCacheConfig(const void* func, hipFuncCache_t cacheConfig) {
 hipError_t hipFuncSetSharedMemConfig(const void* func, hipSharedMemConfig config) {
   HIP_INIT_API(hipFuncSetSharedMemConfig, func, config);
 
-  // No way to set Shared Memory config function yet.
+  if (func == nullptr) { HIP_RETURN(hipErrorInvalidDeviceFunction); }
+  if (config != hipSharedMemBankSizeDefault && config != hipSharedMemBankSizeFourByte &&
+      config != hipSharedMemBankSizeEightByte) {
+    HIP_RETURN(hipErrorInvalidValue);
+  }
+
+  // No way to set shared memory config yet
 
   HIP_RETURN(hipSuccess);
 }
@@ -218,11 +269,15 @@ hipError_t ihipLaunchKernel_validate(hipFunction_t f, uint32_t globalWorkSizeX,
                    "Both, kernelParams and extra Params are provided, only one should be provided");
     return hipErrorInvalidValue;
   }
-  if (globalWorkSizeX == 0 || globalWorkSizeY == 0 || globalWorkSizeZ == 0 || blockDimX == 0 ||
-      blockDimY == 0 || blockDimZ == 0) {
+
+  if (globalWorkSizeX == 0 || globalWorkSizeY == 0 || globalWorkSizeZ == 0) {
     return hipErrorInvalidValue;
   }
 
+  if (blockDimX == 0 || blockDimY == 0 || blockDimZ == 0) {
+    return hipErrorInvalidConfiguration;
+  }
+
   const amd::Device* device = g_devices[deviceId]->devices()[0];
   const auto& info = device->info();
   if (sharedMemBytes > info.localMemSizePerCU_) { //sharedMemPerBlock
@@ -234,6 +289,11 @@ hipError_t ihipLaunchKernel_validate(hipFunction_t f, uint32_t globalWorkSizeX,
   }
   hip::DeviceFunc* function = hip::DeviceFunc::asFunction(f);
   amd::Kernel* kernel = function->kernel();
+  const amd::KernelSignature& signature = kernel->signature();
+  if ((signature.numParameters() > 0) && (kernelParams == nullptr) && (extra == nullptr)) {
+    LogPrintfError("%s","At least one of kernelParams or extra Params should be provided");
+    return hipErrorInvalidValue;
+  }
   if (!kernel->getDeviceKernel(*device)) {
     return hipErrorInvalidDevice;
   }
@@ -272,32 +332,6 @@ hipError_t ihipLaunchKernel_validate(hipFunction_t f, uint32_t globalWorkSizeX,
       return hipErrorLaunchFailure;
     }
   }
-  address kernargs = nullptr;
-  // 'extra' is a struct that contains the following info: {
-  //   HIP_LAUNCH_PARAM_BUFFER_POINTER, kernargs,
-  //   HIP_LAUNCH_PARAM_BUFFER_SIZE, &kernargs_size,
-  //   HIP_LAUNCH_PARAM_END }
-  if (extra != nullptr) {
-    if (extra[0] != HIP_LAUNCH_PARAM_BUFFER_POINTER || extra[2] != HIP_LAUNCH_PARAM_BUFFER_SIZE ||
-        extra[4] != HIP_LAUNCH_PARAM_END) {
-      return hipErrorInvalidValue;
-    }
-    kernargs = reinterpret_cast<address>(extra[1]);
-  }
-
-  const amd::KernelSignature& signature = kernel->signature();
-  for (size_t i = 0; i < signature.numParameters(); ++i) {
-    const amd::KernelParameterDescriptor& desc = signature.at(i);
-    if (kernelParams == nullptr) {
-      assert(kernargs != nullptr);
-      kernel->parameters().set(i, desc.size_, kernargs + desc.offset_,
-                               desc.type_ == T_POINTER /*svmBound*/);
-    } else {
-      assert(extra == nullptr);
-      kernel->parameters().set(i, desc.size_, kernelParams[i],
-                               desc.type_ == T_POINTER /*svmBound*/);
-    }
-  }
   return hipSuccess;
 }
 
@@ -318,7 +352,6 @@ hipError_t ihipLaunchKernelCommand(amd::Command*& command, hipFunction_t f,
   size_t localWorkSize[3] = {blockDimX, blockDimY, blockDimZ};
   amd::NDRangeContainer ndrange(3, globalWorkOffset, globalWorkSize, localWorkSize);
   amd::Command::EventWaitList waitList;
-  address kernargs = nullptr;
 
   bool profileNDRange = (startEvent != nullptr || stopEvent != nullptr);
 
@@ -334,10 +367,44 @@ hipError_t ihipLaunchKernelCommand(amd::Command*& command, hipFunction_t f,
     return hipErrorOutOfMemory;
   }
 
-  // Capture the kernel arguments
-  if (CL_SUCCESS != kernelCommand->captureAndValidate()) {
-    kernelCommand->release();
-    return hipErrorOutOfMemory;
+  address kernargs = nullptr;
+  // 'extra' is a struct that contains the following info: {
+  //   HIP_LAUNCH_PARAM_BUFFER_POINTER, kernargs,
+  //   HIP_LAUNCH_PARAM_BUFFER_SIZE, &kernargs_size,
+  //   HIP_LAUNCH_PARAM_END }
+  if (extra != nullptr) {
+    assert(kernelParams == nullptr);
+    if (extra[0] != HIP_LAUNCH_PARAM_BUFFER_POINTER || extra[2] != HIP_LAUNCH_PARAM_BUFFER_SIZE ||
+        extra[4] != HIP_LAUNCH_PARAM_END) {
+      return hipErrorInvalidValue;
+    }
+    kernargs = reinterpret_cast<address>(extra[1]);
+  }
+
+  if (DEBUG_HIP_KERNARG_COPY_OPT) {
+    if (CL_SUCCESS != kernelCommand->AllocCaptureSetValidate(kernelParams, kernargs)) {
+      kernelCommand->release();
+      return hipErrorOutOfMemory;
+    }
+
+  } else {
+    for (size_t i = 0; i < kernel->signature().numParameters(); ++i) {
+      const amd::KernelParameterDescriptor& desc = kernel->signature().at(i);
+      if (kernelParams == nullptr) {
+        assert(kernargs != nullptr);
+        kernel->parameters().set(i, desc.size_, kernargs + desc.offset_,
+                                 desc.type_ == T_POINTER /*svmBound*/);
+      } else {
+        kernel->parameters().set(i, desc.size_, kernelParams[i],
+                                 desc.type_ == T_POINTER /*svmBound*/);
+      }
+    }
+
+    // Capture the kernel arguments
+    if (CL_SUCCESS != kernelCommand->captureAndValidate()) {
+      kernelCommand->release();
+      return hipErrorOutOfMemory;
+    }
   }
 
   command = kernelCommand;
@@ -362,7 +429,7 @@ hipError_t ihipModuleLaunchKernel(hipFunction_t f, uint32_t globalWorkSizeX,
   }
   hip::DeviceFunc* function = hip::DeviceFunc::asFunction(f);
   amd::Kernel* kernel = function->kernel();
-  amd::ScopedLock lock(function->dflock_);
+  amd::ScopedLock lock (DEBUG_HIP_KERNARG_COPY_OPT ? nullptr : &function->dflock_);
 
   hipError_t status = ihipLaunchKernel_validate(
       f, globalWorkSizeX, globalWorkSizeY, globalWorkSizeZ, blockDimX, blockDimY, blockDimZ,
@@ -404,7 +471,7 @@ hipError_t ihipModuleLaunchKernel(hipFunction_t f, uint32_t globalWorkSizeX,
 
   if (stopEvent != nullptr) {
     hip::Event* eStop = reinterpret_cast<hip::Event*>(stopEvent);
-    if (eStop->flags & hipEventDisableSystemFence) {
+    if (eStop->flags_ & hipEventDisableSystemFence) {
       command->setEventScope(amd::Device::kCacheStateIgnore);
     } else {
       command->setEventScope(amd::Device::kCacheStateSystem);
@@ -439,9 +506,10 @@ hipError_t hipModuleLaunchKernel(hipFunction_t f, uint32_t gridDimX, uint32_t gr
 
   STREAM_CAPTURE(hipModuleLaunchKernel, hStream, f, gridDimX, gridDimY, gridDimZ, blockDimX,
                  blockDimY, blockDimZ, sharedMemBytes, kernelParams, extra);
-  if (gridDimX > std::numeric_limits<int32_t>::max() ||
-      gridDimY > std::numeric_limits<int32_t>::max() ||
-      gridDimZ > std::numeric_limits<int32_t>::max()) {
+
+  constexpr auto int32_max = static_cast<uint64_t>(std::numeric_limits<int32_t>::max());
+  constexpr auto uint16_max = static_cast<uint64_t>(std::numeric_limits<uint16_t>::max()) + 1;
+  if (gridDimX > int32_max || gridDimY > uint16_max || gridDimZ > uint16_max) {
     HIP_RETURN(hipErrorInvalidValue);
   }
   size_t globalWorkSizeX = static_cast<size_t>(gridDimX) * blockDimX;
@@ -494,20 +562,6 @@ hipError_t hipHccModuleLaunchKernel(hipFunction_t f, uint32_t globalWorkSizeX,
                                     extra, startEvent, stopEvent));
 }
 
-hipError_t hipModuleLaunchKernelExt(hipFunction_t f, uint32_t globalWorkSizeX,
-                                    uint32_t globalWorkSizeY, uint32_t globalWorkSizeZ,
-                                    uint32_t blockDimX, uint32_t blockDimY, uint32_t blockDimZ,
-                                    size_t sharedMemBytes, hipStream_t hStream, void** kernelParams,
-                                    void** extra, hipEvent_t startEvent, hipEvent_t stopEvent) {
-  HIP_INIT_API(hipModuleLaunchKernelExt, f, globalWorkSizeX, globalWorkSizeY, globalWorkSizeZ,
-               blockDimX, blockDimY, blockDimZ, sharedMemBytes, hStream, kernelParams, extra,
-               startEvent, stopEvent);
-
-  HIP_RETURN(ihipModuleLaunchKernel(f, globalWorkSizeX, globalWorkSizeY, globalWorkSizeZ, blockDimX,
-                                    blockDimY, blockDimZ, sharedMemBytes, hStream, kernelParams,
-                                    extra, startEvent, stopEvent));
-}
-
 hipError_t hipModuleLaunchCooperativeKernel(hipFunction_t f, unsigned int gridDimX,
                                             unsigned int gridDimY, unsigned int gridDimZ,
                                             unsigned int blockDimX, unsigned int blockDimY,
@@ -665,7 +719,19 @@ hipError_t hipModuleLaunchCooperativeKernelMultiDevice(hipFunctionLaunchParams*
 
 }
 
-extern "C" hipError_t hipLaunchKernel_common(const void* hostFunction, dim3 gridDim, dim3 blockDim,
+hipError_t hipGetFuncBySymbol(hipFunction_t* functionPtr, const void* symbolPtr) {
+  HIP_INIT_API(hipGetFuncBySymbol, functionPtr, symbolPtr);
+
+  hipError_t hip_error = PlatformState::instance().getStatFunc(functionPtr,
+                         symbolPtr, ihipGetDevice());
+
+  if ((hip_error != hipSuccess) || (functionPtr == nullptr)) {
+    HIP_RETURN(hipErrorInvalidDeviceFunction);
+  }
+  HIP_RETURN(hipSuccess);
+}
+
+hipError_t hipLaunchKernel_common(const void* hostFunction, dim3 gridDim, dim3 blockDim,
                                              void** args, size_t sharedMemBytes,
                                              hipStream_t stream) {
   STREAM_CAPTURE(hipLaunchKernel, stream, hostFunction, gridDim, blockDim, args, sharedMemBytes);
@@ -673,26 +739,26 @@ extern "C" hipError_t hipLaunchKernel_common(const void* hostFunction, dim3 grid
                           nullptr, 0);
 }
 
-extern "C" hipError_t hipLaunchKernel(const void* hostFunction, dim3 gridDim, dim3 blockDim,
+hipError_t hipLaunchKernel(const void* hostFunction, dim3 gridDim, dim3 blockDim,
                                       void** args, size_t sharedMemBytes, hipStream_t stream) {
   HIP_INIT_API(hipLaunchKernel, hostFunction, gridDim, blockDim, args, sharedMemBytes, stream);
-  HIP_RETURN(hipLaunchKernel_common(hostFunction, gridDim, blockDim, args, sharedMemBytes, stream));
+  HIP_RETURN_DURATION(hipLaunchKernel_common(hostFunction, gridDim, blockDim, args, sharedMemBytes, stream));
 }
 
-extern "C" hipError_t hipLaunchKernel_spt(const void* hostFunction, dim3 gridDim, dim3 blockDim,
+hipError_t hipLaunchKernel_spt(const void* hostFunction, dim3 gridDim, dim3 blockDim,
                                           void** args, size_t sharedMemBytes, hipStream_t stream) {
   HIP_INIT_API(hipLaunchKernel, hostFunction, gridDim, blockDim, args, sharedMemBytes, stream);
   PER_THREAD_DEFAULT_STREAM(stream);
   HIP_RETURN(hipLaunchKernel_common(hostFunction, gridDim, blockDim, args, sharedMemBytes, stream));
 }
 
-extern "C" hipError_t hipExtLaunchKernel(const void* hostFunction, dim3 gridDim, dim3 blockDim,
+hipError_t hipExtLaunchKernel(const void* hostFunction, dim3 gridDim, dim3 blockDim,
                                          void** args, size_t sharedMemBytes, hipStream_t stream,
                                          hipEvent_t startEvent, hipEvent_t stopEvent, int flags) {
   HIP_INIT_API(hipExtLaunchKernel, hostFunction, gridDim, blockDim, args, sharedMemBytes,
                stream, startEvent, stopEvent, flags);
 
-  if (!hip::isValid(stream) || !hip::isValid(startEvent) || !hip::isValid(stopEvent)) {
+  if (!hip::isValid(startEvent) || !hip::isValid(stopEvent)) {
     HIP_RETURN(hipErrorInvalidValue);
   }
 
@@ -706,7 +772,14 @@ hipError_t hipLaunchCooperativeKernel_common(const void* f, dim3 gridDim, dim3 b
                                              void** kernelParams, uint32_t sharedMemBytes,
                                              hipStream_t hStream) {
   if (!hip::isValid(hStream)) {
-    return hipErrorInvalidValue;
+    return hipErrorContextIsDestroyed;
+  }
+
+  STREAM_CAPTURE(hipLaunchCooperativeKernel, hStream, f, gridDim, blockDim, kernelParams,
+                 sharedMemBytes);
+
+  if (f == nullptr) {
+    return hipErrorInvalidDeviceFunction;
   }
 
   hipFunction_t func = nullptr;
@@ -726,6 +799,10 @@ hipError_t hipLaunchCooperativeKernel_common(const void* f, dim3 gridDim, dim3 b
     return hipErrorInvalidConfiguration;
   }
 
+  if (sharedMemBytes > device->info().localMemSizePerCU_) {
+    return hipErrorCooperativeLaunchTooLarge;
+  }
+
   return ihipModuleLaunchKernel(func, static_cast<uint32_t>(globalWorkSizeX),
                                 static_cast<uint32_t>(globalWorkSizeY),
                                 static_cast<uint32_t>(globalWorkSizeZ), blockDim.x, blockDim.y,
@@ -752,7 +829,7 @@ hipError_t hipLaunchCooperativeKernel_spt(const void* f, dim3 gridDim, dim3 bloc
 
 hipError_t ihipLaunchCooperativeKernelMultiDevice(hipLaunchParams* launchParamsList, int numDevices,
                                                   unsigned int flags, uint32_t extFlags) {
-  if (launchParamsList == nullptr) {
+  if (launchParamsList == nullptr || numDevices > g_devices.size()) {
     return hipErrorInvalidValue;
   }
 
@@ -834,7 +911,7 @@ hipError_t hipModuleGetTexRef(textureReference** texRef, hipModule_t hmod, const
 
   /* Get address and size for the global symbol */
   if (hipSuccess != PlatformState::instance().getDynTexRef(name, hmod, texRef)) {
-    LogPrintfError("Cannot get texRef for name: %s at module:0x%x \n", name, hmod);
+    LogPrintfError("Cannot get texRef for name: %s at module:0x%x", name, hmod);
     HIP_RETURN(hipErrorNotFound);
   }
 
@@ -850,3 +927,135 @@ hipError_t hipModuleGetTexRef(textureReference** texRef, hipModule_t hmod, const
 
   HIP_RETURN(err);
 }
+
+
+hipError_t hipLinkAddData(hipLinkState_t hip_link_state, hipJitInputType input_type, void* image,
+                          size_t image_size, const char* name, unsigned int num_options,
+                          hipJitOption* options_ptr, void** option_values) {
+
+  HIP_INIT_API(hipLinkAddData, hip_link_state, image, image_size, name, num_options, options_ptr, option_values);
+
+  if (image == nullptr || image_size <= 0) {
+    HIP_RETURN(hipErrorInvalidImage);
+  }
+
+  if (input_type == hipJitInputCubin || input_type == hipJitInputPtx ||
+      input_type == hipJitInputFatBinary || input_type == hipJitInputObject ||
+      input_type ==  hipJitInputLibrary || input_type == hipJitInputNvvm ||
+      input_type == hipJitInputLLVMBitcode || input_type == hipJitInputLLVMBundledBitcode ||
+      input_type == hipJitInputLLVMArchivesOfBundledBitcode ) {
+    HIP_RETURN(hipErrorInvalidValue);
+  }
+
+  std::string input_name;
+  if (name) {
+    input_name = name;
+  }
+
+  LinkProgram* hip_link_prog_ptr =
+      reinterpret_cast<LinkProgram*>(hip_link_state);
+
+  if (!LinkProgram::isLinkerValid(hip_link_prog_ptr)) {
+    HIP_RETURN(hipErrorInvalidHandle);
+  }
+
+  if (!hip_link_prog_ptr->AddLinkerData(image, image_size, input_name, input_type)) {
+    HIP_RETURN(hipErrorInvalidConfiguration);
+  }
+
+  HIP_RETURN(hipSuccess);
+}
+
+hipError_t hipLinkAddFile(hipLinkState_t hip_link_state, hipJitInputType input_type, const char* file_path,
+                          unsigned int num_options, hipJitOption* options_ptr, void** option_values) {
+  HIP_INIT_API(hipLinkAddFile, hip_link_state, input_type, file_path, num_options, options_ptr, option_values);
+
+  if (hip_link_state == nullptr) {
+    HIP_RETURN(hipErrorInvalidHandle);
+  }
+
+  if (input_type == hipJitInputCubin || input_type == hipJitInputPtx ||
+      input_type == hipJitInputFatBinary || input_type == hipJitInputObject ||
+      input_type ==  hipJitInputLibrary || input_type == hipJitInputNvvm ||
+      input_type == hipJitInputLLVMBitcode || input_type == hipJitInputLLVMBundledBitcode ||
+      input_type == hipJitInputLLVMArchivesOfBundledBitcode ) {
+    HIP_RETURN(hipErrorInvalidValue);
+  }
+
+  LinkProgram* hip_link_prog_ptr =
+      reinterpret_cast<LinkProgram*>(hip_link_state);
+
+  if (!LinkProgram::isLinkerValid(hip_link_prog_ptr)) {
+    HIP_RETURN(hipErrorInvalidValue);
+  }
+
+  if (!hip_link_prog_ptr->AddLinkerFile(std::string(file_path), input_type)) {
+    HIP_RETURN(hipErrorInvalidConfiguration);
+  }
+
+  HIP_RETURN(hipSuccess);
+}
+
+hipError_t hipLinkCreate(unsigned int num_options, hipJitOption* options_ptr,
+                              void** options_vals_pptr, hipLinkState_t* hip_link_state_ptr) {
+  HIP_INIT_API(hipLinkCreate, num_options, options_ptr, options_vals_pptr, hip_link_state_ptr);
+
+  if (hip_link_state_ptr == nullptr) {
+    HIP_RETURN(hipErrorInvalidValue);
+  }
+
+  if (num_options != 0) {
+    for (int i = 0; i < num_options; i++) {
+      if (options_ptr == nullptr || options_vals_pptr == nullptr) {
+        HIP_RETURN(hipErrorInvalidValue);
+      }
+    }
+  }
+
+  std::string name("LinkerProgram");
+  LinkProgram* hip_link_prog_ptr = new LinkProgram(name);
+  if (!hip_link_prog_ptr->AddLinkerOptions(num_options, options_ptr, options_vals_pptr)) {
+    HIP_RETURN(hipErrorInvalidConfiguration);
+  }
+
+  *hip_link_state_ptr = reinterpret_cast<hipLinkState_t>(hip_link_prog_ptr);
+
+  HIP_RETURN(hipSuccess);
+}
+
+hipError_t hipLinkComplete(hipLinkState_t hip_link_state, void** bin_out, size_t* size_out) {
+  HIP_INIT_API(hipLinkComplete, hip_link_state, bin_out, size_out);
+
+  if (bin_out == nullptr || size_out == nullptr) {
+    HIP_RETURN(hipErrorInvalidValue);
+  }
+
+  LinkProgram* hip_link_prog_ptr =
+      reinterpret_cast<LinkProgram*>(hip_link_state);
+
+  if (!LinkProgram::isLinkerValid(hip_link_prog_ptr)) {
+    HIP_RETURN(hipErrorInvalidValue);
+  }
+
+  if (!hip_link_prog_ptr->LinkComplete(bin_out, size_out)) {
+    HIP_RETURN(hipErrorInvalidConfiguration);
+  }
+
+  HIP_RETURN(hipSuccess);
+}
+
+hipError_t hipLinkDestroy(hipLinkState_t hip_link_state) {
+  HIP_INIT_API(hipLinkDestroy, hip_link_state);
+
+  LinkProgram* hip_link_prog_ptr =
+      reinterpret_cast<LinkProgram*>(hip_link_state);
+
+  if (!LinkProgram::isLinkerValid(hip_link_prog_ptr)) {
+    HIP_RETURN(hipErrorInvalidValue);
+  }
+
+  delete hip_link_prog_ptr;
+  HIP_RETURN(hipSuccess);
+}
+
+}  // namespace hip
