@@ -26,7 +26,9 @@
 #include "platform/runtime.hpp"
 
 #include <unordered_map>
+#include <mutex>
 
+namespace hip {
 constexpr unsigned __hipFatMAGIC2 = 0x48495046;  // "HIPF"
 
 PlatformState* PlatformState::platform_;  // Initiaized as nullptr by default
@@ -67,17 +69,20 @@ static bool isCompatibleCodeObject(const std::string& codeobj_target_id, const c
   return codeobj_target_id == short_name;
 }
 
-extern "C" hip::FatBinaryInfo** __hipRegisterFatBinary(const void* data) {
+void** __hipRegisterFatBinary(const void* data) {
   const __CudaFatBinaryWrapper* fbwrapper = reinterpret_cast<const __CudaFatBinaryWrapper*>(data);
   if (fbwrapper->magic != __hipFatMAGIC2 || fbwrapper->version != 1) {
     LogPrintfError("Cannot Register fat binary. FatMagic: %u version: %u ", fbwrapper->magic,
                    fbwrapper->version);
     return nullptr;
   }
-  return PlatformState::instance().addFatBinary(fbwrapper->binary);
+
+  bool success{};
+  auto fat_binary_info = PlatformState::instance().addFatBinary(fbwrapper->binary, success);
+  return success ? reinterpret_cast<void**>(fat_binary_info) : nullptr;
 }
 
-extern "C" void __hipRegisterFunction(hip::FatBinaryInfo** modules, const void* hostFunction,
+void __hipRegisterFunction(hip::FatBinaryInfo** modules, const void* hostFunction,
                                       char* deviceFunction, const char* deviceName,
                                       unsigned int threadLimit, uint3* tid, uint3* bid,
                                       dim3* blockDim, dim3* gridDim, int* wSize) {
@@ -86,9 +91,14 @@ extern "C" void __hipRegisterFunction(hip::FatBinaryInfo** modules, const void*
     return var ? atoi(var) : 1;
   }()};
   hipError_t hip_error = hipSuccess;
-  hip::Function* func = new hip::Function(std::string(deviceName), modules);
-  hip_error = PlatformState::instance().registerStatFunction(hostFunction, func);
-  guarantee((hip_error == hipSuccess), "Cannot register Static function, error: %d \n", hip_error);
+  // Compiler might share same hostFunction and hence it's needless to have another
+  // hip::Function and hip::Function is stored in map with hostFunction as key.
+  // Creating hip::Function in such case, Leaks it.
+  if (PlatformState::instance().getStatFuncName(hostFunction) == nullptr) {
+    hip::Function* func = new hip::Function(std::string(deviceName), modules);
+    hip_error = PlatformState::instance().registerStatFunction(hostFunction, func);
+  }
+  guarantee((hip_error == hipSuccess), "Cannot register Static function, error: %d", hip_error);
 
   if (!enable_deferred_loading) {
     HIP_INIT_VOID();
@@ -96,7 +106,7 @@ extern "C" void __hipRegisterFunction(hip::FatBinaryInfo** modules, const void*
 
     for (size_t dev_idx = 0; dev_idx < g_devices.size(); ++dev_idx) {
       hip_error = PlatformState::instance().getStatFunc(&hfunc, hostFunction, dev_idx);
-      guarantee((hip_error == hipSuccess), "Cannot retrieve Static function, error: %d \n",
+      guarantee((hip_error == hipSuccess), "Cannot retrieve Static function, error: %d",
                                             hip_error);
     }
   }
@@ -107,7 +117,7 @@ extern "C" void __hipRegisterFunction(hip::FatBinaryInfo** modules, const void*
 // global variable in host code. The shadow host variable is used to keep
 // track of the value of the device side global variable between kernel
 // executions.
-extern "C" void __hipRegisterVar(
+void __hipRegisterVar(
     hip::FatBinaryInfo** modules,  // The device modules containing code object
     void* var,                     // The shadow variable in host code
     char* hostVar,                 // Variable name in host code
@@ -120,10 +130,10 @@ extern "C" void __hipRegisterVar(
   hip::Var* var_ptr = new hip::Var(std::string(hostVar), hip::Var::DeviceVarKind::DVK_Variable,
                                    size, 0, 0, modules);
   hipError_t err = PlatformState::instance().registerStatGlobalVar(var, var_ptr);
-  guarantee((err == hipSuccess), "Cannot register Static Global Var, error:%d \n", err);
+  guarantee((err == hipSuccess), "Cannot register Static Global Var, error:%d", err);
 }
 
-extern "C" void __hipRegisterSurface(
+void __hipRegisterSurface(
     hip::FatBinaryInfo** modules,  // The device modules containing code object
     void* var,                     // The shadow variable in host code
     char* hostVar,                 // Variable name in host code
@@ -132,10 +142,10 @@ extern "C" void __hipRegisterSurface(
   hip::Var* var_ptr = new hip::Var(std::string(hostVar), hip::Var::DeviceVarKind::DVK_Surface,
                                    sizeof(surfaceReference), 0, 0, modules);
   hipError_t err = PlatformState::instance().registerStatGlobalVar(var, var_ptr);
-  guarantee((err == hipSuccess), "Cannot register Static Glbal Var, err:%d \n", err);
+  guarantee((err == hipSuccess), "Cannot register Static Glbal Var, err:%d", err);
 }
 
-extern "C" void __hipRegisterManagedVar(
+void __hipRegisterManagedVar(
     void* hipModule,  // Pointer to hip module returned from __hipRegisterFatbinary
     void** pointer,   // Pointer to a chunk of managed memory with size \p size and alignment \p
                      // align HIP runtime allocates such managed memory and assign it to \p pointer
@@ -148,21 +158,21 @@ extern "C" void __hipRegisterManagedVar(
     hip::Stream* stream = hip::getNullStream();
     if (stream != nullptr) {
       status = ihipMemcpy(*pointer, init_value, size, hipMemcpyHostToDevice, *stream);
-      guarantee((status == hipSuccess), "Error during memcpy to managed memory, error:%d \n!", 
+      guarantee((status == hipSuccess), "Error during memcpy to managed memory, error:%d!",
                                          status);
     } else {
       ClPrint(amd::LOG_ERROR, amd::LOG_API, "Host Queue is NULL");
     }
   } else {
-    guarantee(false, "Error during allocation of managed memory!, error: %d \n", status);
+    guarantee(false, "Error during allocation of managed memory!, error: %d", status);
   }
   hip::Var* var_ptr = new hip::Var(std::string(name), hip::Var::DeviceVarKind::DVK_Managed, pointer,
                                    size, align, reinterpret_cast<hip::FatBinaryInfo**>(hipModule));
   status = PlatformState::instance().registerStatManagedVar(var_ptr);
-  guarantee((status == hipSuccess), "Cannot register Static Managed Var, error: %d \n", status);
+  guarantee((status == hipSuccess), "Cannot register Static Managed Var, error: %d", status);
 }
 
-extern "C" void __hipRegisterTexture(
+void __hipRegisterTexture(
     hip::FatBinaryInfo** modules,  // The device modules containing code object
     void* var,                     // The shadow variable in host code
     char* hostVar,                 // Variable name in host code
@@ -171,15 +181,49 @@ extern "C" void __hipRegisterTexture(
   hip::Var* var_ptr = new hip::Var(std::string(hostVar), hip::Var::DeviceVarKind::DVK_Texture,
                                    sizeof(textureReference), 0, 0, modules);
   hipError_t err = PlatformState::instance().registerStatGlobalVar(var, var_ptr);
-  guarantee((err == hipSuccess), "Cannot register Static Global Var, status: %d \n", err);
+  guarantee((err == hipSuccess), "Cannot register Static Global Var, status: %d", err);
 }
 
-extern "C" void __hipUnregisterFatBinary(hip::FatBinaryInfo** modules) {
+void __hipUnregisterFatBinary(hip::FatBinaryInfo** modules) {
+  static std::once_flag unregister_device_sync;
+  std::call_once(unregister_device_sync, [](){
+    for (auto& hipDevice : g_devices) {
+      // By synchronizing devices ensure that all HSA signal handlers
+      // complete before removeFatBinary
+      hipDevice->SyncAllStreams(true);
+    }
+  });
   hipError_t err = PlatformState::instance().removeFatBinary(modules);
-  guarantee((err == hipSuccess), "Cannot Unregister Fat Binary, error:%d \n", err);
+  guarantee((err == hipSuccess), "Cannot Unregister Fat Binary, error:%d", err);
 }
 
-extern "C" hipError_t hipConfigureCall(dim3 gridDim, dim3 blockDim, size_t sharedMem,
+void __hipRegisterFunction(void** modules, const void* hostFunction, char* deviceFunction,
+                           const char* deviceName, unsigned int threadLimit, uint3* tid, uint3* bid,
+                           dim3* blockDim, dim3* gridDim, int* wSize) {
+  return __hipRegisterFunction(reinterpret_cast<hip::FatBinaryInfo**>(modules), hostFunction,
+                               deviceFunction, deviceName, threadLimit, tid, bid, blockDim, gridDim,
+                               wSize);
+}
+void __hipRegisterSurface(void** modules, void* var, char* hostVar, char* deviceVar, int type,
+                          int ext) {
+  return __hipRegisterSurface(reinterpret_cast<hip::FatBinaryInfo**>(modules), var, hostVar,
+                              deviceVar, type, ext);
+}
+void __hipRegisterTexture(void** modules, void* var, char* hostVar, char* deviceVar, int type,
+                          int norm, int ext) {
+  return __hipRegisterTexture(reinterpret_cast<hip::FatBinaryInfo**>(modules), var, hostVar,
+                              deviceVar, type, norm, ext);
+}
+void __hipRegisterVar(void** modules, void* var, char* hostVar, char* deviceVar, int ext,
+                      size_t size, int constant, int global) {
+  return __hipRegisterVar(reinterpret_cast<hip::FatBinaryInfo**>(modules), var, hostVar,
+                          deviceVar, ext, size, constant, global);
+}
+void __hipUnregisterFatBinary(void** modules) {
+  return __hipUnregisterFatBinary(reinterpret_cast<hip::FatBinaryInfo**>(modules));
+}
+
+hipError_t hipConfigureCall(dim3 gridDim, dim3 blockDim, size_t sharedMem,
                                        hipStream_t stream) {
   HIP_INIT_API(hipConfigureCall, gridDim, blockDim, sharedMem, stream);
 
@@ -188,7 +232,7 @@ extern "C" hipError_t hipConfigureCall(dim3 gridDim, dim3 blockDim, size_t share
   HIP_RETURN(hipSuccess);
 }
 
-extern "C" hipError_t __hipPushCallConfiguration(dim3 gridDim, dim3 blockDim, size_t sharedMem,
+hipError_t __hipPushCallConfiguration(dim3 gridDim, dim3 blockDim, size_t sharedMem,
                                                  hipStream_t stream) {
   HIP_INIT_API(__hipPushCallConfiguration, gridDim, blockDim, sharedMem, stream);
 
@@ -197,7 +241,7 @@ extern "C" hipError_t __hipPushCallConfiguration(dim3 gridDim, dim3 blockDim, si
   HIP_RETURN(hipSuccess);
 }
 
-extern "C" hipError_t __hipPopCallConfiguration(dim3* gridDim, dim3* blockDim, size_t* sharedMem,
+hipError_t __hipPopCallConfiguration(dim3* gridDim, dim3* blockDim, size_t* sharedMem,
                                                 hipStream_t* stream) {
   HIP_INIT_API(__hipPopCallConfiguration, gridDim, blockDim, sharedMem, stream);
 
@@ -211,7 +255,7 @@ extern "C" hipError_t __hipPopCallConfiguration(dim3* gridDim, dim3* blockDim, s
   HIP_RETURN(hipSuccess);
 }
 
-extern "C" hipError_t hipSetupArgument(const void* arg, size_t size, size_t offset) {
+hipError_t hipSetupArgument(const void* arg, size_t size, size_t offset) {
   HIP_INIT_API(hipSetupArgument, arg, size, offset);
 
   PlatformState::instance().setupArgument(arg, size, offset);
@@ -219,7 +263,7 @@ extern "C" hipError_t hipSetupArgument(const void* arg, size_t size, size_t offs
   HIP_RETURN(hipSuccess);
 }
 
-extern "C" hipError_t hipLaunchByPtr(const void* hostFunction) {
+hipError_t hipLaunchByPtr(const void* hostFunction) {
   HIP_INIT_API(hipLaunchByPtr, hostFunction);
 
   ihipExec_t exec;
@@ -228,13 +272,13 @@ extern "C" hipError_t hipLaunchByPtr(const void* hostFunction) {
   hip::Stream* stream = reinterpret_cast<hip::Stream*>(exec.hStream_);
   int deviceId = (stream != nullptr) ? stream->DeviceId() : ihipGetDevice();
   if (deviceId == -1) {
-    LogPrintfError("Wrong DeviceId: %d \n", deviceId);
+    LogPrintfError("Wrong DeviceId: %d", deviceId);
     HIP_RETURN(hipErrorNoDevice);
   }
   hipFunction_t func = nullptr;
   hipError_t hip_error = PlatformState::instance().getStatFunc(&func, hostFunction, deviceId);
   if ((hip_error != hipSuccess) || (func == nullptr)) {
-    LogPrintfError("Could not retrieve hostFunction: 0x%x \n", hostFunction);
+    LogPrintfError("Could not retrieve hostFunction: 0x%x", hostFunction);
     HIP_RETURN(hipErrorInvalidDeviceFunction);
   }
 
@@ -242,6 +286,9 @@ extern "C" hipError_t hipLaunchByPtr(const void* hostFunction) {
   void* extra[] = {HIP_LAUNCH_PARAM_BUFFER_POINTER, &exec.arguments_[0],
                    HIP_LAUNCH_PARAM_BUFFER_SIZE, &size, HIP_LAUNCH_PARAM_END};
 
+  STREAM_CAPTURE(hipLaunchByPtr, exec.hStream_, func, exec.blockDim_, exec.gridDim_,
+                 exec.sharedMem_, extra);
+
   HIP_RETURN(hipModuleLaunchKernel(func, exec.gridDim_.x, exec.gridDim_.y, exec.gridDim_.z,
                                    exec.blockDim_.x, exec.blockDim_.y, exec.blockDim_.z,
                                    exec.sharedMem_, exec.hStream_, nullptr, extra));
@@ -283,18 +330,18 @@ hipError_t ihipCreateGlobalVarObj(const char* name, hipModule_t hmod, amd::Memor
   device::Program* dev_program = program->getDeviceProgram(*hip::getCurrentDevice()->devices()[0]);
 
   if (dev_program == nullptr) {
-    LogPrintfError("Cannot get Device Function for module: 0x%x \n", hmod);
+    LogPrintfError("Cannot get Device Function for module: 0x%x", hmod);
     HIP_RETURN(hipErrorInvalidDeviceFunction);
   }
   /* Find the global Symbols */
   if (!dev_program->createGlobalVarObj(amd_mem_obj, dptr, bytes, name)) {
-    LogPrintfError("Cannot create Global Var obj for symbol: %s \n", name);
+    LogPrintfError("Cannot create Global Var obj for symbol: %s", name);
     HIP_RETURN(hipErrorInvalidSymbol);
   }
 
   HIP_RETURN(hipSuccess);
 }
-
+}  // namespace hip
 
 namespace hip_impl {
 hipError_t ihipOccupancyMaxActiveBlocksPerMultiprocessor(
@@ -346,6 +393,12 @@ hipError_t ihipOccupancyMaxActiveBlocksPerMultiprocessor(
     VgprWaves = maxVGPRs / amd::alignUp(wrkGrpInfo->usedVGPRs_, VgprGranularity);
   }
 
+  if (VgprWaves == 0) {
+    // This should not happen ideally, but in case the usedVGPRs_/availableVGPRs_ values are
+    // incorrect, it can lead to a crash. By returning error, API can exit gracefully.
+    return hipErrorUnknown;
+  }
+
   size_t GprWaves = VgprWaves;
   if (wrkGrpInfo->usedSGPRs_ > 0) {
     size_t maxSGPRs = device.info().sgprsPerSimd_;
@@ -382,14 +435,20 @@ hipError_t ihipOccupancyMaxActiveBlocksPerMultiprocessor(
   // If the best block size is smaller than the block size used to fit the maximum,
   // then we need to make the grid bigger for full occupancy.
   const int bestBlocksPerCU = alu_limited_threads / (*bestBlockSize);
+  uint32_t maxCUs = device.info().maxComputeUnits_;
+  if (wrkGrpInfo->isWGPMode_ == false && device.settings().enableWgpMode_ == true) {
+    maxCUs *= 2;
+  } else if ((wrkGrpInfo->isWGPMode_ == true && device.settings().enableWgpMode_ == false)) {
+    maxCUs /= 2;
+  }
   // Unless those blocks are further constrained by LDS size.
-  *numBlocksPerGrid = device.info().maxComputeUnits_ * std::min(bestBlocksPerCU, lds_occupancy_wgs);
+  *numBlocksPerGrid = (maxCUs * std::min(bestBlocksPerCU, lds_occupancy_wgs));
 
   return hipSuccess;
 }
 }  // namespace hip_impl
 
-extern "C" {
+namespace hip {
 hipError_t hipOccupancyMaxPotentialBlockSize(int* gridSize, int* blockSize, const void* f,
                                              size_t dynSharedMemPerBlk, int blockSizeLimit) {
   HIP_INIT_API(hipOccupancyMaxPotentialBlockSize, f, dynSharedMemPerBlk, blockSizeLimit);
@@ -558,21 +617,30 @@ hipError_t hipOccupancyMaxActiveBlocksPerMultiprocessorWithFlags(int* numBlocks,
   *numBlocks = num_blocks;
   HIP_RETURN(ret);
 }
-}
 
 hipError_t ihipLaunchKernel(const void* hostFunction, dim3 gridDim, dim3 blockDim, void** args,
                             size_t sharedMemBytes, hipStream_t stream, hipEvent_t startEvent,
                             hipEvent_t stopEvent, int flags) {
+  if (!hip::isValid(stream)) {
+    return hipErrorInvalidValue;
+  }
   hipFunction_t func = nullptr;
   int deviceId = hip::Stream::DeviceId(stream);
   hipError_t hip_error = PlatformState::instance().getStatFunc(&func, hostFunction, deviceId);
   if ((hip_error != hipSuccess) || (func == nullptr)) {
-    if (hip_error == hipErrorSharedObjectInitFailed) {
+    if (hip_error == hipErrorNoBinaryForGpu) {
       return hip_error;
     } else {
       return hipErrorInvalidDeviceFunction;
     }
   }
+
+  constexpr auto gridDimYZmax = static_cast<uint64_t>(std::numeric_limits<uint16_t>::max()) + 1;
+  const auto& isa = g_devices[deviceId]->devices()[0]->isa().versionMajor();
+  if (isa >= 12 && (gridDim.y > gridDimYZmax || gridDim.z > gridDimYZmax)) {
+    return hipErrorInvalidConfiguration;
+  }
+
   size_t globalWorkSizeX = static_cast<size_t>(gridDim.x) * blockDim.x;
   size_t globalWorkSizeY = static_cast<size_t>(gridDim.y) * blockDim.y;
   size_t globalWorkSizeZ = static_cast<size_t>(gridDim.z) * blockDim.z;
@@ -668,11 +736,8 @@ void PlatformState::init() {
   initialized_ = true;
   for (auto& it : statCO_.modules_) {
     hipError_t err = digestFatBinary(it.first, it.second);
-    if (err == hipErrorNoBinaryForGpu) {
+    if (err != hipSuccess) {
       HIP_ERROR_PRINT(err, "continue parsing remaining modules");
-    } else if (err != hipSuccess) {
-      HIP_ERROR_PRINT(err);
-      return;
     }
   }
   for (auto& it : statCO_.vars_) {
@@ -747,11 +812,17 @@ hipError_t PlatformState::getDynFunc(hipFunction_t* hfunc, hipModule_t hmod,
   return it->second->getDynFunc(hfunc, func_name);
 }
 
+bool PlatformState::isValidDynFunc(const void* hfunc) {
+  amd::ScopedLock lock(lock_);
+  return std::any_of(dynCO_map_.begin(), dynCO_map_.end(),
+                     [&](auto& it) { return it.second->isValidDynFunc(hfunc); });
+}
+
 hipError_t PlatformState::getDynGlobalVar(const char* hostVar, hipModule_t hmod,
                                           hipDeviceptr_t* dev_ptr, size_t* size_ptr) {
   amd::ScopedLock lock(lock_);
 
-  if (hostVar == nullptr || dev_ptr == nullptr || size_ptr == nullptr) {
+  if (hostVar == nullptr) {
     return hipErrorInvalidValue;
   }
 
@@ -760,14 +831,20 @@ hipError_t PlatformState::getDynGlobalVar(const char* hostVar, hipModule_t hmod,
     LogPrintfError("Cannot find the module: 0x%x", hmod);
     return hipErrorNotFound;
   }
-  *dev_ptr = nullptr;
+  if (dev_ptr) {
+    *dev_ptr = nullptr;
+  }
   IHIP_RETURN_ONFAIL(it->second->getManagedVarPointer(hostVar, dev_ptr, size_ptr));
   // if dev_ptr is nullptr, hostvar is not in managed variable list
-  if (*dev_ptr == nullptr) {
+  if ((dev_ptr && *dev_ptr == nullptr) || (size_ptr && *size_ptr == 0)) {
     hip::DeviceVar* dvar = nullptr;
     IHIP_RETURN_ONFAIL(it->second->getDeviceVar(&dvar, hostVar));
-    *dev_ptr = dvar->device_ptr();
-    *size_ptr = dvar->size();
+    if (dev_ptr != nullptr) {
+      *dev_ptr = dvar->device_ptr();
+    }
+    if (size_ptr != nullptr) {
+      *size_ptr = dvar->size();
+    }
   }
   return hipSuccess;
 }
@@ -829,8 +906,8 @@ hipError_t PlatformState::digestFatBinary(const void* data, hip::FatBinaryInfo*&
   return statCO_.digestFatBinary(data, programs);
 }
 
-hip::FatBinaryInfo** PlatformState::addFatBinary(const void* data) {
-  return statCO_.addFatBinary(data, initialized_);
+hip::FatBinaryInfo** PlatformState::addFatBinary(const void* data, bool& success) {
+  return statCO_.addFatBinary(data, initialized_, success);
 }
 
 hipError_t PlatformState::removeFatBinary(hip::FatBinaryInfo** module) {
@@ -897,3 +974,57 @@ void PlatformState::popExec(ihipExec_t& exec) {
   exec = std::move(hip::tls.exec_stack_.top());
   hip::tls.exec_stack_.pop();
 }
+
+std::shared_ptr<UniqueFD> PlatformState::GetUniqueFileHandle(const std::string& file_path) {
+  amd::ScopedLock lock(ufd_lock_);
+
+  if (ufd_map_.cend() == ufd_map_.find(file_path)) {
+    // Get the file desc and file size from amd::Os API
+    amd::Os::FileDesc fdesc;
+    size_t fsize = 0;
+    if (!amd::Os::GetFileHandle(file_path.c_str(), &fdesc, &fsize)) {
+      return nullptr;
+    }
+    ufd_map_.insert(std::make_pair(file_path, std::make_shared<UniqueFD>(file_path, fdesc, fsize)));
+  }
+
+  // we should have an entry at this time.
+  return ufd_map_[file_path];
+}
+
+bool PlatformState::CloseUniqueFileHandle(const std::shared_ptr<UniqueFD>& ufd) {
+  amd::ScopedLock lock(ufd_lock_);
+
+  // if use_count is 2, then there is 1 entry in the map and the current entry is the last close.
+  if (ufd.use_count() == 2) {
+    ufd_map_.erase(ufd->fpath_);
+    if (!amd::Os::CloseFileHandle(ufd->fdesc_)) {
+      return false;
+    }
+  }
+  return true;
+}
+
+void* PlatformState::getDynamicLibraryHandle() {
+  amd::ScopedLock lock(lock_);
+
+  if (dynamicLibraryHandle_ != nullptr) {
+    return dynamicLibraryHandle_;
+  }
+
+#ifdef _WIN32
+  const char* libName = "amdhip64.dll";
+#else
+  const char* libName = "libamdhip64.so";
+#endif
+
+  dynamicLibraryHandle_ = amd::Os::loadLibrary(libName);
+  return dynamicLibraryHandle_;
+}
+
+void PlatformState::setDynamicLibraryHandle(void* handle){
+  amd::ScopedLock lock(lock_);
+  dynamicLibraryHandle_ = handle;
+}
+
+} //namespace hip
