@@ -25,8 +25,6 @@
 #include "hip_conversions.hpp"
 #include "platform/sampler.hpp"
 
-hipError_t ihipFree(void* ptr);
-
 struct __hip_texture {
   uint32_t imageSRD[HIP_IMAGE_OBJECT_SIZE_DWORD];
   uint32_t samplerSRD[HIP_SAMPLER_OBJECT_SIZE_DWORD];
@@ -57,6 +55,9 @@ struct __hip_texture {
   }
 };
 
+namespace hip {
+
+hipError_t ihipFree(void* ptr);
 amd::Image* ihipImageCreate(const cl_channel_order channelOrder,
                             const cl_channel_type channelType,
                             const cl_mem_object_type imageType,
@@ -67,6 +68,7 @@ amd::Image* ihipImageCreate(const cl_channel_order channelOrder,
                             const size_t imageRowPitch,
                             const size_t imageSlicePitch,
                             const uint32_t numMipLevels,
+                            const size_t offset,
                             amd::Memory* buffer,
                             hipError_t& status);
 
@@ -108,9 +110,22 @@ hipError_t ihipCreateTextureObject(hipTextureObject_t* pTexObject,
   // If hipResourceDesc::resType is set to hipResourceTypeMipmappedArray,
   // hipResourceDesc::res::mipmap::mipmap must be set to a valid HIP mipmapped array handle
   // and hipTextureDesc::normalizedCoords must be set to true.
-  if ((pResDesc->resType == hipResourceTypeMipmappedArray) &&
-      ((pResDesc->res.mipmap.mipmap == nullptr) || (pTexDesc->normalizedCoords == 0))) {
-    return hipErrorInvalidValue;
+  if (pResDesc->resType == hipResourceTypeMipmappedArray) {
+    bool mipMapSupport = true;
+    amd::Context& context = *hip::getCurrentDevice()->asContext();
+    const std::vector<amd::Device*>& devices = context.devices();
+    for (auto& dev : devices) {
+      if (!dev->settings().checkExtension(ClKhrMipMapImage)) {
+        mipMapSupport = false; // Now PAL-backend can support mipmap, Rocm-backend cannot.
+      }
+    }
+    if (mipMapSupport == false) {
+      LogInfo("Mipmap not supported on the device");
+      return hipErrorNotSupported;
+    }
+    if (pResDesc->res.mipmap.mipmap == nullptr || pTexDesc->normalizedCoords == 0) {
+      return hipErrorInvalidValue;
+    }
   }
 
   // If hipResourceDesc::resType is set to hipResourceTypeLinear,
@@ -119,7 +134,7 @@ hipError_t ihipCreateTextureObject(hipTextureObject_t* pTexObject,
   if ((pResDesc->resType == hipResourceTypeLinear) &&
       ((pResDesc->res.linear.devPtr == nullptr) ||
        (!amd::isMultipleOf(pResDesc->res.linear.devPtr, info.imageBaseAddressAlignment_)) ||
-       ((pResDesc->res.linear.sizeInBytes / hip::getElementSize(pResDesc->res.linear.desc)) >= info.imageMaxBufferSize_))) {
+       (pResDesc->res.linear.sizeInBytes >= info.imageMaxBufferSize_ * hip::getElementSize(pResDesc->res.linear.desc)))) {
     return hipErrorInvalidValue;
   }
 
@@ -239,8 +254,8 @@ hipError_t ihipCreateTextureObject(hipTextureObject_t* pTexObject,
       if (image == nullptr) {
         return hipErrorInvalidValue;
       }
-    } else if (image->parent()) {
-      image->retain();  // Because it will be released as a view in ihipDestroyTextureObject()
+    } else {
+      image->retain();  // will be released in ihipDestroyTextureObject()
     }
     break;
   }
@@ -280,6 +295,8 @@ hipError_t ihipCreateTextureObject(hipTextureObject_t* pTexObject,
       if (image == nullptr) {
         return hipErrorInvalidValue;
       }
+    } else {
+      image->retain();  // will be released in ihipDestroyTextureObject()
     }
     break;
   }
@@ -301,9 +318,13 @@ hipError_t ihipCreateTextureObject(hipTextureObject_t* pTexObject,
                             0, /* imageRowPitch */
                             0, /* imageSlicePitch */
                             0, /* numMipLevels */
+                            0, /* offset */
                             buffer,
                             status);
-    buffer->release();
+    if (buffer != nullptr) {
+      buffer->release();
+    }
+
     if (image == nullptr) {
       return status;
     }
@@ -328,6 +349,7 @@ hipError_t ihipCreateTextureObject(hipTextureObject_t* pTexObject,
                             pResDesc->res.pitch2D.pitchInBytes, /* imageRowPitch */
                             0, /* imageSlicePitch */
                             0, /* numMipLevels */
+                            0, /* offset */
                             buffer,
                             status);
     if (buffer != nullptr) {
@@ -370,13 +392,7 @@ hipError_t ihipDestroyTextureObject(hipTextureObject_t texObject) {
     return hipErrorNotSupported;
   }
 
-  const hipResourceType type = texObject->resDesc.resType;
-  const bool isImageFromBuffer = (type == hipResourceTypeLinear) || (type == hipResourceTypePitch2D);
-  const bool isImageView = ((type == hipResourceTypeArray) || (type == hipResourceTypeMipmappedArray)) &&
-                           texObject->image->parent() != nullptr;
-  // If the texture object was created from an array, then the array owns the image SRD.
-  // Otherwise, if the texture object is a view, or was created from a buffer, then it owns the image SRD.
-  if (isImageFromBuffer || isImageView) {
+  if (texObject != nullptr && texObject->image) {
     texObject->image->release();
   }
 
@@ -499,7 +515,7 @@ inline hipError_t ihipGetTextureAlignmentOffset(size_t* offset,
   // If the device memory pointer was returned from hipMalloc(),
   // the offset is guaranteed to be 0 and NULL may be passed as the offset parameter.
   if ((alignedOffset != 0) && (offset == nullptr)) {
-    LogPrintfError("Texture object not aligned with offset %u \n", alignedOffset);
+    LogPrintfError("Texture object not aligned with offset %u", alignedOffset);
     return hipErrorInvalidValue;
   }
 
@@ -540,7 +556,8 @@ hipError_t ihipBindTexture(size_t* offset,
   }
 
   // Align the user ptr to HW requirments.
-  resDesc.res.linear.devPtr = static_cast<char*>(const_cast<void*>(devPtr)) - *offset;
+  resDesc.res.linear.devPtr =
+      static_cast<char*>(const_cast<void*>(devPtr)) - (offset != nullptr ? *offset : 0);
   hipTextureDesc texDesc = hip::getTextureDesc(texref);
 
   return ihipCreateTextureObject(const_cast<hipTextureObject_t*>(&texref->textureObject), &resDesc, &texDesc, nullptr);
@@ -555,7 +572,8 @@ hipError_t ihipBindTexture2D(size_t* offset,
                              size_t pitch) {
   if ((texref == nullptr) ||
       (devPtr == nullptr) ||
-      (desc == nullptr)) {
+      (desc == nullptr) ||
+      (pitch == 0)) {
     return hipErrorInvalidValue;
   }
 
@@ -580,7 +598,8 @@ hipError_t ihipBindTexture2D(size_t* offset,
   }
 
   // Align the user ptr to HW requirments.
-  resDesc.res.pitch2D.devPtr = static_cast<char*>(const_cast<void*>(devPtr)) - *offset;
+  resDesc.res.pitch2D.devPtr =
+      static_cast<char*>(const_cast<void*>(devPtr)) - (offset != nullptr ? *offset : 0);
   hipTextureDesc texDesc = hip::getTextureDesc(texref);
 
   return ihipCreateTextureObject(const_cast<hipTextureObject_t*>(&texref->textureObject), &resDesc, &texDesc, nullptr);
@@ -614,12 +633,6 @@ hipError_t hipBindTexture2D(size_t* offset,
 hipError_t ihipBindTextureToArray(const textureReference* texref,
                                   hipArray_const_t array,
                                   const hipChannelFormatDesc* desc) {
-  if ((texref == nullptr) ||
-      (array == nullptr) ||
-      (desc == nullptr)) {
-    return hipErrorInvalidValue;
-  }
-
   // Any previous address or HIP array state associated with the texture reference is superseded by this function.
   // Any memory previously bound to hTexRef is unbound.
   // No need to check for errors.
@@ -645,6 +658,10 @@ hipError_t hipBindTextureToArray(const textureReference* texref,
                                  const hipChannelFormatDesc* desc) {
   HIP_INIT_API(hipBindTextureToArray, texref, array, desc);
 
+  if ((texref == nullptr) || (array == nullptr) || (desc == nullptr)) {
+    return hipErrorInvalidValue;
+  }
+
   hipDeviceptr_t refDevPtr = nullptr;
   size_t refDevSize = 0;
   HIP_RETURN_ONFAIL(PlatformState::instance().getStatGlobalVar(texref, ihipGetDevice(), &refDevPtr,
@@ -741,9 +758,12 @@ hipError_t hipGetChannelDesc(hipChannelFormatDesc* desc,
                              hipArray_const_t array) {
   HIP_INIT_API(hipGetChannelDesc, desc, array);
 
-  if ((desc == nullptr) || (array == nullptr)) {
+  if (desc == nullptr) {
     HIP_RETURN(hipErrorInvalidValue);
   }
+  if (array == nullptr) {
+    HIP_RETURN(hipErrorInvalidHandle);
+  }
   amd::Device* device = hip::getCurrentDevice()->devices()[0];
   const device::Info& info = device->info();
   if (!info.imageSupport_) {
@@ -889,7 +909,7 @@ hipError_t hipTexRefGetAddressMode(hipTextureAddressMode* pam,
   if ((dim != 0) && (dim != 1)) {
     LogPrintfError(
         "Currently only 2 dimensions (0,1) are valid,"
-        "dim : %d \n",
+        "dim : %d",
         dim);
     HIP_RETURN(hipErrorInvalidValue);
   }
@@ -911,7 +931,7 @@ hipError_t hipTexRefSetAddressMode(textureReference* texRef,
   if ((dim < 0) || (dim > 2)) {
     LogPrintfError(
         "Currently only 3 dimensions (0,1,2) are valid,"
-        "dim : %d \n",
+        "dim : %d",
         dim);
     HIP_RETURN(hipErrorInvalidValue);
   }
@@ -984,6 +1004,11 @@ hipError_t hipTexRefSetArray(textureReference* texRef,
     HIP_RETURN(err);
   }
 
+  // If flag = HIP_TRSA_OVERRIDE_FORMAT,
+  // Override the texref format with a format inferred from the array
+  texRef->format = array->Format;
+  texRef->numChannels = array->NumChannels;
+
   hipResourceDesc resDesc = {};
   resDesc.resType = hipResourceTypeArray;
   resDesc.res.array.array = const_cast<hipArray_t>(array);
@@ -1015,7 +1040,7 @@ hipError_t hipTexRefGetAddress(hipDeviceptr_t* dptr,
   // TODO use ihipGetTextureObjectResourceDesc() to not pollute the API trace.
   hipError_t error = ihipGetTextureObjectResourceDesc(&resDesc, texRef->textureObject);
   if (error != hipSuccess) {
-    LogPrintfError("hipGetTextureObjectResourceDesc failed with error code: %s \n",
+    LogPrintfError("hipGetTextureObjectResourceDesc failed with error code: %s",
                    ihipGetErrorName(error));
     HIP_RETURN(error);
   }
@@ -1073,7 +1098,7 @@ hipError_t hipTexRefSetAddress(size_t* ByteOffset,
   }
 
   // Align the user ptr to HW requirments.
-  resDesc.res.linear.devPtr = static_cast<char*>(dptr) - *ByteOffset;
+  resDesc.res.linear.devPtr = static_cast<char*>(dptr) - (ByteOffset ? *ByteOffset : 0);
   hipTextureDesc texDesc = hip::getTextureDesc(texRef);
 
   err = ihipCreateTextureObject(&texRef->textureObject, &resDesc, &texDesc, nullptr);
@@ -1245,7 +1270,7 @@ hipError_t hipTexRefGetMaxAnisotropy(int* pmaxAnsio,
 
   *pmaxAnsio = texRef->maxAnisotropy;
 
-  HIP_RETURN(hipErrorInvalidValue);
+  HIP_RETURN(hipSuccess);
 }
 
 hipError_t hipTexRefGetMipmapFilterMode(hipTextureFilterMode* pfm,
@@ -1311,10 +1336,10 @@ hipError_t hipTexRefGetMipmapLevelClamp(float* pminMipmapLevelClamp,
   HIP_RETURN(hipErrorInvalidValue);
 }
 
-hipError_t hipTexRefGetMipmappedArray(hipMipmappedArray_t* pArray,
+hipError_t hipTexRefGetMipMappedArray(hipMipmappedArray_t* pArray,
                                       const textureReference* texRef) {
   // TODO overload operator<<(ostream&, textureReference&).
-  HIP_INIT_API(hipTexRefGetMipmappedArray, pArray, &texRef);
+  HIP_INIT_API(hipTexRefGetMipMappedArray, pArray, &texRef);
 
   if ((pArray == nullptr) || (texRef == nullptr)) {
     HIP_RETURN(hipErrorInvalidValue);
@@ -1573,3 +1598,4 @@ hipError_t hipTexObjectGetTextureDesc(HIP_TEXTURE_DESC* pTexDesc,
 
   HIP_RETURN(hipSuccess);
 }
+}  // namespace hip
