@@ -375,9 +375,9 @@ RUNTIME_EXIT
 RUNTIME_ENTRY(cl_int, clEnqueueAcquireD3D11ObjectsKHR,
               (cl_command_queue command_queue, cl_uint num_objects, const cl_mem* mem_objects,
                cl_uint num_events_in_wait_list, const cl_event* event_wait_list, cl_event* event)) {
-  return amd::clEnqueueAcquireExtObjectsAMD(command_queue, num_objects, mem_objects,
-                                            num_events_in_wait_list, event_wait_list, event,
-                                            CL_COMMAND_ACQUIRE_D3D11_OBJECTS_KHR);
+  return amd::clDXTranslateErrorCode(amd::clEnqueueAcquireExtObjectsAMD(
+      command_queue, num_objects, mem_objects, num_events_in_wait_list, event_wait_list, event,
+      CL_COMMAND_ACQUIRE_D3D11_OBJECTS_KHR));
 }
 RUNTIME_EXIT
 
@@ -388,9 +388,9 @@ RUNTIME_EXIT
 RUNTIME_ENTRY(cl_int, clEnqueueReleaseD3D11ObjectsKHR,
               (cl_command_queue command_queue, cl_uint num_objects, const cl_mem* mem_objects,
                cl_uint num_events_in_wait_list, const cl_event* event_wait_list, cl_event* event)) {
-  return amd::clEnqueueReleaseExtObjectsAMD(command_queue, num_objects, mem_objects,
-                                            num_events_in_wait_list, event_wait_list, event,
-                                            CL_COMMAND_RELEASE_D3D11_OBJECTS_KHR);
+  return amd::clDXTranslateErrorCode(amd::clEnqueueReleaseExtObjectsAMD(
+      command_queue, num_objects, mem_objects, num_events_in_wait_list, event_wait_list, event,
+      CL_COMMAND_RELEASE_D3D11_OBJECTS_KHR));
 }
 RUNTIME_EXIT
 
