@@ -30,7 +30,7 @@
  */
 
 //! GPU Blit Manager Implementation
-namespace device {
+namespace amd::device {
 
 //! Blit Manager Abstraction class
 class BlitManager : public amd::HeapObject {
@@ -77,7 +77,7 @@ class BlitManager : public amd::HeapObject {
                           const amd::Coord3D& origin,  //!< Source origin
                           const amd::Coord3D& size,    //!< Size of the copy region
                           bool entire = false,         //!< Entire buffer will be updated
-                          amd::CopyMetadata copyMetadata = 
+                          amd::CopyMetadata copyMetadata =
                                     amd::CopyMetadata()//!< Memory copy MetaData
                           ) const = 0;
 
@@ -88,7 +88,7 @@ class BlitManager : public amd::HeapObject {
                               const amd::BufferRect& hostRect,  //!< Destination rectangle
                               const amd::Coord3D& size,         //!< Size of the copy region
                               bool entire = false,              //!< Entire buffer will be updated
-                              amd::CopyMetadata copyMetadata = 
+                              amd::CopyMetadata copyMetadata =
                                     amd::CopyMetadata()         //!< Memory copy MetaData
                               ) const = 0;
 
@@ -100,7 +100,7 @@ class BlitManager : public amd::HeapObject {
                          size_t rowPitch,             //!< Row pitch for host memory
                          size_t slicePitch,           //!< Slice pitch for host memory
                          bool entire = false,          //!< Entire buffer will be updated
-                         amd::CopyMetadata copyMetadata = 
+                         amd::CopyMetadata copyMetadata =
                                     amd::CopyMetadata()//!< Memory copy MetaData
                          ) const = 0;
 
@@ -110,7 +110,7 @@ class BlitManager : public amd::HeapObject {
                            const amd::Coord3D& origin,  //!< Destination origin
                            const amd::Coord3D& size,    //!< Size of the copy region
                            bool entire = false,         //!< Entire buffer will be updated
-                           amd::CopyMetadata copyMetadata = 
+                           amd::CopyMetadata copyMetadata =
                                      amd::CopyMetadata() //!< Memory copy MetaData
                            ) const = 0;
 
@@ -121,7 +121,7 @@ class BlitManager : public amd::HeapObject {
                                const amd::BufferRect& bufRect,   //!< Source rectangle
                                const amd::Coord3D& size,         //!< Size of the copy region
                                bool entire = false,              //!< Entire buffer will be updated
-                               amd::CopyMetadata copyMetadata = 
+                               amd::CopyMetadata copyMetadata =
                                     amd::CopyMetadata()          //!< Memory copy MetaData
                                ) const = 0;
 
@@ -133,7 +133,7 @@ class BlitManager : public amd::HeapObject {
                           size_t rowPitch,             //!< Row pitch for host memory
                           size_t slicePitch,           //!< Slice pitch for host memory
                           bool entire = false,          //!< Entire buffer will be updated
-                          amd::CopyMetadata copyMetadata = 
+                          amd::CopyMetadata copyMetadata =
                                     amd::CopyMetadata() //!< Memory copy MetaData
                           ) const = 0;
 
@@ -192,7 +192,7 @@ class BlitManager : public amd::HeapObject {
                          const amd::Coord3D& dstOrigin,  //!< Destination origin
                          const amd::Coord3D& size,       //!< Size of the copy region
                          bool entire = false,            //!< Entire buffer will be updated
-                         amd::CopyMetadata copyMetadata = 
+                         amd::CopyMetadata copyMetadata =
                                     amd::CopyMetadata() //!< Memory copy MetaData
                          ) const = 0;
 
@@ -233,6 +233,12 @@ class BlitManager : public amd::HeapObject {
                              uint64_t mask
   ) const = 0;
 
+  //! Stream batch memory operation
+  virtual bool batchMemOps(const void* paramArray,
+                           size_t paramSize,
+                           uint32_t count
+  ) const = 0;
+
   //! Enables synchronization on blit operations
   void enableSynchronization() { syncOperation_ = true; }
 
@@ -300,7 +306,7 @@ class HostBlitManager : public device::BlitManager {
                          size_t rowPitch,             //!< Row pitch for host memory
                          size_t slicePitch,           //!< Slice pitch for host memory
                          bool entire = false,         //!< Entire buffer will be updated
-                         amd::CopyMetadata copyMetadata = 
+                         amd::CopyMetadata copyMetadata =
                                     amd::CopyMetadata() //!< Memory copy MetaData
                          ) const;
 
@@ -333,7 +339,7 @@ class HostBlitManager : public device::BlitManager {
                           size_t rowPitch,             //!< Row pitch for host memory
                           size_t slicePitch,           //!< Slice pitch for host memory
                           bool entire = false,         //!< Entire buffer will be updated
-                          amd::CopyMetadata copyMetadata = 
+                          amd::CopyMetadata copyMetadata =
                                     amd::CopyMetadata() //!< Memory copy MetaData
                           ) const;
 
@@ -392,7 +398,7 @@ class HostBlitManager : public device::BlitManager {
                          const amd::Coord3D& dstOrigin,  //!< Destination origin
                          const amd::Coord3D& size,       //!< Size of the copy region
                          bool entire = false,            //!< Entire buffer will be updated
-                         amd::CopyMetadata copyMetadata = 
+                         amd::CopyMetadata copyMetadata =
                                     amd::CopyMetadata()  //!< Memory copy MetaData
                          ) const;
 
@@ -422,36 +428,24 @@ class HostBlitManager : public device::BlitManager {
   const amd::Device& dev_;  //!< Physical device
 
   // Packed Fill Buffer
-  class FillBufferInfo {
-  public:
-    FillBufferInfo(): fill_size_(0), expanded_pattern_(0), pattern_expanded_(false) {}
+  struct FillBufferInfo {
+    static constexpr uint32_t kExtendedSize = 2 * sizeof(uint64_t);
 
-    static bool PackInfo(const device::Memory& memory, size_t fill_size,
-                  size_t fill_origin, const void* pattern, size_t pattern_size,
-                  std::vector<FillBufferInfo>& packed_info);
-
-  private:
-    static bool ExpandPattern64(uint64_t pattern, size_t pattern_size, uint64_t& pattern64);
-
-    static inline void ClearBits64(uint64_t& pattern, uint64_t num_bits) {
-      pattern &= ~(~(static_cast<uint64_t>(0)) << num_bits);
+    static void PackInfo(const device::Memory& memory, size_t fill_size,
+                         size_t fill_origin, const void* pattern, size_t pattern_size,
+                         std::vector<FillBufferInfo>& packed_info);
 
+    FillBufferInfo(size_t fill_size): fill_size_(fill_size), pattern_expanded_(false) {
+      memset(&expanded_pattern_, 0, sizeof(expanded_pattern_));
     }
 
-    void clearInfo () {
-      fill_size_ = 0;
-      expanded_pattern_ = 0;
-      pattern_expanded_ = false;
-    }
+    void ExpandPattern(uint32_t pattern_size, const void* pattern);
 
-  public:
-    size_t fill_size_;          // Fill size for this command
-    uint64_t expanded_pattern_; // Pattern for this command
-    bool pattern_expanded_;     // Boolean to check if pattern is expanded
+    size_t fill_size_;                        //!< Fill size for this command
+    uint8_t expanded_pattern_[kExtendedSize]; //!< Pattern for this command - 16 bytes
+    bool pattern_expanded_;                   //!< Boolean to check if pattern is expanded
   };
 
-
-
  private:
   //! Disable copy constructor
   HostBlitManager(const HostBlitManager&);
@@ -460,6 +454,6 @@ class HostBlitManager : public device::BlitManager {
   HostBlitManager& operator=(const HostBlitManager&);
 };
 
-/*@}*/} // namespace device
+/*@}*/} // namespace amd::device
 
 #endif /*BLIT_HPP_*/
