@@ -74,6 +74,12 @@ typedef amd_comgr_status_t (*t_amd_comgr_symbol_get_info)(amd_comgr_symbol_t sym
 typedef amd_comgr_status_t (*t_amd_comgr_demangle_symbol_name)(amd_comgr_data_t MangledSymbolName, amd_comgr_data_t* DemangledSymbolName);
 typedef amd_comgr_status_t (*t_amd_comgr_populate_mangled_names)(amd_comgr_data_t data, size_t *count);
 typedef amd_comgr_status_t (*t_amd_comgr_get_mangled_name)(amd_comgr_data_t data, size_t index, size_t *size, char *mangled_name);
+typedef amd_comgr_status_t (*t_amd_comgr_populate_name_expression_map)(amd_comgr_data_t data, size_t *count);
+typedef amd_comgr_status_t (*t_amd_comgr_map_name_expression_to_symbol_name)(amd_comgr_data_t data, size_t *size, char *name_expression, char* symbol_name);
+typedef amd_comgr_status_t (*t_amd_comgr_action_info_set_device_lib_linking)(amd_comgr_action_info_t action_info, bool link_device_lib);
+typedef amd_comgr_status_t (*t_amd_comgr_action_info_set_bundle_entry_ids)(amd_comgr_action_info_t action_info, const char* bundle_entry_ids[], size_t count);
+typedef amd_comgr_status_t (*t_amd_comgr_set_data_from_file_slice)(amd_comgr_data_t data, int fd, uint64_t offset, uint64_t size);
+typedef amd_comgr_status_t (*t_amd_comgr_lookup_code_object)(amd_comgr_data_t data, amd_comgr_code_object_info_t *info, size_t size);
 
 struct ComgrEntryPoints {
   void* handle;
@@ -125,13 +131,21 @@ struct ComgrEntryPoints {
   t_amd_comgr_demangle_symbol_name      amd_comgr_demangle_symbol_name;
   t_amd_comgr_populate_mangled_names    amd_comgr_populate_mangled_names;
   t_amd_comgr_get_mangled_name          amd_comgr_get_mangled_name;
+  t_amd_comgr_populate_name_expression_map  amd_comgr_populate_name_expression_map;
+  t_amd_comgr_map_name_expression_to_symbol_name amd_comgr_map_name_expression_to_symbol_name;
+  t_amd_comgr_action_info_set_device_lib_linking amd_comgr_action_info_set_device_lib_linking;
+  t_amd_comgr_action_info_set_bundle_entry_ids   amd_comgr_action_info_set_bundle_entry_ids;
+  t_amd_comgr_set_data_from_file_slice   amd_comgr_set_data_from_file_slice;
+  t_amd_comgr_lookup_code_object  amd_comgr_lookup_code_object;
 };
 
 #ifdef COMGR_DYN_DLL
 #define COMGR_DYN(NAME) cep_.NAME
 #define GET_COMGR_SYMBOL(NAME) cep_.NAME = \
   reinterpret_cast<t_##NAME>(Os::getSymbol(cep_.handle, #NAME)); \
-  if (nullptr == cep_.NAME) { return false; }
+  if (nullptr == cep_.NAME) { \
+    ClPrint(amd::LOG_ERROR, amd::LOG_CODE, "Failed to load COMGR function %s", #NAME); \
+              return false; }
 #define GET_COMGR_OPTIONAL_SYMBOL(NAME) cep_.NAME = \
   reinterpret_cast<t_##NAME>(Os::getSymbol(cep_.handle, #NAME));
 #else
@@ -285,13 +299,6 @@ public:
   }
   static amd_comgr_status_t demangle_symbol_name(amd_comgr_data_t MangledSymbolName,
                                                  amd_comgr_data_t* DemangledSymbolName) {
-#if defined(COMGR_DYN_DLL)
-    if (cep_.amd_comgr_demangle_symbol_name == nullptr) {
-      ClPrint(amd::LOG_ERROR, amd::LOG_CODE,
-              "Failed to load COMGR function amd_comgr_demangle_symbol_name");
-      return AMD_COMGR_STATUS_ERROR;
-    }
-#endif
     return COMGR_DYN(amd_comgr_demangle_symbol_name)(MangledSymbolName, DemangledSymbolName);
   }
   static amd_comgr_status_t populate_mangled_names(amd_comgr_data_t data, size_t *count) {
@@ -300,8 +307,34 @@ public:
   static amd_comgr_status_t get_mangled_name(amd_comgr_data_t data, size_t index, size_t *size, char *mangled_name) {
     return COMGR_DYN(amd_comgr_get_mangled_name)(data, index, size, mangled_name);
   }
-
-
+  static amd_comgr_status_t populate_name_expression_map(amd_comgr_data_t data, size_t *count) {
+    return COMGR_DYN(amd_comgr_populate_name_expression_map)(data, count);
+  }
+  static amd_comgr_status_t map_name_expression_to_symbol_name(amd_comgr_data_t data, size_t *size, char *name_expression, char* symbol_name) {
+    return COMGR_DYN(amd_comgr_map_name_expression_to_symbol_name)(data, size, name_expression, symbol_name);
+  }
+  static amd_comgr_status_t action_info_set_device_lib_linking(amd_comgr_action_info_t action_info, bool link_device_lib) {
+    return COMGR_DYN(amd_comgr_action_info_set_device_lib_linking)(action_info, link_device_lib);
+  }
+  static amd_comgr_status_t set_data_from_file_slice(amd_comgr_data_t data, int fd, uint64_t offset, uint64_t size) {
+    return COMGR_DYN(amd_comgr_set_data_from_file_slice)(data, fd, offset, size);
+  }
+  static amd_comgr_status_t lookup_code_object(amd_comgr_data_t data, amd_comgr_code_object_info_t* info_list, size_t info_list_size) {
+    return COMGR_DYN(amd_comgr_lookup_code_object)(data, info_list, info_list_size);
+  }
+  static amd_comgr_status_t action_info_set_bundle_entry_ids(amd_comgr_action_info_t action_info,
+    const char* bundle_entry_ids[], size_t count) {
+#if defined(COMGR_DYN_DLL)
+    if (cep_.amd_comgr_action_info_set_bundle_entry_ids == nullptr) {
+      // comgr version 2.7 or less is loaded
+      ClPrint(amd::LOG_ERROR, amd::LOG_CODE,
+              "Failed to load COMGR function amd_comgr_action_info_set_bundle_entry_ids");
+      return AMD_COMGR_STATUS_ERROR;
+    }
+#endif
+    return COMGR_DYN(amd_comgr_action_info_set_bundle_entry_ids)(action_info, bundle_entry_ids,
+                     count);
+  }
  private:
   static ComgrEntryPoints cep_;
   static bool is_ready_;
