@@ -34,7 +34,7 @@ extern amd::AppProfile* rocCreateAppProfile();
 #endif
 
 #if defined(WITH_PAL_DEVICE)
-// namespace pal {
+// namespace amd::pal {
 extern bool PalDeviceLoad();
 extern void PalDeviceUnload();
 //}
@@ -73,7 +73,7 @@ constexpr char hsaIsaNamePrefix[] = "amdgcn-amd-amdhsa--";
 
 } // namespace
 
-namespace device {
+namespace amd::device {
 extern const char* BlitLinearSourceCode;
 extern const char* BlitImageSourceCode;
 
@@ -83,6 +83,16 @@ bool VirtualDevice::ActiveWait() const {
 
 }
 
+static_assert(static_cast<uint32_t>(device::Memory::MemAccess::kMemAccessNone)
+            == static_cast<uint32_t>(amd::Device::VmmAccess::kNone),
+            "Mem Access Flag None mismatch between Device and Memory!");
+static_assert(static_cast<uint32_t>(device::Memory::MemAccess::kMemAccessRead)
+            == static_cast<uint32_t>(amd::Device::VmmAccess::kReadOnly),
+            "Mem Access Flag Read mismatch between Device and Memory!");
+static_assert(static_cast<uint32_t>(device::Memory::MemAccess::kMemAccessReadWrite)
+            == static_cast<uint32_t>(amd::Device::VmmAccess::kReadWrite),
+            "Mem Access Flag Read Write mismatch between Device and Memory!");
+
 namespace amd {
 
 amd::Monitor Device::lockP2P_("Lock P2P ON/OFF");
@@ -112,31 +122,25 @@ std::pair<const Isa*, const Isa*> Isa::supportedIsas() {
   //                                        Supported        Version       Features
   //                                                                                    SIMD/ SIMD  Instr Bank    LDS     Mem
   // Target ID                 HSAIL ID     ROC    PAL      Maj/Min/Stp   SRAMECC XNACK CU    Width Width Width   Size    Banks
-    {"gfx700",                 "Kaveri",    true,  false,   7,  0,  0,    NONE,   NONE, 4,    16,   1,    256,    64 * Ki, 32}, // Also Spectre, Spooky, Kalindi
-    {"gfx701",                 "Hawaii",    true,  false,   7,  0,  1,    NONE,   NONE, 4,    16,   1,    256,    64 * Ki, 32}, // Actually Hawaiipro
-    {"gfx702",                 "gfx702",    true,  false,   7,  0,  2,    NONE,   NONE, 4,    16,   1,    256,    64 * Ki, 32}, // Actually Hawaii (can execute Hawiipro code)
-    {"gfx703",                 nullptr,     false, false,   7,  0,  3,    NONE,   NONE, 4,    16,   1,    256,    64 * Ki, 32}, // Mullins
-    {"gfx704",                 "Bonaire",   false, false,   7,  0,  4,    NONE,   NONE, 4,    16,   1,    256,    64 * Ki, 32},
-    {"gfx705",                 "Mullins",   false, false,   7,  0,  5,    NONE,   NONE, 4,    16,   1,    256,    64 * Ki, 32}, // Actually Godavari
     {"gfx801",                 nullptr,     true,  true,    8,  0,  1,    NONE,   ANY,  4,    16,   1,    256,    64 * Ki, 32},
     {"gfx801:xnack-",          nullptr,     true,  false,   8,  0,  1,    NONE,   OFF,  4,    16,   1,    256,    64 * Ki, 32},
-    {"gfx801:xnack+",          "Carrizo",   true,  true,    8,  0,  1,    NONE,   ON,   4,    16,   1,    256,    64 * Ki, 32},
-    {"gfx802",                 "Tonga",     true,  true,    8,  0,  2,    NONE,   NONE, 4,    16,   1,    256,    64 * Ki, 32}, // Also Iceland
-    {"gfx803",                 "Fiji",      true,  true,    8,  0,  3,    NONE,   NONE, 4,    16,   1,    256,    64 * Ki, 32}, // Also Ellesmere/Polaris10, Baffin/Polaris11, Polaris12, Polaris22/VegaM
-    {"gfx805",                 nullptr,     true,  true,    8,  0,  5,    NONE,   NONE, 4,    16,   1,    256,    64 * Ki, 32}, // Tongapro
+    {"gfx801:xnack+",          "gfx801",    true,  true,    8,  0,  1,    NONE,   ON,   4,    16,   1,    256,    64 * Ki, 32},
+    {"gfx802",                 "gfx802",    true,  true,    8,  0,  2,    NONE,   NONE, 4,    16,   1,    256,    64 * Ki, 32},
+    {"gfx803",                 "gfx803",    true,  true,    8,  0,  3,    NONE,   NONE, 4,    16,   1,    256,    64 * Ki, 32},
+    {"gfx805",                 nullptr,     true,  true,    8,  0,  5,    NONE,   NONE, 4,    16,   1,    256,    64 * Ki, 32},
     {"gfx810",                 nullptr,     true,  true,    8,  1,  0,    NONE,   ANY,  4,    16,   1,    256,    64 * Ki, 32},
     {"gfx810:xnack-",          nullptr,     true,  false,   8,  1,  0,    NONE,   OFF,  4,    16,   1,    256,    64 * Ki, 32},
-    {"gfx810:xnack+",          "Stoney",    true,  true,    8,  1,  0,    NONE,   ON,   4,    16,   1,    256,    64 * Ki, 32},
-    {"gfx900",                 "gfx901",    true,  true,    9,  0,  0,    NONE,   ANY,  4,    16,   1,    256,    64 * Ki, 32}, // Also Greenland
+    {"gfx810:xnack+",          "gfx810",    true,  true,    8,  1,  0,    NONE,   ON,   4,    16,   1,    256,    64 * Ki, 32},
+    {"gfx900",                 "gfx901",    true,  true,    9,  0,  0,    NONE,   ANY,  4,    16,   1,    256,    64 * Ki, 32},
     {"gfx900:xnack-",          "gfx900",    true,  true,    9,  0,  0,    NONE,   OFF,  4,    16,   1,    256,    64 * Ki, 32},
     {"gfx900:xnack+",          "gfx901",    true,  true,    9,  0,  0,    NONE,   ON,   4,    16,   1,    256,    64 * Ki, 32},
-    {"gfx902",                 "gfx903",    true,  true,    9,  0,  2,    NONE,   ANY,  4,    16,   1,    256,    64 * Ki, 32}, // Also Raven
+    {"gfx902",                 "gfx903",    true,  true,    9,  0,  2,    NONE,   ANY,  4,    16,   1,    256,    64 * Ki, 32},
     {"gfx902:xnack-",          "gfx902",    true,  true,    9,  0,  2,    NONE,   OFF,  4,    16,   1,    256,    64 * Ki, 32},
     {"gfx902:xnack+",          "gfx903",    true,  true,    9,  0,  2,    NONE,   ON,   4,    16,   1,    256,    64 * Ki, 32},
-    {"gfx904",                 "gfx905",    true,  true,    9,  0,  4,    NONE,   ANY,  4,    16,   1,    256,    64 * Ki, 32}, // Also Vega12
+    {"gfx904",                 "gfx905",    true,  true,    9,  0,  4,    NONE,   ANY,  4,    16,   1,    256,    64 * Ki, 32},
     {"gfx904:xnack-",          "gfx904",    true,  true,    9,  0,  4,    NONE,   OFF,  4,    16,   1,    256,    64 * Ki, 32},
     {"gfx904:xnack+",          "gfx905",    true,  true,    9,  0,  4,    NONE,   ON,   4,    16,   1,    256,    64 * Ki, 32},
-    {"gfx906",                 "gfx907",    true,  true,    9,  0,  6,    ANY,    ANY,  4,    16,   1,    256,    64 * Ki, 32}, // Also Vega20
+    {"gfx906",                 "gfx907",    true,  true,    9,  0,  6,    ANY,    ANY,  4,    16,   1,    256,    64 * Ki, 32},
     {"gfx906:sramecc-",        "gfx907",    true,  true,    9,  0,  6,    OFF,    ANY,  4,    16,   1,    256,    64 * Ki, 32},
     {"gfx906:sramecc+",        nullptr,     true,  true,    9,  0,  6,    ON,     ANY,  4,    16,   1,    256,    64 * Ki, 32},
     {"gfx906:xnack-",          "gfx906",    true,  true,    9,  0,  6,    ANY,    OFF,  4,    16,   1,    256,    64 * Ki, 32},
@@ -154,7 +158,7 @@ std::pair<const Isa*, const Isa*> Isa::supportedIsas() {
     {"gfx908:sramecc-:xnack+", nullptr,     true,  false,   9,  0,  8,    OFF,    ON,   4,    16,   1,    256,    64 * Ki, 32},
     {"gfx908:sramecc+:xnack-", nullptr,     true,  false,   9,  0,  8,    ON,     OFF,  4,    16,   1,    256,    64 * Ki, 32},
     {"gfx908:sramecc+:xnack+", nullptr,     true,  false,   9,  0,  8,    ON,     ON,   4,    16,   1,    256,    64 * Ki, 32},
-    {"gfx909",                 nullptr,     false, true,    9,  0,  2,    NONE,   ANY,  4,    16,   1,    256,    64 * Ki, 32}, // Also Raven2 (can execute Raven code)
+    {"gfx909",                 nullptr,     false, true,    9,  0,  2,    NONE,   ANY,  4,    16,   1,    256,    64 * Ki, 32},
     {"gfx909:xnack-",          nullptr,     false, true,    9,  0,  2,    NONE,   OFF,  4,    16,   1,    256,    64 * Ki, 32},
     {"gfx909:xnack+",          nullptr,     false, true,    9,  0,  2,    NONE,   ON,   4,    16,   1,    256,    64 * Ki, 32},
     {"gfx90a",                 nullptr,     true,  false,   9,  0,  10,   ANY,    ANY,  4,    16,   1,    256,    64 * Ki, 32},
@@ -193,9 +197,21 @@ std::pair<const Isa*, const Isa*> Isa::supportedIsas() {
     {"gfx942:sramecc-:xnack+", nullptr,     true,  false,   9,  4,  2,    OFF,    ON,   4,    16,   1,    256,    64 * Ki, 32},
     {"gfx942:sramecc+:xnack-", nullptr,     true,  false,   9,  4,  2,    ON,     OFF,  4,    16,   1,    256,    64 * Ki, 32},
     {"gfx942:sramecc+:xnack+", nullptr,     true,  false,   9,  4,  2,    ON,     ON,   4,    16,   1,    256,    64 * Ki, 32},
-    {"gfx90c",                 nullptr,     true,  true,    9,  0,  12,   NONE,   ANY,  4,    16,   1,    256,    64 * Ki, 32}, // Also Renoir
+    {"gfx90c",                 nullptr,     true,  true,    9,  0,  12,   NONE,   ANY,  4,    16,   1,    256,    64 * Ki, 32},
     {"gfx90c:xnack-",          "gfx90c",    true,  true,    9,  0,  12,   NONE,   OFF,  4,    16,   1,    256,    64 * Ki, 32},
     {"gfx90c:xnack+",          "gfx90d",    true,  true,    9,  0,  12,   NONE,   ON,   4,    16,   1,    256,    64 * Ki, 32},
+    {"gfx9-generic",           nullptr,     true,  true,    9,  0,  0,    NONE,   ANY,  4,    16,   1,    256,    64 * Ki, 32},
+    {"gfx9-generic:xnack-",    nullptr,     true,  true,    9,  0,  0,    NONE,   OFF,  4,    16,   1,    256,    64 * Ki, 32},
+    {"gfx9-generic:xnack+",    nullptr,     true,  true,    9,  0,  0,    NONE,   ON,   4,    16,   1,    256,    64 * Ki, 32},
+    {"gfx9-4-generic",         nullptr,     true,  true,    9,  4,  0,    ANY,    ANY,  4,    16,   1,    256,    64 * Ki, 32},
+    {"gfx9-4-generic:sramecc-",nullptr,     true,  true,    9,  4,  0,    OFF,    ANY,  4,    16,   1,    256,    64 * Ki, 32},
+    {"gfx9-4-generic:sramecc+",nullptr,     true,  true,    9,  4,  0,    ON,     ANY,  4,    16,   1,    256,    64 * Ki, 32},
+    {"gfx9-4-generic:xnack-",  nullptr,     true,  true,    9,  4,  0,    ANY,    OFF,  4,    16,   1,    256,    64 * Ki, 32},
+    {"gfx9-4-generic:xnack+",  nullptr,     true,  true,    9,  4,  0,    ANY,    ON,   4,    16,   1,    256,    64 * Ki, 32},
+    {"gfx9-4-generic:sramecc-:xnack-",nullptr,true,true,    9,  4,  0,    OFF,    OFF,  4,    16,   1,    256,    64 * Ki, 32},
+    {"gfx9-4-generic:sramecc-:xnack+",nullptr,true,true,    9,  4,  0,    OFF,    ON,   4,    16,   1,    256,    64 * Ki, 32},
+    {"gfx9-4-generic:sramecc+:xnack-",nullptr,true,true,    9,  4,  0,    ON,     OFF,  4,    16,   1,    256,    64 * Ki, 32},
+    {"gfx9-4-generic:sramecc+:xnack+",nullptr,true,true,    9,  4,  0,    ON,     ON,   4,    16,   1,    256,    64 * Ki, 32},
     {"gfx1010",                "gfx1010",   true,  true,    10, 1,  0,    NONE,   ANY,  2,    32,   1,    256,    64 * Ki, 32},
     {"gfx1010:xnack-",         "gfx1010",   true,  true,    10, 1,  0,    NONE,   OFF,  2,    32,   1,    256,    64 * Ki, 32},
     {"gfx1010:xnack+",         nullptr,     true,  true,    10, 1,  0,    NONE,   ON,   2,    32,   1,    256,    64 * Ki, 32},
@@ -208,6 +224,9 @@ std::pair<const Isa*, const Isa*> Isa::supportedIsas() {
     {"gfx1013",                "gfx1013",   true,  false,   10, 1,  3,    NONE,   ANY,  2,    32,   1,    256,    64 * Ki, 32},
     {"gfx1013:xnack-",         "gfx1013",   true,  false,   10, 1,  3,    NONE,   OFF,  2,    32,   1,    256,    64 * Ki, 32},
     {"gfx1013:xnack+",         nullptr,     true,  false,   10, 1,  3,    NONE,   ON,   2,    32,   1,    256,    64 * Ki, 32},
+    {"gfx10-1-generic",        nullptr,     true,  true,    10, 1,  0,    NONE,   ANY,  2,    32,   1,    256,    64 * Ki, 32},
+    {"gfx10-1-generic:xnack-", nullptr,     true,  true,    10, 1,  0,    NONE,   OFF,  2,    32,   1,    256,    64 * Ki, 32},
+    {"gfx10-1-generic:xnack+", nullptr,     true,  true,    10, 1,  0,    NONE,   ON,   2,    32,   1,    256,    64 * Ki, 32},
     {"gfx1030",                "gfx1030",   true,  true,    10, 3,  0,    NONE,   NONE, 2,    32,   1,    256,    64 * Ki, 32},
     {"gfx1031",                "gfx1031",   true,  true,    10, 3,  1,    NONE,   NONE, 2,    32,   1,    256,    64 * Ki, 32},
     {"gfx1032",                "gfx1032",   true,  true,    10, 3,  2,    NONE,   NONE, 2,    32,   1,    256,    64 * Ki, 32},
@@ -215,10 +234,19 @@ std::pair<const Isa*, const Isa*> Isa::supportedIsas() {
     {"gfx1034",                "gfx1034",   true,  true,    10, 3,  4,    NONE,   NONE, 2,    32,   1,    256,    64 * Ki, 32},
     {"gfx1035",                "gfx1035",   true,  true,    10, 3,  5,    NONE,   NONE, 2,    32,   1,    256,    64 * Ki, 32},
     {"gfx1036",                "gfx1036",   true,  true,    10, 3,  6,    NONE,   NONE, 2,    32,   1,    256,    64 * Ki, 32},
+    {"gfx10-3-generic",        nullptr,     true,  true,    10, 3,  0,    NONE,   NONE, 2,    32,   1,    256,    64 * Ki, 32},
     {"gfx1100",                "gfx1100",   true,  true,    11, 0,  0,    NONE,   NONE, 2,    32,   1,    256,    64 * Ki, 32},
     {"gfx1101",                "gfx1101",   true,  true,    11, 0,  1,    NONE,   NONE, 2,    32,   1,    256,    64 * Ki, 32},
     {"gfx1102",                "gfx1102",   true,  true,    11, 0,  2,    NONE,   NONE, 2,    32,   1,    256,    64 * Ki, 32},
     {"gfx1103",                "gfx1103",   true,  true,    11, 0,  3,    NONE,   NONE, 2,    32,   1,    256,    64 * Ki, 32},
+    {"gfx1150",                "gfx1150",   true,  true,    11, 5,  0,    NONE,   NONE, 2,    32,   1,    256,    64 * Ki, 32},
+    {"gfx1151",                "gfx1151",   true,  true,    11, 5,  1,    NONE,   NONE, 2,    32,   1,    256,    64 * Ki, 32},
+    {"gfx1152",                "gfx1152",   true,  true,    11, 5,  2,    NONE,   NONE, 2,    32,   1,    256,    64 * Ki, 32},
+    {"gfx1153",                "gfx1153",   true,  true,    11, 5,  3,    NONE,   NONE, 2,    32,   1,    256,    64 * Ki, 32},
+    {"gfx11-generic",          nullptr,     true,  true,    11, 0,  0,    NONE,   NONE, 2,    32,   1,    256,    64 * Ki, 32},
+    {"gfx1200",                "gfx1200",   true,  true,    12, 0,  0,    NONE,   NONE, 2,    32,   1,    256,    64 * Ki, 32},
+    {"gfx1201",                "gfx1201",   true,  true,    12, 0,  1,    NONE,   NONE, 2,    32,   1,    256,    64 * Ki, 32},
+    {"gfx12-generic",          nullptr,     true,  true,    12, 0,  0,    NONE,   NONE, 2,    32,   1,    256,    64 * Ki, 32},
   };
   return std::make_pair(std::begin(supportedIsas_), std::end(supportedIsas_));
 }
@@ -233,13 +261,34 @@ std::string Isa::isaName() const {
 }
 
 bool Isa::isCompatible(const Isa &codeObjectIsa, const Isa &agentIsa) {
-  if (codeObjectIsa.versionMajor() != agentIsa.versionMajor() ||
-      codeObjectIsa.versionMinor() != agentIsa.versionMinor() ||
-      codeObjectIsa.versionStepping() != agentIsa.versionStepping())
-    return false;
-
-  assert(codeObjectIsa.isSrameccSupported() == agentIsa.isSrameccSupported() &&
-         agentIsa.sramecc() != Feature::Any);
+  bool isGeneric = std::strstr(codeObjectIsa.targetId(), "generic") != nullptr;
+  if (isGeneric) {
+    if (codeObjectIsa.versionMajor() != agentIsa.versionMajor() ||
+        codeObjectIsa.versionMinor() > agentIsa.versionMinor() ||
+        (codeObjectIsa.versionMinor() == agentIsa.versionMinor() &&
+         codeObjectIsa.versionStepping() > agentIsa.versionStepping())) {
+      return false;
+    }
+#ifdef DEBUG
+    // Only check in DEBUG mode
+    if (std::strstr(agentIsa.targetId(), "gfx906") != nullptr) {
+      // For the generic target of gfx906, codeObjectIsa.isSrameccSupported() == false while
+      // agentIsa.isSrameccSupported() = true
+      assert(agentIsa.sramecc() != Feature::Any);
+    }
+    else {
+      assert(codeObjectIsa.isSrameccSupported() == agentIsa.isSrameccSupported() &&
+             agentIsa.sramecc() != Feature::Any);
+    }
+#endif
+  } else {
+    if (codeObjectIsa.versionMajor() != agentIsa.versionMajor() ||
+        codeObjectIsa.versionMinor() != agentIsa.versionMinor() ||
+        codeObjectIsa.versionStepping() != agentIsa.versionStepping())
+      return false;
+    assert(codeObjectIsa.isSrameccSupported() == agentIsa.isSrameccSupported() &&
+           agentIsa.sramecc() != Feature::Any);
+  }
   if ((codeObjectIsa.sramecc() == Feature::Enabled ||
        codeObjectIsa.sramecc() == Feature::Disabled) &&
       codeObjectIsa.sramecc() != agentIsa.sramecc())
@@ -292,20 +341,16 @@ std::vector<Device*>* Device::devices_ = nullptr;
 AppProfile Device::appProfile_;
 
 Context* Device::glb_ctx_ = nullptr;
-Monitor Device::p2p_stage_ops_("P2P Staging Lock", true);
+// P2P Staging Lock
+Monitor Device::p2p_stage_ops_(true);
 Memory* Device::p2p_stage_ = nullptr;
 
-Monitor MemObjMap::AllocatedLock_ ROCCLR_INIT_PRIORITY(101) ("Guards MemObjMap allocation list");
+std::shared_mutex MemObjMap::AllocatedLock_ ROCCLR_INIT_PRIORITY(101);
 std::map<uintptr_t, amd::Memory*> MemObjMap::MemObjMap_ ROCCLR_INIT_PRIORITY(101);
 std::map<uintptr_t, amd::Memory*> MemObjMap::VirtualMemObjMap_ ROCCLR_INIT_PRIORITY(101);
 
-size_t MemObjMap::size() {
-  amd::ScopedLock lock(AllocatedLock_);
-  return MemObjMap_.size();
-}
-
 void MemObjMap::AddMemObj(const void* k, amd::Memory* v) {
-  amd::ScopedLock lock(AllocatedLock_);
+  std::unique_lock lock(AllocatedLock_);
   auto rval = MemObjMap_.insert({ reinterpret_cast<uintptr_t>(k), v });
   if (!rval.second) {
     DevLogPrintfError("Memobj map already has an entry for ptr: 0x%x",
@@ -314,14 +359,14 @@ void MemObjMap::AddMemObj(const void* k, amd::Memory* v) {
 }
 
 void MemObjMap::RemoveMemObj(const void* k) {
-  amd::ScopedLock lock(AllocatedLock_);
+  std::unique_lock lock(AllocatedLock_);
   auto rval = MemObjMap_.erase(reinterpret_cast<uintptr_t>(k));
   guarantee(rval == 1, "Memobj map does not have ptr: 0x%x",
                         reinterpret_cast<uintptr_t>(k));
 }
 
 amd::Memory* MemObjMap::FindMemObj(const void* k, size_t* offset) {
-  amd::ScopedLock lock(AllocatedLock_);
+  std::shared_lock lock(AllocatedLock_);
   uintptr_t key = reinterpret_cast<uintptr_t>(k);
   auto it = MemObjMap_.upper_bound(key);
   if (it == MemObjMap_.begin()) {
@@ -330,7 +375,9 @@ amd::Memory* MemObjMap::FindMemObj(const void* k, size_t* offset) {
 
   --it;
   amd::Memory* mem = it->second;
-  if (key >= it->first && key < (it->first + mem->getSize())) {
+  size_t mem_size = (mem->getMemFlags() & ROCCLR_MEM_PHYMEM) ? sizeof(mem->getUserData().hsa_handle)
+                                                             : mem->getSize();
+  if (key >= it->first && key < (it->first + mem_size)) {
     if (offset != nullptr) {
       *offset = key - it->first;
     }
@@ -340,8 +387,44 @@ amd::Memory* MemObjMap::FindMemObj(const void* k, size_t* offset) {
     return nullptr;
   }
 }
+
+void MemObjMap::UpdateAccess(amd::Device *peerDev) {
+  if (peerDev == nullptr) {
+    return;
+  }
+  // Provides access to all memory allocated on peerDev but
+  // hsa_amd_agents_allow_access was not called because there was no peer
+  std::shared_lock lock(AllocatedLock_);
+  for (auto it : MemObjMap_) {
+    const std::vector<Device*>& devices = it.second->getContext().devices();
+    if (devices.size() == 1 && devices[0] == peerDev) {
+      device::Memory* devMem = it.second->getDeviceMemory(*devices[0]);
+      if (!devMem->getAllowedPeerAccess()) {
+        peerDev->deviceAllowAccess(reinterpret_cast<void*>(it.first));
+        devMem->setAllowedPeerAccess(true);
+      }
+    }
+  }
+}
+
+void MemObjMap::Purge(amd::Device* dev) {
+  assert(dev != nullptr);
+  std::unique_lock lock(AllocatedLock_);
+  for (auto it = MemObjMap_.cbegin(); it != MemObjMap_.cend(); ) {
+    amd::Memory* memObj = it->second;
+    unsigned int flags = memObj->getMemFlags();
+    const std::vector<Device*>& devices = memObj->getContext().devices();
+    if (devices.size() == 1 && devices[0] == dev && !(flags & ROCCLR_MEM_INTERNAL_MEMORY)) {
+      memObj->release();
+      it = MemObjMap_.erase(it);
+    } else {
+      ++it;
+    }
+  }
+}
+
 void MemObjMap::AddVirtualMemObj(const void* k, amd::Memory* v) {
-  amd::ScopedLock lock(AllocatedLock_);
+  std::unique_lock lock(AllocatedLock_);
   auto rval = VirtualMemObjMap_.insert({ reinterpret_cast<uintptr_t>(k), v });
   if (!rval.second) {
     DevLogPrintfError("Virtual Memobj map already has an entry for ptr: 0x%x",
@@ -350,14 +433,14 @@ void MemObjMap::AddVirtualMemObj(const void* k, amd::Memory* v) {
 }
 
 void MemObjMap::RemoveVirtualMemObj(const void* k) {
-  amd::ScopedLock lock(AllocatedLock_);
+  std::unique_lock lock(AllocatedLock_);
   auto rval = VirtualMemObjMap_.erase(reinterpret_cast<uintptr_t>(k));
   guarantee(rval == 1, "Virtual Memobj map does not have ptr: 0x%x",
                        reinterpret_cast<uintptr_t>(k));
 }
 
 amd::Memory* MemObjMap::FindVirtualMemObj(const void* k) {
-  amd::ScopedLock lock(AllocatedLock_);
+  std::shared_lock lock(AllocatedLock_);
   uintptr_t key = reinterpret_cast<uintptr_t>(k);
   auto it = VirtualMemObjMap_.upper_bound(key);
   if (it == VirtualMemObjMap_.begin()) {
@@ -374,41 +457,112 @@ amd::Memory* MemObjMap::FindVirtualMemObj(const void* k) {
   }
 }
 
-void MemObjMap::UpdateAccess(amd::Device *peerDev) {
-  if (peerDev == nullptr) {
-    return;
+//==================================================================================================
+bool Device::ValidateVirtualAddressRange(amd::Memory* vaddr_base_obj, amd::Memory* vaddr_sub_obj) {
+
+  // Check if the start of the subbuffer is >= to base start.
+  if (vaddr_base_obj->getSvmPtr() > vaddr_sub_obj->getSvmPtr()) {
+    LogError("Sub buffer cannot start with addr lesser than base_start.");
+    return false;
   }
 
-  // Provides access to all memory allocated on peerDev but
-  // hsa_amd_agents_allow_access was not called because there was no peer
-  amd::ScopedLock lock(AllocatedLock_);
-  for (auto it : MemObjMap_) {
-    const std::vector<Device*>& devices = it.second->getContext().devices();
-    if (devices.size() == 1 && devices[0] == peerDev) {
-      device::Memory* devMem = it.second->getDeviceMemory(*devices[0]);
-      if (!devMem->getAllowedPeerAccess()) {
-        peerDev->deviceAllowAccess(reinterpret_cast<void*>(it.first));
-        devMem->setAllowedPeerAccess(true);
-      }
-    }
+  // Check if the new size belongs to the vaddr_base_obj range.
+  address vaddr_base_end = reinterpret_cast<address>(vaddr_base_obj->getSvmPtr())
+                             + vaddr_base_obj->getSize();
+  address vaddr_sub_end = reinterpret_cast<address>(vaddr_sub_obj->getSvmPtr())
+                            + vaddr_sub_obj->getSize();
+
+  if (vaddr_sub_end > vaddr_base_end) {
+    LogError("Sub buffer memory end cannot be greater than base_end. Return nullptr");
+    return false;
   }
+
+  return true;
 }
 
-void MemObjMap::Purge(amd::Device* dev) {
-  assert(dev != nullptr);
+//==================================================================================================
+amd::Memory* Device::CreateVirtualBuffer(amd::Context& device_context, void* vptr, size_t size,
+                                         int deviceId, bool parent, bool kForceAlloc) {
 
-  amd::ScopedLock lock(AllocatedLock_);
-  for (auto it = MemObjMap_.cbegin(); it != MemObjMap_.cend(); ) {
-    amd::Memory* memObj = it->second;
-    unsigned int flags = memObj->getMemFlags();
-    const std::vector<Device*>& devices = memObj->getContext().devices();
-    if (devices.size() == 1 && devices[0] == dev && !(flags & ROCCLR_MEM_INTERNAL_MEMORY)) {
-      memObj->release();
-      it = MemObjMap_.erase(it);
-    } else {
-      ++it;
+  amd::Memory* vaddr_base_obj = nullptr;
+  amd::Memory* vaddr_sub_obj = nullptr;
+  constexpr bool kSysMemAlloc = false;
+  constexpr bool kSkipAlloc = false;
+
+  if (parent) {
+    vaddr_base_obj = new (GlbCtx()) amd::Buffer(GlbCtx(), CL_MEM_VA_RANGE_AMD, size, vptr);
+    if (vaddr_base_obj == nullptr) {
+      LogError("failed to new a va range curr_mem_obj object!");
+      return nullptr;
+    }
+    // This curr_mem_obj->create() does not create an actual memory but stores the memory info
+    // with given vptr on ROCr backend.
+    if (!vaddr_base_obj->create(nullptr, kSysMemAlloc, kSkipAlloc, kForceAlloc)) {
+      LogError("failed to create a va range mem object");
+      vaddr_base_obj->release();
+      return nullptr;
+    }
+
+    amd::MemObjMap::AddVirtualMemObj(vaddr_base_obj->getSvmPtr(), vaddr_base_obj);
+  } else {
+    // If not parent, but sub-buffer/child, then validate the address range
+    vaddr_base_obj = amd::MemObjMap::FindVirtualMemObj(vptr);
+    if (vaddr_base_obj == nullptr) {
+      LogPrintfError("Cannot find entry in VirtualMemObjMap: 0x%x \n", vptr);
+      return nullptr;
+    }
+    assert(vaddr_base_obj->getMemFlags() & CL_MEM_VA_RANGE_AMD);
+
+    size_t offset = (reinterpret_cast<address>(vptr)
+                     - reinterpret_cast<address>(vaddr_base_obj->getSvmPtr()));
+    Context& ctx = vaddr_base_obj->getContext();
+    vaddr_sub_obj = new (ctx) amd::Buffer(*vaddr_base_obj,CL_MEM_VA_RANGE_AMD, offset, size);
+
+    // This curr_mem_obj->create() does not create an actual memory but stores the memory info
+    // with given vptr on ROCr backend.
+    if (!vaddr_sub_obj->create(nullptr, kSysMemAlloc, kSkipAlloc, kForceAlloc)) {
+      LogError("failed to create a va range mem object");
+      vaddr_sub_obj->release();
+      return nullptr;
+    }
+
+    vaddr_sub_obj->getUserData().deviceId = deviceId;
+
+    if (!ValidateVirtualAddressRange(vaddr_base_obj, vaddr_sub_obj)) {
+      LogError("Validation failed on address range, returning nullptr");
+      return nullptr;
     }
   }
+
+  if (vptr != nullptr) {
+    // Assert to make sure that amd::Memory object has set the right ptr.
+    guarantee(vptr == (parent ? vaddr_base_obj->getSvmPtr() : vaddr_sub_obj->getSvmPtr()),
+                                 "amd::Memory object does not have the right ptr");
+  }
+
+  return parent ? vaddr_base_obj : vaddr_sub_obj;
+}
+
+//==================================================================================================
+bool Device::DestroyVirtualBuffer(amd::Memory* vaddr_mem_obj) {
+
+  // Argument nullptr check.
+  if (vaddr_mem_obj == nullptr || vaddr_mem_obj->getSvmPtr() == nullptr) {
+    LogPrintfError("Mem obj passed is nullptr, vaddr_mem_obj: %p \n", vaddr_mem_obj);
+    return false;
+  }
+
+  if (vaddr_mem_obj->parent() != nullptr) {
+    // If parent is not nullptr, this is the sub-buffer object.
+    amd::Memory* vaddr_base_obj = amd::MemObjMap::FindVirtualMemObj(vaddr_mem_obj->getSvmPtr());
+    if (vaddr_base_obj == nullptr) {
+      LogPrintfError("Cannot find mem obj for ptr: 0x%x", vaddr_mem_obj->getSvmPtr());
+      return false;
+    }
+    vaddr_base_obj->removeSubBuffer(vaddr_mem_obj);
+  }
+
+  return true;
 }
 
 Device::BlitProgram::~BlitProgram() {
@@ -444,7 +598,7 @@ bool Device::BlitProgram::create(amd::Device* device, const std::string& extraKe
   // Build all kernels
   std::string opt = "-cl-internal-kernel ";
   if (!device->settings().useLightning_) {
-    opt += "-Wf,--force_disable_spir -fno-lib-no-inline -fno-sc-keep-calls ";
+    opt += "-Wf,--force_disable_spir ";
   }
 
   if (!extraOptions.empty()) {
@@ -453,6 +607,14 @@ bool Device::BlitProgram::create(amd::Device* device, const std::string& extraKe
   if (!GPU_DUMP_BLIT_KERNELS) {
     opt += " -fno-enable-dump";
   }
+  if (device->settings().kernel_arg_opt_) {
+    opt += " -Wb,-amdgpu-kernarg-preload-count=8 ";
+  }
+#if defined(__clang__)
+#if __has_feature(address_sanitizer)
+  opt += " -fsanitize=address ";
+#endif
+#endif
   if ((retval = program_->build(devices, opt.c_str(), nullptr, nullptr, GPU_DUMP_BLIT_KERNELS))
       != CL_SUCCESS) {
     DevLogPrintfError("Build failed for Kernel: %s with error code %d\n",
@@ -529,6 +691,7 @@ Device::Device()
       blitProgram_(nullptr),
       context_(nullptr),
       heap_buffer_(nullptr),
+      initial_heap_buffer_(nullptr),
       arena_mem_obj_(nullptr),
       vaCacheAccess_(nullptr),
       vaCacheMap_(nullptr),
@@ -537,22 +700,26 @@ Device::Device()
 }
 
 Device::~Device() {
-  if (vaCacheMap_) {
-    CondLog(vaCacheMap_->size() != 0, "Application didn't unmap all host memory!");
-    delete vaCacheMap_;
-  }
-
   if (heap_buffer_ != nullptr) {
     delete heap_buffer_;
     heap_buffer_ = nullptr;
   }
 
-  delete vaCacheAccess_;
+  if (initial_heap_buffer_ != nullptr) {
+    delete initial_heap_buffer_;
+    initial_heap_buffer_ = nullptr;
+  }
 
   if (arena_mem_obj_ != nullptr) {
     arena_mem_obj_->release();
   }
 
+  if (vaCacheMap_) {
+    CondLog(vaCacheMap_->size() != 0, "Application didn't unmap all host memory!");
+    delete vaCacheMap_;
+  }
+
+  delete vaCacheAccess_;
   delete settings_;
   delete[] info_.extensions_;
 }
@@ -583,10 +750,23 @@ bool Device::ValidateHsail() {
   return true;
 }
 
+size_t GetMaxStackSize(const std::string& procName) {
+  if (procName.find("gfx9") != std::string::npos || procName.find("gfx8")
+                                                    != std::string::npos) {
+    return kMaxStackSize9X;
+  } else if (procName.find("gfx11") != std::string::npos || procName.find("gfx10")
+                                                            != std::string::npos) {
+    return kMaxStackSize11X;
+  } else {
+    return kMaxStackSize12X;
+  }
+}
+
 bool Device::create(const Isa &isa) {
   assert(!vaCacheAccess_ && !vaCacheMap_);
   isa_ = &isa;
-  vaCacheAccess_ = new amd::Monitor("VA Cache Ops Lock", true);
+  // VA Cache Ops Lock
+  vaCacheAccess_ = new amd::Monitor(true);
   if (nullptr == vaCacheAccess_) {
     return false;
   }
@@ -598,6 +778,7 @@ bool Device::create(const Isa &isa) {
   if (!amd::IS_HIP) {
     stack_size_ = 16 * Ki;
   }
+  maxStackSize_ = GetMaxStackSize(isa_->processorName());
   return true;
 }
 
@@ -773,15 +954,10 @@ bool Device::disableP2P(amd::Device* ptrDev) {
 }
 
 bool Device::UpdateStackSize(uint64_t stackSize) {
-  // Amount of space used by each wave is in units of 256 dwords. 
-  // As per COMPUTE_TMPRING_SIZE.WAVE_SIZE 24:12
-  // The field size supports a range of 0->(2M-256) dwords per wave64. 
-  // Per lane this works out to 131056 bytes or 128K - 16
-  uint64_t kStackSize = ((128 * Ki) - 16); 
-  if (stackSize > kStackSize) {
+  if (stackSize > maxStackSize_) {
     return false;
   }
-  stack_size_ = stackSize;
+  stack_size_ = amd::alignUp(stackSize, 16);
   return true;
 }
 
@@ -844,20 +1020,16 @@ bool Device::IpcCreate(void* dev_ptr, size_t* mem_size, void* handle, size_t* me
   }
 
   // Calculate the memory offset from the original base ptr
-  *mem_offset = reinterpret_cast<address>(dev_ptr) - reinterpret_cast<address>(orig_dev_ptr);
+  *mem_offset = reinterpret_cast<address>(dev_ptr)
+                - reinterpret_cast<address>(orig_dev_ptr)
+                + amd_mem_obj->getOffset();
+
   *mem_size = amd_mem_obj->getSize();
 
-  // Check if the dev_ptr is greater than memory allocated
-  if (*mem_offset > *mem_size) {
-    DevLogPrintfError(
-        "Memory offset: %u cannot be greater than size of original memory allocated: %u", *mem_size,
-        *mem_offset);
-    return false;
-  }
   auto dev_mem = static_cast<device::Memory*>(amd_mem_obj->getDeviceMemory(*this));
   auto result = dev_mem->ExportHandle(handle);
 
-  return true;
+  return result;
 }
 
 // ================================================================================================
@@ -882,9 +1054,6 @@ bool Device::IpcAttach(const void* handle, size_t mem_size, size_t mem_offset, u
   if (mem_obj_exist == nullptr) {
     // Add the original mem_ptr to the MemObjMap with newly created amd_mem_obj
     amd::MemObjMap::AddMemObj(amd_mem_obj->getSvmPtr(), amd_mem_obj);
-
-    // Make sure the mem_offset doesnt overflow the allocated memory
-    guarantee((mem_offset < mem_size), "IPC mem offset greater than allocated size");
   } else {
     amd_mem_obj->release();
     amd_mem_obj = mem_obj_exist;
@@ -927,9 +1096,24 @@ bool Device::IpcDetach(void* dev_ptr) const {
   return true;
 }
 
+std::vector<amd::CommandQueue*> Device::getActiveQueues() {
+  amd::ScopedLock lock(activeQueuesLock_);
+  for (auto it = activeQueues.begin(); it != activeQueues.end();) {
+    if ((*it)->referenceCount() == 0) {
+      // It is being terminated in HostQueue::terminate().
+      // We should not wait for commands in a queue being terminated.
+      it = activeQueues.erase(it);
+    } else {
+      // In case the queue will be destroyed in Stream::Destroy().
+      (*it)->retain();
+      ++it;
+    }
+  }
+  return std::vector<amd::CommandQueue*>(activeQueues.begin(), activeQueues.end());
+}
 }  // namespace amd
 
-namespace device {
+namespace amd::device {
 
 Settings::Settings() : value_(0) {
   assert((ClExtTotal < (8 * sizeof(extensions_))) && "Too many extensions!");
@@ -938,27 +1122,19 @@ Settings::Settings() : value_(0) {
   customHostAllocator_ = false;
   waitCommand_ = AMD_OCL_WAIT_COMMAND;
   supportDepthsRGB_ = false;
-  commandQueues_ = 200;  //!< Field value set to maximum number
-                         //!< concurrent Virtual GPUs for default
-
-  overrideLclSet = (!flagIsDefault(GPU_MAX_WORKGROUP_SIZE)) ? 1 : 0;
-  overrideLclSet |=
-      (!flagIsDefault(GPU_MAX_WORKGROUP_SIZE_2D_X) || !flagIsDefault(GPU_MAX_WORKGROUP_SIZE_2D_Y))
-      ? 2
-      : 0;
-  overrideLclSet |=
-      (!flagIsDefault(GPU_MAX_WORKGROUP_SIZE_3D_X) || !flagIsDefault(GPU_MAX_WORKGROUP_SIZE_3D_Y) ||
-       !flagIsDefault(GPU_MAX_WORKGROUP_SIZE_3D_Z))
-      ? 4
-      : 0;
-
   fenceScopeAgent_ = AMD_OPT_FLUSH;
-  if (amd::IS_HIP) {
-    if (flagIsDefault(GPU_SINGLE_ALLOC_PERCENT)) {
-      GPU_SINGLE_ALLOC_PERCENT = 100;
-    }
 
+  // Amend certain flags for OpenCL
+  if (!amd::IS_HIP) {
+    if (flagIsDefault(GPU_SINGLE_ALLOC_PERCENT)) {
+      GPU_SINGLE_ALLOC_PERCENT = 85;
+    }
+    if (flagIsDefault(GPU_FORCE_BLIT_COPY_SIZE)) {
+      GPU_FORCE_BLIT_COPY_SIZE = 0;
+    }
   }
+
+  gwsInitSupported_ = true;
 }
 
 void Memory::saveMapInfo(const void* mapAddress, const amd::Coord3D origin,
@@ -1406,4 +1582,4 @@ bool ClBinary::isSPIRV() const {
   return false;
 }
 
-}  // namespace device
+}  // namespace amd::device
