@@ -57,6 +57,7 @@
 #include <set>
 #include <unordered_set>
 #include <utility>
+#include <shared_mutex>
 
 namespace amd {
 class Command;
@@ -79,6 +80,7 @@ class PerfCounterCommand;
 class ReleaseObjectCommand;
 class StallQueueCommand;
 class Marker;
+class AccumulateCommand;
 class ThreadTraceCommand;
 class ThreadTraceMemObjectsCommand;
 class SignalCommand;
@@ -90,6 +92,7 @@ class SvmMapMemoryCommand;
 class SvmUnmapMemoryCommand;
 class SvmPrefetchAsyncCommand;
 class StreamOperationCommand;
+class BatchMemoryOperationCommand;
 class VirtualMapCommand;
 class ExternalSemaphoreCmd;
 class Isa;
@@ -123,6 +126,16 @@ enum MemRangeAttribute : uint32_t {
 constexpr int CpuDeviceId = static_cast<int>(-1);
 constexpr int InvalidDeviceId = static_cast<int>(-2);
 
+// Max scratch size is device dependent.
+constexpr size_t kWave32 = 32;
+constexpr size_t kWave64 = 64;
+constexpr size_t kScratchBits12X = 18;
+constexpr size_t kScratchBits9X = 15;
+constexpr size_t kCompilerRequired = 64;
+constexpr size_t kMaxStackSize12X = (((1 << kScratchBits12X) - 1) * 256 / kWave32) - kCompilerRequired;
+constexpr size_t kMaxStackSize11X = (((1 << kScratchBits9X) - 1) * 256 / kWave32) - kCompilerRequired;
+constexpr size_t kMaxStackSize9X = (((1 << kScratchBits9X) - 1) * 256 / kWave64) - kCompilerRequired;
+
 enum class ExternalSemaphoreHandleType : uint32_t {
   OpaqueFd = 1,        // Handle is an opaque file descriptor
   OpaqueWin32 = 2,     // Handle is an opaque shared NT handle
@@ -243,7 +256,7 @@ inline void WriteAqlArgAt(unsigned char* dst, //!< The write pointer to the buff
   *(reinterpret_cast<T*>(dst + offset)) = src;
 }
 
-namespace device {
+namespace amd::device {
 class ClBinary;
 class BlitManager;
 class Program;
@@ -344,6 +357,9 @@ struct Info : public amd::EmbeddedObject {
   //! Max width of 2D image in pixels.
   size_t image2DMaxWidth_;
 
+  //! Max width of 2DA image in pixels.
+  size_t image2DAMaxWidth_[2];
+
   //! Max height of 2D image in pixels.
   size_t image2DMaxHeight_;
 
@@ -408,10 +424,13 @@ struct Info : public amd::EmbeddedObject {
   //  the device implement error correction.
   uint32_t errorCorrectionSupport_;
 
-  //! true if the device and the host have a unified memory subsystem and
-  //  is false otherwise.
+  //! true if the device and the host have a unified memory and is false otherwise.
   uint32_t hostUnifiedMemory_;
 
+  //! true if the device and the host have a unified memory management subsystem and
+  //  is false otherwise.
+  uint32_t iommuv2_;
+
   //! Describes the resolution of device timer.
   size_t profilingTimerResolution_;
 
@@ -480,6 +499,9 @@ struct Info : public amd::EmbeddedObject {
   //! Returns max number of pixels for a 1D image
   size_t image1DMaxWidth_;
 
+  //! Returns max number of pixels for a 1DA image
+  size_t image1DAMaxWidth_;
+
   //! Returns max number of pixels for a 1D image created from a buffer object
   size_t imageMaxBufferSize_;
 
@@ -632,15 +654,29 @@ struct Info : public amd::EmbeddedObject {
   uint32_t vgprsPerSimd_;
   uint32_t vgprAllocGranularity_;
   uint32_t numSDMAengines_; //!< Number of available SDMA engines
+
+  uint32_t luidLowPart_;        //!< Luid low 4 bytes, available in Windows only
+  uint32_t luidHighPart_;       //!< Luid high 4 bytes, available in Windows only
+  uint32_t luidDeviceNodeMask_; //!< Luid node mask
 };
 
 //! Device settings
 class Settings : public amd::HeapObject {
  public:
+
+  enum KernelArgImpl {
+    HostKernelArgs = 0,       //!< Kernel Arguments are put into host memory
+    DeviceKernelArgs,         //!< Device memory kernel arguments with no memory
+                              //!< ordering workaround (e.g. XGMI)
+    DeviceKernelArgsReadback, //!< Device memory kernel arguments with kernel
+                              //!< argument readback workaround
+    DeviceKernelArgsHDP       //!< Device memory kernel arguments with kernel
+                              //!< argument readback plus HDP flush workaround.
+  };
+
   uint64_t extensions_;  //!< Supported OCL extensions
   union {
     struct {
-      uint overrideLclSet : 3;        //!< Bit mask to override the local size
       uint apuSystem_ : 1;            //!< Device is APU system with shared memory
       uint supportRA_ : 1;            //!< Support RA channel order format
       uint waitCommand_ : 1;          //!< Enables a wait for every submitted command
@@ -660,14 +696,14 @@ class Settings : public amd::HeapObject {
       uint enableCoopMultiDeviceGroups_ : 1; //!< Enable cooperative groups multi device
       uint fenceScopeAgent_ : 1;      //!< Enable fence scope agent in AQL dispatch packet
       uint rocr_backend_ : 1;         //!< Device uses ROCr backend for submissions
-      uint reserved_ : 11;
+      uint gwsInitSupported_:1;       //!< Check if GWS is supported on this machine.
+      uint kernel_arg_opt_: 1;        //!< Enables kernel arg optimization for blit kernels
+      uint kernel_arg_impl_ : 2;      //!< Kernel argument implementation
+      uint reserved_ : 7;
     };
     uint value_;
   };
 
-  uint commandQueues_;  //!< Field value for maximum number
-                        //!< concurrent Virtual GPUs for each backend
-
   //! Default constructor
   Settings();
 
@@ -683,6 +719,8 @@ class Settings : public amd::HeapObject {
   //! Enable the specified extension
   void enableExtension(uint name) { extensions_ |= static_cast<uint64_t>(1) << name; }
 
+  size_t stagedXferSize_ = 0;     //!< Staged buffer size
+
  private:
   //! Disable copy constructor
   Settings(const Settings&);
@@ -702,6 +740,13 @@ class Memory : public amd::HeapObject {
     CpuWriteOnly = 0x00000002,  //!< Lock for CPU write only operation
   };
 
+  //! Memory Access flags at device level
+  enum MemAccess {
+    kMemAccessNone = 0,        //! No Access
+    kMemAccessRead = 1,        //! Read Access
+    kMemAccessReadWrite = 3    //! Read and Write Access
+  };
+
   union SyncFlags {
     struct {
       uint skipParent_ : 1;  //!< Skip parent synchronization
@@ -742,11 +787,14 @@ class Memory : public amd::HeapObject {
   Memory(amd::Memory& owner)
       : flags_(0), owner_(&owner), version_(0), mapMemory_(NULL), indirectMapCount_(0) {
     size_ = owner.getSize();
+    memAccess_ = MemAccess::kMemAccessNone;
   }
 
   //! Constructor (no owner), always eager allocation.
   Memory(size_t size)
-      : flags_(0), owner_(NULL), version_(0), mapMemory_(NULL), indirectMapCount_(0), size_(size) {}
+      : flags_(0), owner_(NULL), version_(0), mapMemory_(NULL), indirectMapCount_(0), size_(size) {
+    memAccess_ = MemAccess::kMemAccessNone;
+  }
 
   enum GLResourceOP {
     GLDecompressResource = 0,  // orders the GL driver to decompress any depth-stencil or MSAA
@@ -897,6 +945,12 @@ class Memory : public amd::HeapObject {
     }
   }
 
+  //! Set access to the memory in this device.
+  void SetAccess(MemAccess memAccess) { memAccess_ = memAccess; }
+
+  //! Get current access of the memory in device.
+  MemAccess GetAccess() const { return memAccess_; }
+
  protected:
   enum Flags {
     HostMemoryDirectAccess = 0x00000001,  //!< GPU has direct access to the host memory
@@ -910,6 +964,8 @@ class Memory : public amd::HeapObject {
   };
   uint flags_;  //!< Memory object flags
 
+  MemAccess memAccess_; //!< Memory Access flag
+
   amd::Memory* owner_;  //!< The Memory instance that we cache,
                         //!< or NULL if we're device-private workspace.
 
@@ -1223,7 +1279,7 @@ class VirtualDevice : public amd::HeapObject {
   VirtualDevice(amd::Device& device)
     : device_(device)
     , blitMgr_(NULL)
-    , execution_("Virtual device execution lock", true)
+    , execution_(true) /* Virtual device execution lock */
     , index_(0) {}
 
   //! Destroy this virtual device.
@@ -1231,7 +1287,7 @@ class VirtualDevice : public amd::HeapObject {
 
   //! Return the physical device for this virtual device.
   const amd::Device& device() const { return device_(); }
-
+  virtual uint64_t getQueueID() = 0;
   virtual void submitReadMemory(amd::ReadMemoryCommand& cmd) = 0;
   virtual void submitWriteMemory(amd::WriteMemoryCommand& cmd) = 0;
   virtual void submitCopyMemory(amd::CopyMemoryCommand& cmd) = 0;
@@ -1241,6 +1297,7 @@ class VirtualDevice : public amd::HeapObject {
   virtual void submitKernel(amd::NDRangeKernelCommand& command) = 0;
   virtual void submitNativeFn(amd::NativeFnCommand& cmd) = 0;
   virtual void submitMarker(amd::Marker& cmd) = 0;
+  virtual void submitAccumulate(amd::AccumulateCommand& cmd) = 0;
   virtual void submitExternalSemaphoreCmd(amd::ExternalSemaphoreCmd& cmd) = 0;
   virtual void submitFillMemory(amd::FillMemoryCommand& cmd) = 0;
   virtual void submitMigrateMemObjects(amd::MigrateMemObjectsCommand& cmd) = 0;
@@ -1262,9 +1319,13 @@ class VirtualDevice : public amd::HeapObject {
     ShouldNotReachHere();
   }
   virtual void submitStreamOperation(amd::StreamOperationCommand& cmd) { ShouldNotReachHere(); }
+  virtual void submitBatchMemoryOperation(amd::BatchMemoryOperationCommand& cmd) {
+    ShouldNotReachHere();
+  }
   virtual void submitVirtualMap(amd::VirtualMapCommand& cmd) { ShouldNotReachHere(); }
 
   virtual address allocKernelArguments(size_t size, size_t alignment) { return nullptr; }
+  virtual void ReleaseHwQueue() {}
 
   //! Get the blit manager object
   device::BlitManager& blitMgr() const { return *blitMgr_; }
@@ -1278,14 +1339,17 @@ class VirtualDevice : public amd::HeapObject {
   //! Returns true if device has active wait setting
   bool ActiveWait() const;
 
-  //! Returns the status of queue handler callback
-  virtual bool isHandlerPending() const = 0;
-
   //! Returns fence state of the VirtualGPU
   virtual bool isFenceDirty() const = 0;
+  //! Init hidden heap for device memory allocations
+  virtual void HiddenHeapInit() = 0;
+  //! Dispatch captured AQL packet
+  virtual bool dispatchAqlPacket(uint8_t* aqlpacket,
+                                 const std::string& kernelName,
+                                 amd::AccumulateCommand* vcmd = nullptr) = 0;
 
-  //! Resets fence state of the VirtualGPU
-  virtual void resetFenceDirty() = 0;
+  //! Returns the number of outstanding HSA async handlers
+  std::atomic<uint64_t>& QueuedAsyncHandlers() const { return queued_async_handlers_; }
 
  private:
   //! Disable default copy constructor
@@ -1302,36 +1366,42 @@ class VirtualDevice : public amd::HeapObject {
 
   amd::Monitor execution_;  //!< Lock to serialise access to all device objects
   uint index_;              //!< The virtual device unique index
+  mutable std::atomic<uint64_t> queued_async_handlers_ = 0; //!< Outstanding HSA async handlers
 };
 
-}  // namespace device
+}  // namespace amd::device
 
 namespace amd {
 
 //! MemoryObject map lookup  class
 class MemObjMap : public AllStatic {
  public:
-  static size_t size();  //!< obtain the size of the container
-  static void AddMemObj(const void* k,
-                        amd::Memory* v);  //!< add the host mem pointer and buffer in the container
-  static void RemoveMemObj(const void* k);  //!< Remove an entry of mem object from the container
-  static amd::Memory* FindMemObj(
-      const void* k,              //!< find the mem object based on the input pointer
-      size_t* offset = nullptr);  //!< Offset in the memory location
+  //!< add the host mem pointer and buffer in the container
+  static void AddMemObj(const void* k, amd::Memory* v);
+
+  //!< Remove an entry of mem object from the container
+  static void RemoveMemObj(const void* k);
+
+  //!< Find the mem object based on the input pointer, outputs the offset
+  static amd::Memory* FindMemObj( const void* k, size_t* offset = nullptr);
   static void UpdateAccess(amd::Device *peerDev);
-  static void Purge(amd::Device* dev); //!< Purge all user allocated memories on the given device
+  //!< Purge all user allocated memories on the given device
+  static void Purge(amd::Device* dev);
+  //!< Same as AddMemObj but for virtual addressing
+  static void AddVirtualMemObj(const void* k, amd::Memory* v);
+
+  //!< Same as RemoveMemObj but for virtual addressing
+  static void RemoveVirtualMemObj(const void* k);
+  //!< Same as FindMemObj but for virtual addressing
+  static amd::Memory* FindVirtualMemObj(const void* k);
 
-  static void AddVirtualMemObj(const void* k,
-                               amd::Memory* v);  //!< Same as AddMemObj but for virtual addressing
-  static void RemoveVirtualMemObj(const void* k);  //!< Same as RemoveMemObj but for virtual addressing
-  static amd::Memory* FindVirtualMemObj(
-      const void* k);  //!< Same as FindMemObj but for virtual addressing
  private:
-  static std::map<uintptr_t, amd::Memory*>
-      MemObjMap_;                      //!< the mem object<->hostptr information container
-  static std::map<uintptr_t, amd::Memory*>
-      VirtualMemObjMap_;               //!< the virtual mem object<->hostptr information container
-  static amd::Monitor AllocatedLock_;  //!< amd monitor locker
+  //!< the mem object<->hostptr information container
+  static std::map<uintptr_t, amd::Memory*> MemObjMap_;
+  //!< the virtual mem object<->hostptr information container
+  static std::map<uintptr_t, amd::Memory*> VirtualMemObjMap_;
+  //!< Shared read/write lock
+  static std::shared_mutex AllocatedLock_;
 };
 
 /// @brief Instruction Set Architecture properties.
@@ -1383,7 +1453,7 @@ class Isa {
 
   /// @returns If the ROCm runtime supports the ISA.
   bool runtimeRocSupported() const {
-    if (!IS_HIP && !ROC_ENABLE_PRE_VEGA && (versionMajor_ == 8)) {
+    if (!IS_HIP && (versionMajor_ == 8)) {
       return false;
     }
     return runtimeRocSupported_;
@@ -1528,7 +1598,6 @@ class Isa {
   uint32_t memChannelBankWidth_;   //!< Memory channel bank width.
   uint32_t localMemSizePerCU_;     //!< Local memory size per CU.
   uint32_t localMemBanks_;         //!< Number of banks of local memory.
-
 }; // class Isa
 
 /*! \addtogroup Runtime
@@ -1581,6 +1650,13 @@ class Device : public RuntimeObject {
     kCacheStateSystem = 2
   } CacheState;
 
+  //<! Enum describing the access permissions of Virtual memory
+  enum class VmmAccess {
+    kNone           = 0x0,
+    kReadOnly       = 0x1,
+    kReadWrite      = 0x3
+  };
+
   typedef std::pair<LinkAttribute, int32_t /* value */> LinkAttrType;
 
   static constexpr size_t kP2PStagingSize = 4 * Mi;
@@ -1588,6 +1664,10 @@ class Device : public RuntimeObject {
   static constexpr size_t kMGInfoSizePerDevice = kMGSyncDataSize + sizeof(MGSyncInfo);
   static constexpr size_t kSGInfoSize = kMGSyncDataSize;
 
+  // Max Scratch size is based on ISA and thus per device.
+  // Def value is as per GFX9 being the least among supported devices.
+  size_t maxStackSize_ = kMaxStackSize9X;
+
   typedef std::list<CommandQueue*> CommandQueues;
 
   struct BlitProgram : public amd::HeapObject {
@@ -1737,6 +1817,17 @@ class Device : public RuntimeObject {
     return NULL;
   }
 
+  virtual void* deviceLocalAlloc(size_t size, bool atomics = false,
+                                 bool pseudo_fine_grain = false, bool contiguous = false) const {
+    ShouldNotCallThis();
+    return NULL;
+  }
+
+  virtual bool isXgmi() const {
+    ShouldNotCallThis();
+    return false;
+  }
+
   virtual bool deviceAllowAccess(void* dst) const {
     ShouldNotCallThis();
     return true;
@@ -1766,6 +1857,34 @@ class Device : public RuntimeObject {
    */
   virtual void svmFree(void* ptr) const = 0;
 
+  /**
+   * Validatates Virtual Address range between parent and sub-buffer.
+   *
+   * @param vaddr_base_obj Parent/base object of the virtual address.
+   * @param vaddr_sub_obj Sub Buffer object of the virtual address.
+   */
+  static bool ValidateVirtualAddressRange(amd::Memory* vaddr_base_obj, amd::Memory* vaddr_sub_obj);
+
+  /**
+   * Abstracts the Virtual Buffer creation and memobj/virtual memobj add/delete logic.
+   *
+   * @param device_context Context the virtual buffer should be created.
+   * @param vptr virtual ptr to store in the buffer object.
+   * @param size Size of the buffer
+   * @param deviceId deviceId
+   * @param parent base_obj or sub_obj
+   * @param ForceAlloc force_alloc
+   */
+  amd::Memory* CreateVirtualBuffer(Context& device_context, void* vptr, size_t size,
+                                           int deviceId, bool parent, bool kForceAlloc = false);
+
+  /**
+   * Deletes Virtual Buffer and creates memob
+   *
+   * @param vaddr_mem_obj amd::Memory object of parent/sub buffer.
+   */
+  bool DestroyVirtualBuffer(amd::Memory* vaddr_mem_obj);
+
   /**
    * Reserve a VA range with no backing store
    *
@@ -1775,12 +1894,62 @@ class Device : public RuntimeObject {
    */
   virtual void* virtualAlloc(void* addr, size_t size, size_t alignment) = 0;
 
+  /**
+   * Set Access permisions for a virtual memory object.
+   *
+   * @param va_addr Virtual Address ptr
+   * @param va_size Virtual Address Size
+   * @param access_flags Access permissions
+   * @param count Number of access permissions
+   */
+  virtual bool SetMemAccess(void* va_addr, size_t va_size, VmmAccess access_flags) = 0;
+
+  /**
+   * Get Access permisions for a virtual memory object.
+   *
+   * @param va_addr Virtual Address ptr
+   * @param access_flags_ptr Access permissions to be filled
+   */
+  virtual bool GetMemAccess(void* va_addr, VmmAccess* access_flags_ptr) const = 0;
+
+  /**
+   * Validate Access permisions for a virtual memory object.
+   *
+   * @param va_addr Virtual Address ptr
+   * @param access_flags_ptr Access permissions to be filled
+   */
+  virtual bool ValidateMemAccess(amd::Memory& mem, bool read_write) const = 0;
+
   /**
    * Free a VA range
    *
    * @param addr Start address of the range
    */
-  virtual void virtualFree(void* addr) = 0;
+  virtual bool virtualFree(void* addr) = 0;
+
+  /**
+   * Export Shareable VMM Handle to FD
+   *
+   * @param amd_mem_obj amd::Memory obj which holds the hsa_handle.
+   * @param flags any flags to be passed
+   * @param shareableHandle exported handle, points to fdesc.
+   */
+  virtual bool ExportShareableVMMHandle(amd::Memory& amd_mem_obj, int flags,
+                                        void* shareableHandle) {
+    ShouldNotCallThis();
+    return false;
+  }
+
+  /**
+   * Import FD from Shareable VMM Handle
+   *
+   * @param osHandle os handle/fdesc/void*
+   * @param amd_mem_obj amd_mem_obj with hsa_handle/memory_obj.
+   */
+  virtual amd::Memory* ImportShareableVMMHandle(void* osHandle) {
+    ShouldNotCallThis();
+    return nullptr;
+  }
 
   /**
    * @return True if the device successfully applied the SVM attributes in HMM for device memory
@@ -1815,13 +1984,9 @@ class Device : public RuntimeObject {
   // Returns the status of HW event, associated with amd::Event
   virtual bool IsHwEventReady(
       const amd::Event& event,    //!< AMD event for HW status validation
-      bool wait = false) const {  //!< If true then forces the event completion
-    return false;
-  };
-
-  // Returns the status of HW event, associated with amd::Event
-  virtual bool IsHwEventReadyForcedWait(
-      const amd::Event& event) const {  //!< AMD event for HW status validation
+      bool wait = false,          //!< If true then forces the event completion
+      uint32_t hip_event_flags = 0 //!< flags associated with the event. 0 = hipEventDefault
+      ) const {
     return false;
   };
 
@@ -1946,6 +2111,21 @@ class Device : public RuntimeObject {
     return false;
   }
 
+  //! Returns the queues that have at least one submitted command
+  std::vector<amd::CommandQueue*> getActiveQueues();
+
+  //! Adds the queue to the set of active command queues
+  void addToActiveQueues(amd::CommandQueue* commandQueue) {
+     amd::ScopedLock lock(activeQueuesLock_);
+     activeQueues.insert(commandQueue);
+  }
+
+  //! Removes the queue from the set of active command queues
+  void removeFromActiveQueues(amd::CommandQueue* commandQueue) {
+    amd::ScopedLock lock(activeQueuesLock_);
+    activeQueues.erase(commandQueue);
+  }
+
   // Notifies device about context destroy
   virtual void ContextDestroy() {}
 
@@ -1957,6 +2137,10 @@ class Device : public RuntimeObject {
   virtual amd::Memory* GetArenaMemObj(const void* ptr, size_t& offset, size_t size = 0) {
     return nullptr;
   }
+
+  //! Returns stack size set for the device
+  size_t MaxStackSize() const { return maxStackSize_; }
+
 #if defined(__clang__)
 #if __has_feature(address_sanitizer)
   virtual device::UriLocator* createUriLocator() const = 0;
@@ -1985,13 +2169,17 @@ class Device : public RuntimeObject {
   static Memory* p2p_stage_;          //!< Staging resources
   std::vector<Device*> enabled_p2p_devices_;  //!< List of user enabled P2P devices for this device
 
-  std::once_flag heap_initialized_; //!< Heap buffer initialization flag
+  std::once_flag heap_initialized_;  //!< Heap buffer initialization flag
+  std::once_flag heap_allocated_;    //!< Heap buffer allocation flag
+
   device::Memory* heap_buffer_;     //!< Preallocated heap buffer for memory allocations on device
 
   amd::Memory* arena_mem_obj_;      //!< Arena memory object
   uint64_t stack_size_{1024};       //!< Device stack size
   device::Memory* initial_heap_buffer_;   //!< Initial heap buffer
   uint64_t initial_heap_size_{HIP_INITIAL_DM_SIZE};  //!< Initial device heap size
+  amd::Monitor activeQueuesLock_ {}; //!< Guards access to the activeQueues set
+  std::unordered_set<amd::CommandQueue*> activeQueues; //!< The set of active queues
  private:
   const Isa *isa_;                //!< Device isa
   bool IsTypeMatching(cl_device_type type, bool offlineDevices);
