@@ -22,7 +22,7 @@
 #include "device/pal/palvirtual.hpp"
 #include <array>
 
-namespace pal {
+namespace amd::pal {
 
 static constexpr std::array<PCIndexSelect, 50> blockIdToIndexSelect = {{
     PCIndexSelect::None,                     // CPF
@@ -662,7 +662,7 @@ static constexpr std::array<std::pair<int, int>, 140> gfx10BlockIdPal = {{
     {0x2B, 0},     // GCR       - 136
     {0x2C, 0},     // PH        - 137
     {0x2D, 0},     // UTCL1     - 138
-    {0x31, 0},     // SqWgp     - 139  For Navi3x
+    {0x31, 0},     // SqWgp     - 139
 }};
 
 void PerfCounter::convertInfo() {
@@ -691,6 +691,7 @@ void PerfCounter::convertInfo() {
     case Pal::GfxIpLevel::GfxIp10_1:
     case Pal::GfxIpLevel::GfxIp10_3:
     case Pal::GfxIpLevel::GfxIp11_0:
+    case Pal::GfxIpLevel::GfxIp11_5:
       if (info_.blockIndex_ < gfx10BlockIdPal.size()) {
         auto p = gfx10BlockIdPal[info_.blockIndex_];
         info_.blockIndex_ = std::get<0>(p);
@@ -805,4 +806,4 @@ uint64_t PerfCounter::getInfo(uint64_t infoType) const {
   return 0;
 }
 
-}  // namespace pal
+}  // namespace amd::pal
