@@ -36,7 +36,7 @@
 #include "device/pal/paldefs.hpp"
 #include "device/pal/palsettings.hpp"
 #include "device/pal/palappprofile.hpp"
-#include "device/pal/palgpuopen.hpp"
+#include "device/pal/palcapturemgr.hpp"
 #include "device/pal/palsignal.hpp"
 #include "acl.h"
 #include "memory"
@@ -54,7 +54,7 @@
  */
 
 //! PAL Device Implementation
-namespace pal {
+namespace amd::pal {
 
 //! A nil device object
 class NullDevice : public amd::Device {
@@ -145,7 +145,28 @@ class NullDevice : public amd::Device {
   }
   virtual void svmFree(void* ptr) const { return; }
   virtual void* virtualAlloc(void* addr, size_t size, size_t alignment) { return nullptr; };
-  virtual void virtualFree(void* addr) { };
+  virtual bool virtualFree(void* addr) { return true; }
+
+  virtual bool SetMemAccess(void* va_addr, size_t va_size, VmmAccess access_flags) {
+    return true;
+  }
+
+  virtual bool GetMemAccess(void* va_addr, VmmAccess* access_flags_ptr) const {
+    return true;
+  }
+
+  virtual bool ValidateMemAccess(amd::Memory& mem, bool read_write) const {
+    return true;
+  }
+
+  virtual bool ExportShareableVMMHandle(amd::Memory& amd_mem_obj, int flags,
+                                        void* shareableHandle) {
+    return false;
+  }
+
+  virtual amd::Memory* ImportShareableVMMHandle(void* osHandle) {
+    return nullptr;
+  }
 
   virtual bool importExtSemaphore(void** extSemaphore,const amd::Os::FileDesc& handle,
                                   amd::ExternalSemaphoreHandleType sem_handle_type) override {
@@ -178,7 +199,8 @@ class NullDevice : public amd::Device {
                       const Pal::GpuMemoryHeapProperties heaps[Pal::GpuHeapCount],
                       size_t maxTextureSize,         //!< Maximum texture size supported in HW
                       uint numComputeRings,          //!< Number of compute rings
-                      uint numExclusiveComputeRings  //!< Number of exclusive compute rings
+                      uint numExclusiveComputeRings, //!< Number of exclusive compute rings
+                      Pal::IDevice* pal_device       //!< PAL device for which info is filled
   );
 };
 
@@ -231,11 +253,18 @@ class Sampler : public device::Sampler {
 class Device : public NullDevice {
  public:
   struct QueueRecycleInfo : public amd::HeapObject {
-    int counter_;                 //!< Lock usage counter
-    Pal::EngineType engineType_;  //!< Engine type
-    uint32_t index_;              //!< HW queue index for scratch buffer access
-    amd::Monitor queue_lock_;     //!< Queue lock for access
-    QueueRecycleInfo() : counter_(1), engineType_(Pal::EngineTypeCompute), index_(0) {}
+    int counter_;                   //!< Lock usage counter
+    Pal::EngineType engineType_;    //!< Engine type
+    uint32_t index_;                //!< HW queue index for scratch buffer access
+    amd::Monitor queue_lock_;       //!< Queue lock for access
+    AqlPacketMgmt aql_packet_mgmt_; //!< AQL packets management class for debugger support
+    QueueRecycleInfo() : counter_(1), engineType_(Pal::EngineTypeCompute), index_(0),
+          queue_lock_(true) /* Queue lock for sharing */ {}
+
+    //! Returns the aql packet list
+    uintptr_t AqlPacketList() const {
+      return reinterpret_cast<uintptr_t>(&aql_packet_mgmt_.aql_packets_);
+    }
   };
 
   //! Locks any access to the virtual GPUs
@@ -530,8 +559,18 @@ class Device : public NullDevice {
   //! SVM free
   virtual void svmFree(void* ptr) const;
 
+  //! Virtual address space allocation(reservation)
   virtual void* virtualAlloc(void* addr, size_t size, size_t alignment);
-  virtual void virtualFree(void* addr);
+  virtual bool virtualFree(void* addr);
+
+  //! Set/Get memory access set by the app
+  virtual bool SetMemAccess(void* va_addr, size_t va_size, VmmAccess access_flags);
+  virtual bool GetMemAccess(void* va_addr, VmmAccess* access_flags_ptr) const;
+  virtual bool ValidateMemAccess(amd::Memory& mem, bool read_write) const;
+
+  virtual bool ExportShareableVMMHandle(amd::Memory& amd_mem_obj, int flags, void* shareableHandle);
+
+  virtual amd::Memory* ImportShareableVMMHandle(void* osHandle);
 
   //! Returns SRD manger object
   SrdManager& srds() const { return *srdManager_; }
@@ -548,7 +587,8 @@ class Device : public NullDevice {
   //! Allow access for peer device
   bool deviceAllowAccess(void* dst) const;
 
-  RgpCaptureMgr* rgpCaptureMgr() const { return rgpCaptureMgr_; }
+  //! Returns a handle to the capture manager (RGP or UberTrace)
+  ICaptureMgr* captureMgr() const { return captureMgr_; }
 
   //! Update free memory for OCL extension
   void updateAllocedMemory(Pal::GpuHeap heap,  //!< PAL GPU heap for update
@@ -636,6 +676,17 @@ class Device : public NullDevice {
   //! Allocates hidden heap for device memory allocations
   void HiddenHeapAlloc(const VirtualGPU& gpu);
 
+  const Pal::GpuMemoryHeapProperties& GetGpuHeapInvisible() const {
+    return heaps_[Pal::GpuHeapInvisible];
+  }
+
+  Pal::gpusize TotalAlloc() const {
+    Pal::gpusize local = allocedMem[Pal::GpuHeapLocal] - resourceCache().persistentCacheSize();
+    Pal::gpusize invisible = allocedMem[Pal::GpuHeapInvisible] - resourceCache().lclCacheSize();
+    Pal::gpusize total_alloced = local + invisible;
+    return total_alloced;
+  }
+
  private:
   static void PAL_STDCALL PalDeveloperCallback(void* pPrivateData, const Pal::uint32 deviceIndex,
                                                Pal::Developer::CallbackType type, void* pCbData);
@@ -701,17 +752,18 @@ class Device : public NullDevice {
   Memory* globalScratchBuf_;             //!< Global scratch buffer
   SrdManager* srdManager_;               //!< SRD manager object
   static AppProfile appProfile_;         //!< application profile
-  mutable bool freeCPUMem_;              //!< flag to mark GPU free SVM CPU mem
+  mutable bool freeCPUMem_ = false;      //!< flag to mark GPU free SVM CPU mem
   Pal::DeviceProperties properties_;     //!< PAL device properties
   Pal::IDevice* device_;                 //!< PAL device object
   mutable std::atomic<Pal::gpusize>
       allocedMem[Pal::GpuHeap::GpuHeapCount];    //!< Free memory counter
   std::unordered_set<Resource*>* resourceList_;  //!< Active resource list
-  RgpCaptureMgr* rgpCaptureMgr_;                 //!< RGP capture manager
+  ICaptureMgr* captureMgr_;                      //!< RGP/UberTrace capture manager
   Pal::GpuMemoryHeapProperties
       heaps_[Pal::GpuHeapCount];         //!< Information about heaps, returned from PAL
   std::map<Pal::IQueue*, QueueRecycleInfo*> queue_pool_;  //!< Pool of PAL queues for recycling
+  amd::Program* trap_handler_ = nullptr;  //!< Trap handler program for debugger setup
 };
 
-/*@}*/  // namespace pal
-}  // namespace pal
+/*@}*/  // namespace amd::pal
+}  // namespace amd::pal
