@@ -73,6 +73,16 @@ static PFNGLXRESOURCEDETACHAMD glxResourceReleaseAMD = nullptr;
 static PFNGLXRESOURCEDETACHAMD glXResourceDetachAMD = nullptr;
 static PFNGLXGETCONTEXTMVPUINFOAMD glXGetContextMVPUInfoAMD = nullptr;
 #else
+typedef PROC(WINAPI* PFNWGLGETPROCADDRESS)(LPCSTR name);
+typedef HGLRC(WINAPI* PFNWGLGETCURRENTCONTEXT)(void);
+typedef HGLRC(WINAPI* PFNWGLCREATECONTEXT)(HDC hdc);
+typedef BOOL(WINAPI* PFNWGLDELETECONTEXT)(HGLRC hglrc);
+typedef BOOL(WINAPI* PFNWGLMAKECURRENT)(HDC hdc, HGLRC hglrc);
+static PFNWGLGETPROCADDRESS pfnWglGetProcAddress = nullptr;
+static PFNWGLGETCURRENTCONTEXT pfnWglGetCurrentContext = nullptr;
+static PFNWGLCREATECONTEXT pfnWglCreateContext = nullptr;
+static PFNWGLDELETECONTEXT pfnWglDeleteContext = nullptr;
+static PFNWGLMAKECURRENT pfnWglMakeCurrent = nullptr;
 static PFNWGLBEGINCLINTEROPAMD wglBeginCLInteropAMD = nullptr;
 static PFNWGLENDCLINTEROPAMD wglEndCLInteropAMD = nullptr;
 static PFNWGLRESOURCEATTACHAMD wglResourceAttachAMD = nullptr;
@@ -80,9 +90,10 @@ static PFNWGLRESOURCEDETACHAMD wglResourceAcquireAMD = nullptr;
 static PFNWGLRESOURCEDETACHAMD wglResourceReleaseAMD = nullptr;
 static PFNWGLRESOURCEDETACHAMD wglResourceDetachAMD = nullptr;
 static PFNWGLGETCONTEXTGPUINFOAMD wglGetContextGPUInfoAMD = nullptr;
+bool gGlFuncInit = false;
 #endif
 
-namespace pal {
+namespace amd::pal {
 
 //
 /// GSL Surface Formats as per defined in cmSurfFmtEnum enum in
@@ -702,35 +713,55 @@ bool Device::initGLInteropPrivateExt(void* GLplatformContext, void* GLdeviceCont
     return false;
   }
 #else
-  if (!wglBeginCLInteropAMD || !wglEndCLInteropAMD || !wglResourceAttachAMD ||
-      !wglResourceDetachAMD || !wglGetContextGPUInfoAMD) {
+  if (!gGlFuncInit) {
+    HMODULE h = static_cast<HMODULE>(amd::Os::loadLibrary("opengl32.dll"));
+    if (nullptr != h) {
+      pfnWglGetProcAddress =
+          reinterpret_cast<PFNWGLGETPROCADDRESS>(GetProcAddress(h, "wglGetProcAddress"));
+      pfnWglGetCurrentContext =
+          reinterpret_cast<PFNWGLGETCURRENTCONTEXT>(GetProcAddress(h, "wglGetCurrentContext"));
+      pfnWglCreateContext =
+        reinterpret_cast<PFNWGLCREATECONTEXT>(GetProcAddress(h, "wglCreateContext"));
+      pfnWglDeleteContext =
+        reinterpret_cast<PFNWGLDELETECONTEXT>(GetProcAddress(h, "wglDeleteContext"));
+      pfnWglMakeCurrent =
+        reinterpret_cast<PFNWGLMAKECURRENT>(GetProcAddress(h, "wglMakeCurrent"));
+      if (!pfnWglGetProcAddress || !pfnWglGetCurrentContext || !pfnWglCreateContext ||
+          !pfnWglDeleteContext || !pfnWglMakeCurrent) {
+        LogError("Couldn't obtain WGL context API");
+        return false;
+      }
+    }
     HGLRC fakeRC = nullptr;
-
-    if (!wglGetCurrentContext()) {
-      fakeRC = wglCreateContext((HDC)GLdeviceContext);
-      wglMakeCurrent((HDC)GLdeviceContext, fakeRC);
+    // @note: For unclear reason runtime can't get AMD API entry points without a context...
+    if (!pfnWglGetCurrentContext()) {
+      fakeRC = pfnWglCreateContext((HDC)GLdeviceContext);
+      pfnWglMakeCurrent((HDC)GLdeviceContext, fakeRC);
     }
-
     wglBeginCLInteropAMD =
-        (PFNWGLBEGINCLINTEROPAMD)wglGetProcAddress("wglBeginCLInteroperabilityAMD");
-    wglEndCLInteropAMD = (PFNWGLENDCLINTEROPAMD)wglGetProcAddress("wglEndCLInteroperabilityAMD");
-    wglResourceAttachAMD = (PFNWGLRESOURCEATTACHAMD)wglGetProcAddress("wglResourceAttachAMD");
-    wglResourceAcquireAMD = (PFNWGLRESOURCEDETACHAMD)wglGetProcAddress("wglResourceAcquireAMD");
-    wglResourceReleaseAMD = (PFNWGLRESOURCEDETACHAMD)wglGetProcAddress("wglResourceReleaseAMD");
-    wglResourceDetachAMD = (PFNWGLRESOURCEDETACHAMD)wglGetProcAddress("wglResourceDetachAMD");
+      (PFNWGLBEGINCLINTEROPAMD)pfnWglGetProcAddress("wglBeginCLInteroperabilityAMD");
+    wglEndCLInteropAMD =
+       (PFNWGLENDCLINTEROPAMD)pfnWglGetProcAddress("wglEndCLInteroperabilityAMD");
+    wglResourceAttachAMD = (PFNWGLRESOURCEATTACHAMD)pfnWglGetProcAddress("wglResourceAttachAMD");
+    wglResourceAcquireAMD =
+      (PFNWGLRESOURCEDETACHAMD)pfnWglGetProcAddress("wglResourceAcquireAMD");
+    wglResourceReleaseAMD =
+      (PFNWGLRESOURCEDETACHAMD)pfnWglGetProcAddress("wglResourceReleaseAMD");
+    wglResourceDetachAMD = (PFNWGLRESOURCEDETACHAMD)pfnWglGetProcAddress("wglResourceDetachAMD");
     wglGetContextGPUInfoAMD =
-        (PFNWGLGETCONTEXTGPUINFOAMD)wglGetProcAddress("wglGetContextGPUInfoAMD");
-
+      (PFNWGLGETCONTEXTGPUINFOAMD)pfnWglGetProcAddress("wglGetContextGPUInfoAMD");
     if (fakeRC) {
-      wglMakeCurrent(nullptr, nullptr);
-      wglDeleteContext(fakeRC);
+      pfnWglMakeCurrent(nullptr, nullptr);
+      pfnWglDeleteContext(fakeRC);
     }
   }
   if (!wglBeginCLInteropAMD || !wglEndCLInteropAMD || !wglResourceAttachAMD ||
       !wglResourceDetachAMD || !wglGetContextGPUInfoAMD) {
+    LogError("Couldn't obtain WGL AMD API");
     return false;
   }
 #endif
+  gGlFuncInit = true;
   return true;
 }
 
@@ -776,13 +807,6 @@ bool Device::glAssociate(void* GLplatformContext, void* GLdeviceContext) const {
       !glCanInterop(GLplatformContext, GLdeviceContext)) {
     return false;
   }
-
-/*
-    if (m_adp->pAsicInfo->svmFineGrainSystem)
-    {
-        flags = GL_INTEROP_SVM;
-    }
-*/
 #ifdef ATI_OS_LINUX
   GLXContext ctx = (GLXContext)GLplatformContext;
   return (glXBeginCLInteropAMD(ctx, 0)) ? true : false;
@@ -793,12 +817,6 @@ bool Device::glAssociate(void* GLplatformContext, void* GLdeviceContext) const {
 }
 
 bool Device::glDissociate(void* GLplatformContext, void* GLdeviceContext) const {
-/*
-    if (m_adp->pAsicInfo->svmFineGrainSystem)
-    {
-        flags = GL_INTEROP_SVM;
-    }
-*/
 #ifdef ATI_OS_LINUX
   GLXContext ctx = (GLXContext)GLplatformContext;
   if (glXEndCLInteropAMD == nullptr) {
@@ -897,7 +915,7 @@ bool Device::resGLAcquire(void* GLplatformContext, void* mbResHandle, uint type)
   GLXContext ctx = (GLXContext)GLplatformContext;
   return (glxResourceAcquireAMD(ctx, &hRes)) ? true : false;
 #else
-  HGLRC hRC = wglGetCurrentContext();
+  HGLRC hRC = pfnWglGetCurrentContext();
   //! @todo A temporary workaround for MT issue in conformance fence_sync
   if (0 == hRC) {
     return true;
@@ -919,7 +937,7 @@ bool Device::resGLRelease(void* GLplatformContext, void* mbResHandle, uint type)
   return (glxResourceReleaseAMD(ctx, &hRes)) ? true : false;
 #else
   // Make the call into the GL driver only if the application GL context is current
-  HGLRC hRC = wglGetCurrentContext();
+  HGLRC hRC = pfnWglGetCurrentContext();
   //! @todo A temporary workaround for MT issue in conformance fence_sync
   if (0 == hRC) {
     return true;
@@ -943,4 +961,4 @@ bool Device::resGLFree(void* GLplatformContext, void* mbResHandle, uint type) co
 #endif
 }
 
-}  // namespace pal
+}  // namespace amd::pal
