@@ -41,7 +41,7 @@
 #include "protocols/rgpServer.h"
 #include "protocols/driverControlServer.h"
 
-namespace pal {
+namespace amd::pal {
 // ================================================================================================
 RgpCaptureMgr::RgpCaptureMgr(Pal::IPlatform* platform, const Device& device)
     : device_(device),
@@ -874,7 +874,9 @@ void RgpCaptureMgr::WriteBarrierStartMarker(const VirtualGPU* gpu,
     RgpSqttMarkerBarrierStart marker = {};
 
     marker.identifier = RgpSqttMarkerIdentifierBarrierStart;
-    marker.cbId = trace_.begin_queue_->queue(MainEngine).cmdBufId();
+    if (trace_.begin_queue_ != nullptr) {
+      marker.cbId = trace_.begin_queue_->queue(MainEngine).cmdBufId();
+    }
     marker.dword02 = data.reason;
     marker.internal = true;
 
@@ -898,8 +900,9 @@ void RgpCaptureMgr::WriteBarrierEndMarker(const VirtualGPU* gpu,
     RgpSqttMarkerBarrierEnd marker = {};
 
     marker.identifier = RgpSqttMarkerIdentifierBarrierEnd;
-    marker.cbId = trace_.begin_queue_->queue(MainEngine).cmdBufId();
-
+    if (trace_.begin_queue_ != nullptr) {
+      marker.cbId = trace_.begin_queue_->queue(MainEngine).cmdBufId();
+    }
     marker.waitOnEopTs = operations.pipelineStalls.eopTsBottomOfPipe;
     marker.vsPartialFlush = operations.pipelineStalls.vsPartialFlush;
     marker.psPartialFlush = operations.pipelineStalls.psPartialFlush;
@@ -966,6 +969,6 @@ void RgpCaptureMgr::WriteComputeBindMarker(const VirtualGPU* gpu, uint64_t api_h
   WriteMarker(gpu, &marker, sizeof(marker));
 }
 
-}  // namespace pal
+}  // namespace amd::pal
 
 #endif // PAL_GPUOPEN_OCL
