@@ -34,7 +34,7 @@
 #include <ctime>
 #include <algorithm>
 
-namespace pal {
+namespace amd::pal {
 
 void HSAILKernel::setWorkGroupInfo(const uint32_t privateSegmentSize,
                                    const uint32_t groupSegmentSize, const uint16_t numSGPRs,
@@ -65,6 +65,8 @@ void HSAILKernel::setWorkGroupInfo(const uint32_t privateSegmentSize,
     workGroupInfo_.availableVGPRs_ = 256;
     workGroupInfo_.preferredSizeMultiple_ = workGroupInfo_.wavefrontPerSIMD_ = 64;
   }
+  workGroupInfo_.maxDynamicSharedSizeBytes_ = static_cast<int>(workGroupInfo_.availableLDSSize_ -
+                                                               workGroupInfo_.localMemSize_);
 }
 
 bool HSAILKernel::setKernelCode(amd::hsa::loader::Symbol* sym, amd_kernel_code_t* akc) {
@@ -224,8 +226,6 @@ bool HSAILKernel::init() {
     return false;
   }
 
-  waveLimiter_.enable();
-
   size_t sizeOfWorkGroupSizeHint = sizeof(workGroupInfo_.compileSizeHint_);
   error = amd::Hsail::QueryInfo(palNullDevice().compiler(), prog().binaryElf(), RT_WORK_GROUP_SIZE_HINT,
                                 openClKernelName.c_str(), workGroupInfo_.compileSizeHint_,
@@ -265,11 +265,10 @@ const HSAILProgram& HSAILKernel::prog() const {
 }
 
 // ================================================================================================
-hsa_kernel_dispatch_packet_t* HSAILKernel::loadArguments(VirtualGPU& gpu, const amd::Kernel& kernel,
-                                                         const amd::NDRangeContainer& sizes,
-                                                         const_address params,
-                                                         size_t ldsAddress, uint64_t vmDefQueue,
-                                                         uint64_t* vmParentWrap) const {
+hsa_kernel_dispatch_packet_t* HSAILKernel::loadArguments(
+    VirtualGPU& gpu, const amd::Kernel& kernel, const amd::NDRangeContainer& sizes,
+    const_address params, size_t ldsAddress, uint64_t vmDefQueue,
+    uint64_t* vmParentWrap, uint32_t* aql_index) const {
   // Provide private and local heap addresses
   static constexpr uint AddressShift = LP64_SWITCH(0, 32);
   const_address parameters = params;
@@ -340,8 +339,7 @@ hsa_kernel_dispatch_packet_t* HSAILKernel::loadArguments(VirtualGPU& gpu, const
         if (amd::IS_HIP) {
           uintptr_t buffer = reinterpret_cast<uintptr_t>(gpu.getOrCreateHostcallBuffer());
           if (!buffer) {
-            ClPrint(amd::LOG_ERROR, amd::LOG_KERN,
-                    "Kernel expects a hostcall buffer, but none found");
+            LogError("Kernel expects a hostcall buffer, but none found");
           }
           assert(it.size_ == sizeof(buffer) && "check the sizes");
           WriteAqlArgAt(hidden_arguments, buffer, it.size_, it.offset_);
@@ -442,6 +440,10 @@ hsa_kernel_dispatch_packet_t* HSAILKernel::loadArguments(VirtualGPU& gpu, const
         // @note: It's not a real AQL queue
         WriteAqlArgAt(hidden_arguments, gpu.hsaQueueMem()->vmAddress(), it.size_, it.offset_);
         break;
+      case amd::KernelParameterDescriptor::HiddenDynamicLdsSize:
+        WriteAqlArgAt(hidden_arguments, static_cast<uint32_t>(ldsAddress - ldsSize()),
+               it.size_, it.offset_);
+        break;
     }
   }
 
@@ -451,9 +453,7 @@ hsa_kernel_dispatch_packet_t* HSAILKernel::loadArguments(VirtualGPU& gpu, const
                                            signature.paramsSize()));
   }
 
-  // hsa_kernel_dispatch_packet_t disp;
-  hsa_kernel_dispatch_packet_t* hsaDisp =
-      reinterpret_cast<hsa_kernel_dispatch_packet_t*>(gpu.cb(0)->SysMemCopy());
+  hsa_kernel_dispatch_packet_t* hsaDisp = gpu.GetAqlPacketSlot(aql_index);
 
   constexpr uint16_t kDispatchPacketHeader =
       (HSA_PACKET_TYPE_KERNEL_DISPATCH << HSA_PACKET_HEADER_TYPE) |
@@ -471,7 +471,7 @@ hsa_kernel_dispatch_packet_t* HSAILKernel::loadArguments(VirtualGPU& gpu, const
   hsaDisp->grid_size_x = global[0];
   hsaDisp->grid_size_y = (sizes.dimensions() > 1) ? global[1] : 1;
   hsaDisp->grid_size_z = (sizes.dimensions() > 2) ? global[2] : 1;
-  hsaDisp->reserved2 = 0;
+  hsaDisp->reserved0 = 0;
 
   // Initialize kernel ISA and execution buffer requirements
   hsaDisp->private_segment_size = spillSegSize();
@@ -552,6 +552,12 @@ bool LightningKernel::postLoad() {
   if (workGroupInfo_.size_ == 0) {
     return false;
   }
+  if ((workGroupInfo_.usedStackSize_ & 0x1) == 0x1) {
+    workGroupInfo_.scratchRegs_ =
+        std::max<uint32_t>(device().StackSize(), workGroupInfo_.scratchRegs_ * sizeof(uint32_t)) ;
+    workGroupInfo_.scratchRegs_ = amd::alignUp(workGroupInfo_.scratchRegs_, 16) / sizeof(uint32_t);
+    workGroupInfo_.privateMemSize_ = workGroupInfo_.scratchRegs_ * sizeof(uint32_t);
+  }
 
   // handle the printf metadata if any
   std::vector<std::string> printfStr;
@@ -567,4 +573,4 @@ bool LightningKernel::postLoad() {
 }
 #endif  // defined(USE_COMGR_LIBRARY)
 
-}  // namespace pal
+}  // namespace amd::pal
