@@ -41,7 +41,7 @@
 #include <sstream>
 #include <iostream>
 
-namespace pal {
+namespace amd::pal {
 
 Memory::Memory(const Device& gpuDev, amd::Memory& owner, size_t size)
     : device::Memory(owner), Resource(gpuDev, size), pinnedMemory_(nullptr), parent_(nullptr) {
@@ -109,16 +109,25 @@ bool Memory::create(Resource::MemoryType memType, Resource::CreateParams* params
     // Assume that allocations will be placed into visible heap when ReBar is enabled
     // Only enable this assumption for small size local buffers
     constexpr size_t kLargeAlloc = (1ull << 27);
-    if (!amd::IS_HIP && (memType == Local) && desc().buffer_ && (size() < kLargeAlloc) && dev().info().largeBar_) {
+    if (!amd::IS_HIP && (memType == Local) && desc().buffer_ && (size() < kLargeAlloc) &&
+        dev().info().largeBar_) {
       memType = Persistent;
     }
+
+    if (dev().settings().apuSystem_) {
+      const Pal::GpuMemoryHeapProperties& invisibleHeap = dev().GetGpuHeapInvisible();
+      Pal::gpusize totalAlloc = dev().TotalAlloc();
+      if (invisibleHeap.logicalSize > 0 && memType == Local &&
+          (totalAlloc > (invisibleHeap.logicalSize * 0.75))) {
+        memType = RemoteUSWC;
+      }
+    }
     // Create a resource in PAL
     result = Resource::create(memType, params, forceLinear);
     if (!result) {
       size_t freeMemory[2];
       // if requested memory is greater than available then exit the loop
       dev().globalFreeMemory(freeMemory);
-
       // Local to Persistent
       if (memoryType() == Local) {
         // For dgpu freeMemory[0] reports a sum of visible+invisible fb
@@ -181,6 +190,7 @@ bool Memory::create(Resource::MemoryType memType, Resource::CreateParams* params
         flags_ |= SubMemoryObject | (parent_->flags_ & HostMemoryDirectAccess);
         break;
       }
+      case Resource::ImageExternalBuffer:
       case Resource::ImageBuffer: {
         Resource::ImageBufferParams* view = reinterpret_cast<Resource::ImageBufferParams*>(params);
         parent_ = reinterpret_cast<const Memory*>(view->memory_);
@@ -198,7 +208,7 @@ bool Memory::create(Resource::MemoryType memType, Resource::CreateParams* params
     }
     if (memRef() != nullptr) {
       ClPrint(amd::LOG_DEBUG, amd::LOG_RESOURCE,
-              "Alloc: %llx bytes, ptr[%p-%p], obj[%p-%p]",
+              "Alloc: %zx bytes, ptr[%llx-%llx], obj[%llx-%llx]",
               size(),
               vmAddress(),
               vmAddress() + size(),
@@ -356,6 +366,7 @@ bool Memory::createInterop() {
     vkRes.owner_ = owner();
     memType = Resource::VkInterop;
     vkRes.handle_ = ext_memory->Handle();
+    vkRes.name_ = ext_memory->Name();
     vkRes.type_ = Resource::InteropTypeless;
     vkRes.nt_handle_ =
       ((ext_memory->Type() != amd::ExternalMemory::HandleType::OpaqueFd) &&
@@ -1069,7 +1080,7 @@ void* Image::allocMapTarget(const amd::Coord3D& origin, const amd::Coord3D& regi
   //! runtime can't use it directly,
   //! because CAL volume map doesn't work properly.
   //! @todo arrays can be added for persistent lock with some CAL changes
-  else if((isPersistentDirectMap(mapFlags & CL_MAP_WRITE) && (getMapCount() == 0)) ||
+  else if ((isPersistentDirectMap(mapFlags & CL_MAP_WRITE) && (getMapCount() == 0)) ||
           isPersistentMapped()) {
     if (nullptr == map(nullptr)) {
       useRemoteResource = true;
@@ -1200,4 +1211,4 @@ bool Image::ValidateMemory(Resource::MemoryType memType) {
   return true;
 }
 
-}  // namespace pal
+}  // namespace amd::pal
