@@ -35,7 +35,7 @@
 #include <iterator>
 #include <sstream>
 
-namespace pal {
+namespace amd::pal {
 
 Segment::Segment() : gpuAccess_(nullptr), cpuAccess_(nullptr), cpuMem_(nullptr) {}
 
@@ -139,10 +139,10 @@ bool Segment::alloc(HSAILProgram& prog, amdgpu_hsa_elf_segment_t segment, size_t
 
 void Segment::copy(size_t offset, const void* src, size_t size) {
   if (cpuAccess_ != nullptr) {
-    amd::Os::fastMemcpy(cpuAddress(offset), src, size);
+    std::memcpy(cpuAddress(offset), src, size);
   } else {
     if (cpuMem_ != nullptr) {
-      amd::Os::fastMemcpy(cpuAddress(offset), src, size);
+      std::memcpy(cpuAddress(offset), src, size);
     }
     amd::ScopedLock k(gpuAccess_->dev().xferMgr().lockXfer());
     VirtualGPU& gpu = *gpuAccess_->dev().xferQueue();
@@ -242,6 +242,14 @@ inline static std::vector<std::string> splitSpaceSeparatedString(char* str) {
   return vec;
 }
 
+inline static std::string GetUriFromMemoryAddress(const void* memory, size_t size) {
+  int pid = amd::Os::getProcessId();
+  std::ostringstream uri_stream;
+  uri_stream << "memory://" << pid << "#offset=0x" << std::hex <<
+    reinterpret_cast<uintptr_t>(memory) << std::dec << "&size=" << size;
+  return uri_stream.str();
+}
+
 bool HSAILProgram::createKernels(void* binary, size_t binSize, bool useUniformWorkGroupSize,
                                  bool internalKernel) {
 #if defined(WITH_COMPILER_LIB)
@@ -256,12 +264,13 @@ bool HSAILProgram::createKernels(void* binary, size_t binSize, bool useUniformWo
   code_object.handle = reinterpret_cast<uint64_t>(binary);
 
   hsa_agent_t agent = {amd::Device::toHandle(&(device()))};
-  hsa_status_t status = executable_->LoadCodeObject(agent, code_object, nullptr);
+  auto uri = GetUriFromMemoryAddress(binary, binSize);
+  hsa_status_t status = executable_->LoadCodeObject(agent, code_object, nullptr, uri);
   if (status != HSA_STATUS_SUCCESS) {
     buildLog_ += "Error: AMD HSA Code Object loading failed.\n";
     return false;
   }
-  status = executable_->Freeze(nullptr);
+  status = loader_->FreezeExecutable(executable_, nullptr);
   if (status != HSA_STATUS_SUCCESS) {
     buildLog_ += "Error: AMD HSA Code Object freeze failed.\n";
     return false;
@@ -269,7 +278,7 @@ bool HSAILProgram::createKernels(void* binary, size_t binSize, bool useUniformWo
 
   size_t kernelNamesSize = 0;
   acl_error errorCode = amd::Hsail::QueryInfo(palNullDevice().compiler(), binaryElf_,
-    RT_KERNEL_NAMES, nullptr, nullptr, &kernelNamesSize);
+                                              RT_KERNEL_NAMES, nullptr, nullptr, &kernelNamesSize);
   if (errorCode != ACL_SUCCESS) {
     buildLog_ += "Error: Querying of kernel names size from the binary failed.\n";
     return false;
@@ -277,7 +286,7 @@ bool HSAILProgram::createKernels(void* binary, size_t binSize, bool useUniformWo
   if (kernelNamesSize > 0) {
     std::vector<char> kernelNames(kernelNamesSize);
     errorCode = amd::Hsail::QueryInfo(palNullDevice().compiler(), binaryElf_, RT_KERNEL_NAMES,
-      nullptr, kernelNames.data(), &kernelNamesSize);
+                                      nullptr, kernelNames.data(), &kernelNamesSize);
     if (errorCode != ACL_SUCCESS) {
       buildLog_ += "Error: Querying of kernel names from the binary failed.\n";
       return false;
@@ -559,7 +568,7 @@ void* PALHSALoaderContext::SegmentAlloc(amdgpu_hsa_elf_segment_t segment, hsa_ag
 bool PALHSALoaderContext::SegmentCopy(amdgpu_hsa_elf_segment_t segment, hsa_agent_t agent,
                                       void* dst, size_t offset, const void* src, size_t size) {
   if (program_->isNull()) {
-    amd::Os::fastMemcpy(reinterpret_cast<address>(dst) + offset, src, size);
+    std::memcpy(reinterpret_cast<address>(dst) + offset, src, size);
     return true;
   }
   Segment* s = reinterpret_cast<Segment*>(dst);
@@ -691,7 +700,7 @@ hsa_status_t PALHSALoaderContext::SamplerDestroy(hsa_agent_t agent,
 
 static hsa_status_t GetKernelNamesCallback(hsa_executable_t hExec, hsa_executable_symbol_t hSymbol,
                                            void* data) {
-  auto symbol = Symbol::Object(hSymbol);
+  auto symbol = amd::hsa::loader::Symbol::Object(hSymbol);
   auto symbolNameList = reinterpret_cast<std::vector<std::string>*>(data);
 
   hsa_symbol_kind_t type;
@@ -728,30 +737,36 @@ bool LightningProgram::createBinary(amd::option::Options* options) {
   return true;
 }
 
+// ================================================================================================
 bool LightningProgram::createKernels(void* binary, size_t binSize, bool useUniformWorkGroupSize,
                                      bool internalKernel) {
 #if defined(USE_COMGR_LIBRARY)
-   // Find the size of global variables from the binary
-  if (!FindGlobalVarSize(binary, binSize)) {
-    buildLog_ += "Error: Cannot Find Global Var Sizes\n";
-    return false;
-  }
-
-  for (const auto& kernelMeta : kernelMetadataMap_) {
-    auto kernelName = kernelMeta.first;
-    auto kernel = new LightningKernel(kernelName, this, internalKernel);
-    if (kernel == nullptr) {
+  // Skip metadata look-up and kernel creation for assembly and internal kernel.
+  // @note: Runtime compiles only the second level trap handler from assembly
+  if ((owner()->language() != amd::Program::Assembly) || !internal_) {
+    // Find the size of global variables from the binary
+    if (!FindGlobalVarSize(binary, binSize)) {
+      buildLog_ += "Error: Cannot Find Global Var Sizes\n";
       return false;
     }
-    if (!kernel->init()) {
-      buildLog_ += "[ROC][Kernel] Could not get Code Prop Meta Data \n";
-      return false;
+
+    for (const auto& kernelMeta : kernelMetadataMap_) {
+      auto kernelName = kernelMeta.first;
+      auto kernel = new LightningKernel(kernelName, this, internalKernel);
+      if (kernel == nullptr) {
+        return false;
+      }
+      if (!kernel->init()) {
+        buildLog_ += "[ROC][Kernel] Could not get Code Prop Meta Data \n";
+        return false;
+      }
+      kernels()[kernelName] = kernel;
+
+      if (codeObjectVer() < 5) {
+        kernel->setUniformWorkGroupSize(useUniformWorkGroupSize);
+      }
     }
-    kernels()[kernelName] = kernel;
-
-    kernel->setUniformWorkGroupSize(useUniformWorkGroupSize);
   }
-
   executable_ = loader_->CreateExecutable(HSA_PROFILE_FULL, nullptr);
   if (executable_ == nullptr) {
     LogError("Error: Executable for AMD HSA Code Object isn't created.");
@@ -762,14 +777,20 @@ bool LightningProgram::createKernels(void* binary, size_t binSize, bool useUnifo
   code_object.handle = reinterpret_cast<uint64_t>(binary);
 
   hsa_agent_t agent = {amd::Device::toHandle(&(device()))};
-
-  hsa_status_t status = executable_->LoadCodeObject(agent, code_object, nullptr);
+  auto uri = GetUriFromMemoryAddress(binary, binSize);
+  hsa_status_t status = executable_->LoadCodeObject(agent, code_object, nullptr, uri);
   if (status != HSA_STATUS_SUCCESS) {
     LogError("Error: AMD HSA Code Object loading failed.");
     return false;
   }
 
-  status = executable_->Freeze(nullptr);
+  if (isInternal() && (owner()->language() == amd::Program::Assembly)) {
+    // Don't register trap handler with the debugger, since user shouldn't see this kernel
+    status = executable_->Freeze(nullptr);
+    trapHandler_ = true;
+  } else {
+    status = loader_->FreezeExecutable(executable_, nullptr);
+  }
   if (status != HSA_STATUS_SUCCESS) {
     LogError("Error: Freezing the executable failed.");
     return false;
@@ -778,13 +799,14 @@ bool LightningProgram::createKernels(void* binary, size_t binSize, bool useUnifo
   return true;
 }
 
-bool LightningProgram::setKernels(void* binary, size_t binSize,
-                                  amd::Os::FileDesc fdesc, size_t foffset, std::string uri) {
+// ================================================================================================
+bool LightningProgram::setKernels(void* binary, size_t binSize, amd::Os::FileDesc fdesc,
+                                  size_t foffset, std::string uri) {
 #if defined(USE_COMGR_LIBRARY)
-  // Collect the information about compiled binary
-  if (!isNull() && (palDevice().rgpCaptureMgr() != nullptr)) {
-    apiHash_ = palDevice().rgpCaptureMgr()->AddElfBinary(binary, binSize, binary, binSize,
-                                                         codeSegGpu_->iMem(), codeSegGpu_->offset());
+  // Collect the information about compiled binary, except the trap handler
+  if (!isNull() && (palDevice().captureMgr() != nullptr) && !isTrapHandler()) {
+    apiHash_ = palDevice().captureMgr()->AddElfBinary(
+        binary, binSize, binary, binSize, codeSegGpu_->iMem(), codeSegGpu_->offset());
   }
 
   for (auto& kit : kernels()) {
@@ -803,4 +825,15 @@ bool LightningProgram::setKernels(void* binary, size_t binSize,
   return true;
 }
 
-}  // namespace pal
+// ================================================================================================
+uint64_t LightningProgram::GetTrapHandlerAddress() const {
+  uint64_t address = 0;
+  hsa_agent_t agent = {amd::Device::toHandle(&(device()))};
+  auto trap_sym = executable_->GetSymbol("trap_entry", &agent);
+  if (trap_sym != nullptr) {
+    trap_sym->GetInfo(HSA_EXECUTABLE_SYMBOL_INFO_KERNEL_OBJECT, &address);
+  }
+  return address;
+}
+
+}  // namespace amd::pal
