@@ -43,7 +43,7 @@
 #include <iostream>
 #include <cmath>
 
-namespace pal {
+namespace amd::pal {
 
 // ================================================================================================
 Pal::Result GpuMemoryReference::MakeResident() const {
@@ -255,23 +255,26 @@ GpuMemoryReference::~GpuMemoryReference() {
   if (nullptr == iMem()) {
     return;
   }
-  if (gpu_ == nullptr) {
-    Device::ScopedLockVgpus lock(device_);
-    // Release all memory objects on all virtual GPUs
-    for (uint idx = 1; idx < device_.vgpus().size(); ++idx) {
-      device_.vgpus()[idx]->releaseMemory(this);
+  // Memory tracking per queue is disabled if alwaysResident is enabled. Thus, runtime can skip
+  // updating residency state per every queue
+  if (!device_.settings().alwaysResident_) {
+    if (gpu_ == nullptr) {
+      Device::ScopedLockVgpus lock(device_);
+      // Release all memory objects on all virtual GPUs
+      for (uint idx = 1; idx < device_.vgpus().size(); ++idx) {
+        device_.vgpus()[idx]->releaseMemory(this);
+      }
+    } else {
+      amd::ScopedLock l(gpu_->execution());
+      gpu_->releaseMemory(this);
+    }
+    if (device_.vgpus().size() != 0) {
+      assert(device_.vgpus()[0] == device_.xferQueue() && "Wrong transfer queue!");
+      // Lock the transfer queue, since it's not handled by ScopedLockVgpus
+      amd::ScopedLock k(device_.xferMgr().lockXfer());
+      device_.vgpus()[0]->releaseMemory(this);
     }
-  } else {
-    amd::ScopedLock l(gpu_->execution());
-    gpu_->releaseMemory(this);
   }
-  if (device_.vgpus().size() != 0) {
-    assert(device_.vgpus()[0] == device_.xferQueue() && "Wrong transfer queue!");
-    // Lock the transfer queue, since it's not handled by ScopedLockVgpus
-    amd::ScopedLock k(device_.xferMgr().lockXfer());
-    device_.vgpus()[0]->releaseMemory(this);
-  }
-
   // Destroy PAL object if it's not a suballocation
   if (cpuAddress_ != nullptr) {
     iMem()->Unmap();
@@ -613,7 +616,7 @@ bool Resource::CreateImage(CreateParams* params, bool forceLinear) {
     ImgSubresRange.startSubres.arraySlice = imageView->layer_;
     viewOwner_ = imageView->resource_;
     image_ = viewOwner_->image_;
-  } else if (memoryType() == ImageBuffer) {
+  } else if (memoryType() == ImageBuffer || memoryType() == ImageExternalBuffer) {
     ImageBufferParams* imageBuffer = reinterpret_cast<ImageBufferParams*>(params);
     viewOwner_ = imageBuffer->resource_;
   }
@@ -636,9 +639,16 @@ bool Resource::CreateImage(CreateParams* params, bool forceLinear) {
     Pal::ImageTiling tiling = forceLinear ? Pal::ImageTiling::Linear : Pal::ImageTiling::Optimal;
     uint32_t rowPitch = 0;
 
-    if (((memoryType() == Persistent) && dev().settings().linearPersistentImage_) ||
-        (memoryType() == ImageBuffer)) {
+    if (memoryType() == ImageBuffer) {
       tiling = Pal::ImageTiling::Linear;
+    } else if (memoryType() == ImageExternalBuffer) {
+      // We cannot get tiling info from vulkan/d3d driver now. So assume it to be optimal.
+      // When we get tiling info, we can easily update here
+      tiling = Pal::ImageTiling::Optimal;
+      // Pal will infer row pitch. If rowPitch != 0 and Pal inferred row picth isn't equal to
+      // rowPitch, assert(false) will be called
+      rowPitch = 0;
+      offset_ += params->owner_->getOrigin();
     } else if (memoryType() == ImageView) {
       tiling = viewOwner_->image_->GetImageCreateInfo().tiling;
       // Find the new pitch in pixels for the new format
@@ -677,7 +687,8 @@ bool Resource::CreateImage(CreateParams* params, bool forceLinear) {
     // createInfo.priority;
   }
 
-  if ((memoryType() != ImageView) && (memoryType() != ImageBuffer)) {
+  if ((memoryType() != ImageView) && (memoryType() != ImageBuffer) &&
+      (memoryType() != ImageExternalBuffer)) {
     Pal::GpuMemoryCreateInfo createInfo = {};
     createInfo.size = amd::alignUp(req.size, MaxGpuAlignment);
     createInfo.alignment = std::max(req.alignment, MaxGpuAlignment);
@@ -710,7 +721,13 @@ bool Resource::CreateImage(CreateParams* params, bool forceLinear) {
     mapCount_++;
   }
   result = image_->BindGpuMemory(memRef_->gpuMem_, offset_);
+
   if (result != Pal::Result::Success) {
+    LogPrintfError(
+        "BindGpuMemory return %d, offset_=%zu, req.size=%zu,  "
+        "viewOwner_->iMem()->Desc().size=%zu\n",
+        result, offset_, req.size,
+        viewOwner_ && viewOwner_->iMem() ? viewOwner_->iMem()->Desc().size : 0);
     return false;
   }
 
@@ -787,7 +804,21 @@ bool Resource::CreateInterop(CreateParams* params) {
   }
   else if (memoryType() == VkInterop) {
     VkInteropParams* vparams = reinterpret_cast<VkInteropParams*>(params);
-    openInfo.hExternalResource = vparams->handle_;
+    if (vparams->handle_) {
+      openInfo.hExternalResource = vparams->handle_;
+    } else if (vparams->name_) {
+      Pal::ExternalHandleInfo eHandleInfo = {};
+      eHandleInfo.objectType = Pal::ExternalObjectType::Allocation;
+      eHandleInfo.pNtObjectName = reinterpret_cast<const wchar_t*>(vparams->name_);
+      SECURITY_ATTRIBUTES securityAttributes = {};
+      securityAttributes.bInheritHandle = TRUE;
+      eHandleInfo.pSecurityAttributes = &securityAttributes;
+      eHandleInfo.accessFlags = GENERIC_READ | GENERIC_WRITE;
+      if (Pal::Result::Success !=
+          dev().iDev()->OpenExternalHandleFromName(eHandleInfo, &openInfo.hExternalResource)) {
+        return false;
+      }
+    }
     openInfo.flags.ntHandle = vparams->nt_handle_;
   }
 #ifdef ATI_OS_WIN
@@ -1040,7 +1071,8 @@ bool Resource::CreateIpc(CreateParams* params) {
   if (nullptr == memRef_) {
     return false;
   }
-  params->owner_->setSvmPtr(reinterpret_cast<void*>(memRef_->iMem()->Desc().gpuVirtAddr));
+  offset_ += params->owner_->getOffset();
+  params->owner_->setSvmPtr(reinterpret_cast<void*>(memRef_->iMem()->Desc().gpuVirtAddr + offset_));
   return true;
 }
 
@@ -1178,6 +1210,7 @@ bool Resource::CreateSvm(CreateParams* params, Pal::gpusize svmPtr) {
     params->owner_->setSvmPtr(
         reinterpret_cast<void*>(memRef_->iMem()->Desc().gpuVirtAddr + subOffset_));
     offset_ += static_cast<size_t>(subOffset_);
+    params->owner_->setOffset(offset_);
   }
   return true;
 }
@@ -1307,9 +1340,10 @@ bool Resource::create(MemoryType memType, CreateParams* params, bool forceLinear
     createInfo.flags.busAddressable = true;
   } else if (memoryType() == VaRange) {
     createInfo.flags.virtualAlloc = true;
+    createInfo.vaRange = Pal::VaRange::Svm;
     if (params->owner_->getSvmPtr() != nullptr) {
-      createInfo.flags.startVaHintFlag = true;
-      createInfo.startVaHint = reinterpret_cast<Pal::gpusize>(params->owner_->getSvmPtr());
+      createInfo.flags.useReservedGpuVa = true;
+      createInfo.pReservedGpuVaOwner = params->svmBase_->iMem();
     }
   }
 
@@ -1332,7 +1366,16 @@ bool Resource::create(MemoryType memType, CreateParams* params, bool forceLinear
     mapCount_++;
   }
   if (memoryType() == VaRange) {
-    params->owner_->setSvmPtr(reinterpret_cast<void*>(memRef_->iMem()->Desc().gpuVirtAddr));
+    void* gpu_virt_addr = reinterpret_cast<void*>(memRef_->iMem()->Desc().gpuVirtAddr);
+
+    // if svm_ptr was not set, set the gpu_virt_addr
+    if (params->owner_->getSvmPtr() == nullptr) {
+      params->owner_->setSvmPtr(gpu_virt_addr);
+    } else if (params->owner_->getSvmPtr() != gpu_virt_addr) {
+      LogPrintfError("Cannot get different ptrs in va_range, svm_ptr: %p new_svm_ptr:%p \n",
+                     params->owner_->getSvmPtr(), gpu_virt_addr);
+      return false;
+    }
   }
   return true;
 }
@@ -1343,20 +1386,19 @@ void Resource::free() {
     return;
   }
 
-  const bool wait =
-      (memoryType() != ImageView) && (memoryType() != ImageBuffer) && (memoryType() != View);
+  const bool wait = (memoryType() != ImageView) && (memoryType() != ImageBuffer) &&
+      (memoryType() != ImageExternalBuffer) && (memoryType() != View);
 
   // OCL has to wait, even if resource is placed in the cache, since reallocation can occur
   // and resource can be reused on another async queue without a wait on a busy operation
   if (wait) {
     if (memRef_->gpu_ == nullptr) {
-      Device::ScopedLockVgpus lock(dev());
+      amd::ScopedLock l(dev().vgpusAccess());
       // Release all memory objects on all virtual GPUs
       for (uint idx = 1; idx < dev().vgpus().size(); ++idx) {
         dev().vgpus()[idx]->waitForEvent(&events_[idx]);
       }
     } else {
-      amd::ScopedLock l(memRef_->gpu_->execution());
       memRef_->gpu_->waitForEvent(&events_[memRef_->gpu_->index()]);
     }
   } else {
@@ -1444,7 +1486,6 @@ bool Resource::partialMemCopyTo(VirtualGPU& gpu, const amd::Coord3D& srcOrigin,
   GpuEvent event;
   EngineType activeEngineID = gpu.engineID_;
   static const bool waitOnBusyEngine = true;
-  assert(!(desc().cardMemory_ && dstResource.desc().cardMemory_) && "Unsupported configuraiton!");
   uint64_t gpuMemoryOffset = 0;
   uint64_t gpuMemoryRowPitch = 0;
   uint64_t imageOffsetx = 0;
@@ -1481,33 +1522,33 @@ bool Resource::partialMemCopyTo(VirtualGPU& gpu, const amd::Coord3D& srcOrigin,
        (size[0] < dev().settings().cpDmaCopySizeMax_));
   if (cp_dma) {
     // Make sure compute is done before CP DMA start
-    gpu.addBarrier(RgpSqqtBarrierReason::MemDependency);
+    gpu.addBarrier(RgpSqqtBarrierReason::MemDependency, BarrierType::KernelToCopy);
   } else {
     gpu.releaseGpuMemoryFence();
     gpu.engineID_ = SdmaEngine;
+
+    if (gpu.validateSdmaOverlap(*this, dstResource)) {
+      // Note: PAL should insert a NOP into the command buffer for synchronization
+      gpu.addBarrier(RgpSqqtBarrierReason::MemDependency, BarrierType::CopyToCopy);
+    }
   }
 
   // Wait for the resources, since runtime may use async transfers
   wait(gpu, waitOnBusyEngine);
   dstResource.wait(gpu, waitOnBusyEngine);
 
-  if (gpu.validateSdmaOverlap(*this, dstResource)) {
-    // Note: PAL should insert a NOP into the command buffer for synchronization
-    gpu.addBarrier();
-  }
-
   Pal::ImageLayout imgLayout = {};
   gpu.eventBegin(gpu.engineID_);
   gpu.queue(gpu.engineID_).addCmdMemRef(memRef());
   gpu.queue(gpu.engineID_).addCmdMemRef(dstResource.memRef());
   if (desc().buffer_ && !dstResource.desc().buffer_) {
-    uint32_t arraySliceIdx = img2Darray ? dstOrigin[2] : 0;
+    uint32_t arraySliceIdx = img2Darray ? dstOrigin[2] : img1Darray ? dstOrigin[1] : 0;
     Pal::SubresId ImgSubresId = {0, dstResource.desc().baseLevel_, arraySliceIdx};
     Pal::MemoryImageCopyRegion copyRegion = {};
     copyRegion.imageSubres = ImgSubresId;
     copyRegion.imageOffset.x = dstOrigin[0];
-    copyRegion.imageOffset.y = dstOrigin[1];
-    copyRegion.imageOffset.z = dstOrigin[2];
+    copyRegion.imageOffset.y = img1Darray ? 0 : dstOrigin[1];
+    copyRegion.imageOffset.z = (img1Darray || img2Darray) ? 0 : dstOrigin[2];
     copyRegion.imageExtent.width = size[0];
     copyRegion.imageExtent.height = size[1];
     copyRegion.imageExtent.depth = size[2];
@@ -1527,12 +1568,12 @@ bool Resource::partialMemCopyTo(VirtualGPU& gpu, const amd::Coord3D& srcOrigin,
     gpu.iCmd()->CmdCopyMemoryToImage(*iMem(), *dstResource.image_, imgLayout, 1, &copyRegion);
   } else if (!desc().buffer_ && dstResource.desc().buffer_) {
     Pal::MemoryImageCopyRegion copyRegion = {};
-    uint32_t arraySliceIdx = img2Darray ? dstOrigin[2] : 0;
+    uint32_t arraySliceIdx = img2Darray ? srcOrigin[2] : img1Darray ? srcOrigin[1] : 0;
     Pal::SubresId ImgSubresId = {0, desc().baseLevel_, arraySliceIdx};
     copyRegion.imageSubres = ImgSubresId;
     copyRegion.imageOffset.x = srcOrigin[0];
-    copyRegion.imageOffset.y = srcOrigin[1];
-    copyRegion.imageOffset.z = srcOrigin[2];
+    copyRegion.imageOffset.y = img1Darray ? 0 : srcOrigin[1];
+    copyRegion.imageOffset.z = (img1Darray || img2Darray) ? 0 : srcOrigin[2];
     copyRegion.imageExtent.width = size[0];
     copyRegion.imageExtent.height = size[1];
     copyRegion.imageExtent.depth = size[2];
@@ -1595,7 +1636,7 @@ bool Resource::partialMemCopyTo(VirtualGPU& gpu, const amd::Coord3D& srcOrigin,
 
   if (cp_dma) {
     // Make sure CP dma is done
-    gpu.addBarrier(RgpSqqtBarrierReason::MemDependency);
+    gpu.addBarrier(RgpSqqtBarrierReason::MemDependency, BarrierType::CopyToKernel);
   }
 
   gpu.eventEnd(gpu.engineID_, event);
@@ -1667,7 +1708,7 @@ bool Resource::hostWrite(VirtualGPU* gpu, const void* hostPtr, const amd::Coord3
     dst = static_cast<void*>(static_cast<char*>(dst) + origin[0]);
 
     // Copy memory
-    amd::Os::fastMemcpy(dst, hostPtr, copySize);
+    std::memcpy(dst, hostPtr, copySize);
   } else {
     size_t dstOffsBase = origin[0] * elementSize_;
 
@@ -1695,7 +1736,7 @@ bool Resource::hostWrite(VirtualGPU* gpu, const void* hostPtr, const amd::Coord3
       // Copy memory line by line
       for (size_t row = 0; row < size[1]; ++row) {
         // Copy memory
-        amd::Os::fastMemcpy((reinterpret_cast<address>(dst) + dstOffs),
+        std::memcpy((reinterpret_cast<address>(dst) + dstOffs),
                             (reinterpret_cast<const_address>(hostPtr) + srcOffs),
                             size[0] * elementSize_);
 
@@ -1737,7 +1778,7 @@ bool Resource::hostRead(VirtualGPU* gpu, void* hostPtr, const amd::Coord3D& orig
     src = static_cast<void*>(static_cast<char*>(src) + origin[0]);
 
     // Copy memory
-    amd::Os::fastMemcpy(hostPtr, src, copySize);
+    std::memcpy(hostPtr, src, copySize);
   } else {
     size_t srcOffsBase = origin[0] * elementSize_;
 
@@ -1765,9 +1806,9 @@ bool Resource::hostRead(VirtualGPU* gpu, void* hostPtr, const amd::Coord3D& orig
       // Copy memory line by line
       for (size_t row = 0; row < size[1]; ++row) {
         // Copy memory
-        amd::Os::fastMemcpy((reinterpret_cast<address>(hostPtr) + dstOffs),
-                            (reinterpret_cast<const_address>(src) + srcOffs),
-                            size[0] * elementSize_);
+        std::memcpy((reinterpret_cast<address>(hostPtr) + dstOffs),
+                    (reinterpret_cast<const_address>(src) + srcOffs),
+                    size[0] * elementSize_);
 
         srcOffs += desc().pitch_ * elementSize_;
         dstOffs += rowPitch;
@@ -1796,8 +1837,9 @@ void* Resource::gpuMemoryMap(size_t* pitch, uint flags, Pal::IGpuMemory* resourc
       Pal::SubresLayout layout;
       image_->GetSubresourceLayout(ImgSubresId, &layout);
       *pitch = layout.rowPitch / elementSize();
+    } else {
+      *pitch = desc().width_;
     }
-    *pitch = desc().width_;
     if (Pal::Result::Success == resource->Map(&address)) {
       return address;
     } else {
@@ -1810,9 +1852,7 @@ void* Resource::gpuMemoryMap(size_t* pitch, uint flags, Pal::IGpuMemory* resourc
 // ================================================================================================
 void Resource::gpuMemoryUnmap(Pal::IGpuMemory* resource) const {
   if (desc_.cardMemory_ && !isPersistentDirectMap()) {
-    // @todo remove const cast
     Unimplemented();
-    //        const_cast<Device&>(dev()).resUnmapLocal(resource);
   } else {
     Pal::Result result = resource->Unmap();
     if (Pal::Result::Success != result) {
@@ -1905,9 +1945,11 @@ bool Resource::isPersistentDirectMap(bool writeMap) const {
 
   // If direct map is possible, then validate it with the current tiling
   if (directMap && desc().tiled_) {
-    //!@note IOL for Linux doesn't support tiling aperture
-    // and runtime doesn't force linear images in persistent
-    directMap = IS_WINDOWS && !dev().settings().linearPersistentImage_;
+    // Latest HW does have tiling apertures
+    directMap = false;
+  }
+  if (memoryType() == View) {
+    directMap = viewOwner_->isPersistentDirectMap(writeMap);
   }
 
   return directMap;
@@ -2055,6 +2097,10 @@ bool CoarseMemorySubAllocator::CreateChunk(const Pal::IGpuMemory* reserved_va) {
   createInfo.heaps[0] = Pal::GpuHeapInvisible;
   createInfo.heaps[1] = Pal::GpuHeapLocal;
   createInfo.mallPolicy = static_cast<Pal::GpuMemMallPolicy>(device_->settings().mallPolicy_);
+  if (amd::IS_HIP && PAL_HIP_IPC_FLAG) {
+    //set interprocess for IPC memory support
+    createInfo.flags.interprocess = 1;
+  }
   GpuMemoryReference* mem_ref = GpuMemoryReference::Create(*device_, createInfo);
   if (mem_ref != nullptr) {
     // Workaround: some chunk memory are not guaranteed to be resident during initial allocation.
@@ -2184,7 +2230,10 @@ bool ResourceCache::addGpuMemory(Resource::Descriptor* desc, GpuMemoryReference*
   size_t size = ref->iMem()->Desc().size;
 
   // Check if runtime can free suballocation
-  if ((desc->type_ == Resource::Local) && !desc->SVMRes_) {
+  if (desc->type_ == Resource::VaRange) {
+    // We do no sub allocate VA Range.
+    result = true;
+  } else if ((desc->type_ == Resource::Local) && !desc->SVMRes_) {
     result = mem_sub_alloc_local_.Free(&lockCacheOps_, ref, offset);
   } else if ((desc->type_ == Resource::Local) && desc->SVMRes_) {
     result = mem_sub_alloc_coarse_.Free(&lockCacheOps_, ref, offset);
@@ -2222,6 +2271,8 @@ bool ResourceCache::addGpuMemory(Resource::Descriptor* desc, GpuMemoryReference*
       cacheSize_ += size;
       if (desc->type_ == Resource::Local) {
         lclCacheSize_ += size;
+      } else if (desc->type_ == Resource::Persistent) {
+        persistentCacheSize_ += size;
       }
       result = true;
     }
@@ -2239,7 +2290,10 @@ GpuMemoryReference* ResourceCache::findGpuMemory(Resource::Descriptor* desc, Pal
   GpuMemoryReference* ref = nullptr;
 
   // Check if the runtime can suballocate memory
-  if ((desc->type_ == Resource::Local) && !desc->SVMRes_) {
+  if (desc->type_ == Resource::VaRange) {
+    // Do not use suballocator for VA_Range.
+    return nullptr;
+  } else if ((desc->type_ == Resource::Local) && !desc->SVMRes_) {
     ref = mem_sub_alloc_local_.Allocate(size, alignment, reserved_va, offset);
   } else if ((desc->type_ == Resource::Local) && desc->SVMRes_) {
     ref = mem_sub_alloc_coarse_.Allocate(size, alignment, reserved_va, offset);
@@ -2276,6 +2330,8 @@ GpuMemoryReference* ResourceCache::findGpuMemory(Resource::Descriptor* desc, Pal
       cacheSize_ -= sizeRes;
       if (entry->type_ == Resource::Local) {
         lclCacheSize_ -= sizeRes;
+      } else if (entry->type_ == Resource::Persistent) {
+        persistentCacheSize_ -= sizeRes;
       }
       delete it.first;
       // Remove the found etry from the cache
@@ -2310,9 +2366,12 @@ void ResourceCache::removeLast() {
     if (resCache_.size() > 0) {
       entry = resCache_.back();
       resCache_.pop_back();
-      cacheSize_ -= entry.second->iMem()->Desc().size;
+      auto mem_size = entry.second->iMem()->Desc().size;
+      cacheSize_ -= mem_size;
       if (entry.first->type_ == Resource::Local) {
-        lclCacheSize_ -= entry.second->iMem()->Desc().size;
+        lclCacheSize_ -= mem_size;
+      } else if (entry.first->type_ == Resource::Persistent) {
+        persistentCacheSize_ -= mem_size;
       }
       // Delete Descriptor
       delete entry.first;
@@ -2323,4 +2382,4 @@ void ResourceCache::removeLast() {
   entry.second->release();
 }
 
-}  // namespace pal
+}  // namespace amd::pal
