@@ -28,8 +28,8 @@
 #include <atomic>
 #include <unordered_map>
 
-//! \namespace pal PAL Resource Implementation
-namespace pal {
+//! \namespace amd::pal PAL Resource Implementation
+namespace amd::pal {
 
 class Device;
 class VirtualGPU;
@@ -142,6 +142,7 @@ class Resource : public amd::HeapObject {
   struct VkInteropParams : public CreateParams {
     InteropType type_;  //!< Vulkan resource type
     amd::Os::FileDesc handle_;
+    const void* name_;
     bool nt_handle_;
   };
 
@@ -178,7 +179,8 @@ class Resource : public amd::HeapObject {
     P2PAccess,         //!< resource is a shared resource for P2P access
     VkInterop,         //!< resource is a Vulkan memory object
     VaRange,           //!< reousrce is a virtual address range
-    IpcMemory          //!< reousrce is a IPC memory object
+    IpcMemory,         //!< reousrce is a IPC memory object
+    ImageExternalBuffer //!< resource is an image view of an external buffer
   };
 
   //! Resource map flags
@@ -369,8 +371,12 @@ class Resource : public amd::HeapObject {
   //! Retunrs true if memory type matches specified
   bool isMemoryType(MemoryType memType) const;
 
-  //! Returns TRUE if resource was allocated as cacheable
-  bool isCacheable() const { return (isMemoryType(Remote) || isMemoryType(Pinned)) ? true : false; }
+  //! Returns TRUE if resource was allocated as CPU accessible and cacheable
+  bool isCacheable() const { return (isMemoryType(Remote) ||
+                                     isMemoryType(Pinned)) ? true : false; }
+
+  //! Returns TRUE if resource was allocated as CPU visible device memory
+  bool IsPersistent() const { return isMemoryType(Persistent) ? true : false; }
 
   bool glAcquire();
   bool glRelease();
@@ -573,9 +579,10 @@ class ResourceCache : public amd::HeapObject {
  public:
   //! Default constructor
   ResourceCache(Device* device, size_t cacheSizeLimit)
-      : lockCacheOps_("PAL resource cache", true),
+      : lockCacheOps_(true), /* PAL resource cache */
         cacheSize_(0),
         lclCacheSize_(0),
+        persistentCacheSize_(0),
         cacheSizeLimit_(cacheSizeLimit),
         mem_sub_alloc_local_(device),
         mem_sub_alloc_coarse_(device),
@@ -608,6 +615,9 @@ class ResourceCache : public amd::HeapObject {
   //! Returns the size of local memory, stored in the cache
   size_t lclCacheSize() const { return lclCacheSize_; }
 
+  //! Returns the size of persistent memory, stored in the cache
+  size_t persistentCacheSize() const { return persistentCacheSize_; }
+
  private:
   //! Disable copy constructor
   ResourceCache(const ResourceCache&);
@@ -622,6 +632,7 @@ class ResourceCache : public amd::HeapObject {
 
   size_t cacheSize_;            //!< Current cache size in bytes
   size_t lclCacheSize_;         //!< Local memory stored in the cache
+  size_t persistentCacheSize_;  //!< Persistent memory stored in the cache
   const size_t cacheSizeLimit_; //!< Cache size limit in bytes
 
   //! PAL resource cache
@@ -633,5 +644,5 @@ class ResourceCache : public amd::HeapObject {
   FineUncachedMemorySubAllocator mem_sub_alloc_fine_uncached_; //!< Allocator for suballocations in Fine uncached SVM
 };
 
-/*@}*/  // namespace pal
-}  // namespace pal
+/*@}*/  // namespace amd::pal
+}  // namespace amd::pal
