@@ -41,7 +41,7 @@
  */
 
 //! PAL Device Implementation
-namespace pal {
+namespace amd::pal {
 
 class Device;
 class Kernel;
@@ -53,6 +53,27 @@ class BlitManager;
 class ThreadTrace;
 class HSAILKernel;
 
+struct AqlPacketMgmt : public amd::EmbeddedObject {
+  static constexpr uint32_t kAqlPacketsListSize = 4 * Ki;
+  AqlPacketMgmt()
+      : packet_index_(0) {
+    memset(aql_vgpus_, 0, sizeof(aql_vgpus_));
+  }
+
+  hsa_kernel_dispatch_packet_t aql_packets_[kAqlPacketsListSize];  //!< The list of AQL packets
+  GpuEvent aql_events_[kAqlPacketsListSize];    //!< The list of gpu for each AQL packet
+  VirtualGPU* aql_vgpus_[kAqlPacketsListSize];  //!< The list of vgpus which had submissions
+  std::atomic<uint64_t> packet_index_;          //!< The active packet slot index
+};
+
+ enum class BarrierType : uint8_t {
+   KernelToKernel = 0,
+   KernelToCopy,
+   CopyToKernel,
+   CopyToCopy,
+   FlushL2
+};
+
 //! Virtual GPU
 class VirtualGPU : public device::VirtualDevice {
  public:
@@ -77,8 +98,7 @@ class VirtualGPU : public device::VirtualDevice {
                          uint max_command_buffers  //!< Number of allocated command buffers
     );
 
-    Queue(VirtualGPU& gpu, Pal::IDevice* iDev, uint64_t residency_limit,
-          uint max_command_buffers)
+    Queue(VirtualGPU& gpu, Pal::IDevice* iDev, uint64_t residency_limit, uint max_command_buffers)
         : lock_(nullptr),
           iQueue_(nullptr),
           iCmdBuffs_(max_command_buffers, nullptr),
@@ -136,7 +156,8 @@ class VirtualGPU : public device::VirtualDevice {
           amd::Os::yield();
           continue;
         }
-        result = iDev_->WaitForFences(1, &iCmdFences_[cbId], true, WaitTimeoutInNsec);
+        result = iDev_->WaitForFences(1, &iCmdFences_[cbId], true,
+                                      std::chrono::nanoseconds{WaitTimeoutInNsec});
         if (Pal::Result::Success == result) {
           break;
         } else if ((Pal::Result::NotReady == result) || (Pal::Result::Timeout == result)) {
@@ -173,6 +194,8 @@ class VirtualGPU : public device::VirtualDevice {
     std::vector<Pal::ICmdBuffer*> iCmdBuffs_;  //!< PAL command buffers
     std::vector<Pal::IFence*> iCmdFences_;     //!< PAL fences, associated with CMD
     const amd::Kernel* last_kernel_;           //!< Last submitted kernel
+    AqlPacketMgmt* aql_mgmt_;                  //!< AQL packet emulation managment
+    void* info_ = nullptr;                     //!< Queue info for RT queues
 
    private:
     void DumpMemoryReferences() const;
@@ -226,6 +249,7 @@ class VirtualGPU : public device::VirtualDevice {
       uint perfCounterEnabled_ : 1;  //!< PerfCounter is enabled
       uint rgpCaptureEnabled_ : 1;   //!< RGP capture is enabled in the runtime
       uint imageBufferWrtBack_ : 1;  //!< Enable image buffer write back
+      uint anyOrder_ : 1;            //!< Kernel launches don't need a barrier
     };
     uint value_;
     State() : value_(0) {}
@@ -253,7 +277,12 @@ class VirtualGPU : public device::VirtualDevice {
     //! Invalidates GPU caches if memory dependency tracking is disabled
     void sync(VirtualGPU& gpu) const {
       if (maxMemObjectsInQueue_ == 0) {
-        gpu.addBarrier(RgpSqqtBarrierReason::MemDependency);
+        // Ignore the barrier in any order mode. The app is responsible for synchronization.
+        // HW will execute the kernels asynchronously
+        if (!gpu.anyOrder()) {
+          // Wait for GPU and invalidate L1 cache
+          gpu.addBarrier(RgpSqqtBarrierReason::MemDependency);
+        }
       }
     }
 
@@ -273,37 +302,6 @@ class VirtualGPU : public device::VirtualDevice {
     size_t maxMemObjectsInQueue_;     //!< Maximum number of mem objects in the queue
   };
 
-
-  class DmaFlushMgmt : public amd::EmbeddedObject {
-   public:
-    DmaFlushMgmt(const Device& dev);
-
-    // Resets DMA command buffer workload
-    void resetCbWorkload(const Device& dev);
-
-    // Finds split size for the current dispatch
-    void findSplitSize(const Device& dev,  //!< GPU device object
-                       uint64_t threads,   //!< Total number of execution threads
-                       uint instructions   //!< Number of ALU instructions
-    );
-
-    // Returns TRUE if DMA command buffer is ready for a flush
-    bool isCbReady(VirtualGPU& gpu,   //!< Virtual GPU object
-                   uint64_t threads,  //!< Total number of execution threads
-                   uint instructions  //!< Number of ALU instructions
-    );
-
-    // Returns dispatch split size
-    uint dispatchSplitSize() const { return dispatchSplitSize_; }
-
-   private:
-    uint64_t maxDispatchWorkload_;  //!< Maximum number of operations for a single dispatch
-    uint64_t maxCbWorkload_;        //!< Maximum number of operations for DMA command buffer
-    uint64_t cbWorkload_;           //!< Current number of operations in DMA command buffer
-    uint aluCnt_;                   //!< All ALUs on the chip
-    uint dispatchSplitSize_;        //!< Dispath split size in elements
-  };
-
  public:
   VirtualGPU(Device& device);
   //! Creates virtual gpu object
@@ -313,6 +311,7 @@ class VirtualGPU : public device::VirtualDevice {
               amd::CommandQueue::Priority priority = amd::CommandQueue::Priority::Normal);
   ~VirtualGPU();
 
+  uint64_t getQueueID() { return hwRing_; }
   void submitReadMemory(amd::ReadMemoryCommand& vcmd);
   void submitWriteMemory(amd::WriteMemoryCommand& vcmd);
   void submitCopyMemory(amd::CopyMemoryCommand& vcmd);
@@ -325,14 +324,14 @@ class VirtualGPU : public device::VirtualDevice {
       const amd::Kernel& kernel,           //!< Kernel for execution
       const_address parameters,            //!< Parameters for the kernel
       bool nativeMem = true,               //!< Native memory objects
-      amd::Event* enqueueEvent = nullptr,  //!< Event provided in the enqueue kernel command
       uint32_t sharedMemBytes = 0,         //!< Shared memory size
-      bool cooperativeGroups = false       //!< TRUE if cooperative groups mode is required
+      bool anyOrder = false                //!< TRUE if any order launch mode is enabled
   );
   void submitNativeFn(amd::NativeFnCommand& vcmd);
   void submitFillMemory(amd::FillMemoryCommand& vcmd);
   void submitMigrateMemObjects(amd::MigrateMemObjectsCommand& cmd);
   void submitMarker(amd::Marker& vcmd);
+  void submitAccumulate(amd::AccumulateCommand& vcmd);
   void submitAcquireExtObjects(amd::AcquireExtObjectsCommand& vcmd);
   void submitReleaseExtObjects(amd::ReleaseExtObjectsCommand& vcmd);
   void submitPerfCounter(amd::PerfCounterCommand& vcmd);
@@ -355,10 +354,16 @@ class VirtualGPU : public device::VirtualDevice {
 
   void profilerAttach(bool enable = false) {}
 
-  bool isHandlerPending() const { return false; }
-
   bool isFenceDirty() const { return false; }
 
+  void HiddenHeapInit() {}
+
+  inline bool dispatchAqlPacket(uint8_t* aqlpacket, const std::string& kernelName,
+                                amd::AccumulateCommand* vcmd = nullptr) {
+    vcmd->addKernelName(kernelName);
+    return false;
+  }
+
   void resetFenceDirty() {}
 
   //! Returns GPU device object associated with this kernel
@@ -402,8 +407,8 @@ class VirtualGPU : public device::VirtualDevice {
   );
 
   //! Embeds memory handle info into the CB associated with this VGPU
-  inline void logVmMemory(const std::string name, //!< Brief description of the memory object
-                          const Memory* memory //!< GPU memory object
+  inline void logVmMemory(const std::string name,  //!< Brief description of the memory object
+                          const Memory* memory     //!< GPU memory object
   );
 
   //! Adds a memory handle into the PAL memory array for Virtual Heap
@@ -412,11 +417,11 @@ class VirtualGPU : public device::VirtualDevice {
 
   //! Adds the last submitted kernel to the queue for tracking a possible hang
   inline void AddKernel(const amd::Kernel& kernel  //!< AMD kernel object
-                        ) const;
+  ) const;
 
   //! Checks if runtime dispatches the same kernel as previously
   inline bool IsSameKernel(const amd::Kernel& kernel  //!< AMD kernel object
-                           ) const;
+  ) const;
 
   //! Adds a dopp desktop texture reference
   void addDoppRef(const Memory* memory,  //!< GPU memory object
@@ -448,15 +453,15 @@ class VirtualGPU : public device::VirtualDevice {
   //! Checks if profiling is enabled
   bool profiling() const { return state_.profiling_; }
 
+  //! Checks if the queue is in any order mode
+  bool anyOrder() const { return state_.anyOrder_; }
+
   //! Returns memory dependency class
   MemoryDependency& memoryDependency() { return memoryDependency_; }
 
   //! Returns hsaQueueMem_
   const Memory* hsaQueueMem() const { return hsaQueueMem_; }
 
-  //! Returns DMA flush management structure
-  const DmaFlushMgmt& dmaFlushMgmt() const { return dmaFlushMgmt_; }
-
   //! Returns the HW ring used on this virtual device
   uint hwRing() const { return hwRing_; }
 
@@ -486,20 +491,29 @@ class VirtualGPU : public device::VirtualDevice {
   //! Returns queue, associated with VirtualGPU
   Queue& queue(EngineType id) const { return *queues_[id]; }
 
-  void addBarrier(RgpSqqtBarrierReason reason = RgpSqqtBarrierReason::Unknown,
-                  bool flushL2 = false) const {
+  void addBarrier(RgpSqqtBarrierReason reason = RgpSqqtBarrierReason::MemDependency,
+                  BarrierType type = BarrierType::KernelToKernel) const {
     Pal::BarrierInfo barrier = {};
     barrier.pipePointWaitCount = 1;
     Pal::HwPipePoint point = Pal::HwPipePostCs;
     barrier.pPipePoints = &point;
     barrier.transitionCount = 1;
-    uint32_t cacheMask = (flushL2) ? Pal::CoherCopy : Pal::CoherShader;
-    Pal::BarrierTransition trans = {cacheMask,
-                                    cacheMask,
-                                    {nullptr,
-                                     {{0, 0, 0}, 0, 0, 0},
-                                     Pal::LayoutShaderRead,
-                                     Pal::LayoutShaderRead}};
+    Pal::BarrierTransition trans = {};
+    trans.srcCacheMask = Pal::CoherShader;
+    trans.dstCacheMask = Pal::CoherShader;
+    trans.imageInfo.oldLayout.usages = Pal::LayoutShaderRead;
+    trans.imageInfo.oldLayout.engines = Pal::LayoutComputeEngine;
+    trans.imageInfo.newLayout.usages = Pal::LayoutShaderRead;
+    trans.imageInfo.newLayout.engines = Pal::LayoutComputeEngine;
+    if (type == BarrierType::KernelToCopy) {
+      trans.dstCacheMask = Pal::CoherCopy;
+    } else if (type == BarrierType::CopyToKernel) {
+      trans.srcCacheMask = Pal::CoherCopy;
+    } else if (type == BarrierType::CopyToCopy) {
+      trans.dstCacheMask = trans.srcCacheMask = Pal::CoherCopy;
+    } else if (type == BarrierType::FlushL2) {
+      trans.dstCacheMask = trans.srcCacheMask = Pal::CoherCopy | Pal::CoherCpu;
+    }
     barrier.pTransitions = &trans;
     barrier.waitPoint = Pal::HwPipePreCs;
     barrier.reason = static_cast<uint32_t>(reason);
@@ -578,6 +592,25 @@ class VirtualGPU : public device::VirtualDevice {
     }
   }
 
+  //! Updates timestamp for AQL packet index
+  void AqlPacketUpdateTs(uint32_t index, GpuEvent gpu_event) {
+    // Save the new CB ID for this slot
+    queues_[MainEngine]->aql_mgmt_->aql_events_[index] = gpu_event;
+    queues_[MainEngine]->aql_mgmt_->aql_vgpus_[index] = this;
+  }
+
+  //! Returns the current active slot for AQL packet
+  hsa_kernel_dispatch_packet_t* GetAqlPacketSlot(uint32_t* index) {
+    auto& mgmt = *queues_[MainEngine]->aql_mgmt_;
+    // Atomic increment global AQL index and wrap around max AQL list size
+    *index = ++mgmt.packet_index_ % AqlPacketMgmt::kAqlPacketsListSize;
+    if (mgmt.aql_events_[*index].isValid()) {
+      // Make sure GPU doesn't process this slot
+      mgmt.aql_vgpus_[*index]->waitForEvent(&mgmt.aql_events_[*index]);
+    }
+    return &mgmt.aql_packets_[*index];
+  }
+
  protected:
   void profileEvent(EngineType engine, bool type) const;
 
@@ -662,8 +695,6 @@ class VirtualGPU : public device::VirtualDevice {
   TimeStampCache* tsCache_;            //!< TimeStamp cache
   MemoryDependency memoryDependency_;  //!< Memory dependency class
 
-  DmaFlushMgmt dmaFlushMgmt_;  //!< DMA flush management
-
   std::vector<amd::Memory*> pinnedMems_;  //!< Pinned memory list
 
   ManagedBuffer managedBuffer_;  //!< Managed write buffer
@@ -695,6 +726,8 @@ class VirtualGPU : public device::VirtualDevice {
   MemoryRange sdmaRange_;                   //!< SDMA memory range for write access
 
   void* hostcallBuffer_;  //!< Hostcall buffer
+
+  using KernelArgImpl = device::Settings::KernelArgImpl;
 };
 
 inline void VirtualGPU::logVmMemory(const std::string name, const Memory* memory) {
@@ -707,7 +740,7 @@ inline void VirtualGPU::logVmMemory(const std::string name, const Memory* memory
     if (PAL_EMBED_KERNEL_MD) {
       iCmd()->CmdCommentString(buf);
     }
-    ClPrint(amd::LOG_INFO, amd::LOG_KERN, "%s", buf);
+    ClPrint(amd::LOG_INFO, amd::LOG_MEM, "%s", buf);
   }
 }
 
@@ -736,5 +769,5 @@ template <bool avoidBarrierSubmit> uint VirtualGPU::Queue::submit(bool forceFlus
   return id;
 }
 
-/*@}*/  // namespace pal
-}  // namespace pal
+/*@}*/  // namespace amd::pal
+}  // namespace amd::pal
