@@ -33,7 +33,7 @@
  */
 
 //! ROC Blit Manager Implementation
-namespace roc {
+namespace amd::roc {
 
 class Device;
 class Kernel;
@@ -208,7 +208,6 @@ class DmaBlitManager : public device::HostBlitManager {
   }
 
  protected:
-  static constexpr uint MaxPinnedBuffers = 4;
 
   //! Synchronizes the blit operations if necessary
   inline void synchronize() const;
@@ -230,11 +229,21 @@ class DmaBlitManager : public device::HostBlitManager {
   //! Assits in transferring data from Host to Local or vice versa
   //! taking into account the Hsail profile supported by Hsa Agent
   bool hsaCopy(const Memory& srcMemory, const Memory& dstMemory, const amd::Coord3D& srcOrigin,
-               const amd::Coord3D& dstOrigin, const amd::Coord3D& size) const;
+               const amd::Coord3D& dstOrigin, const amd::Coord3D& size,
+               amd::CopyMetadata& copyMetadata) const;
+
+  inline bool rocrCopyBuffer(address dst, hsa_agent_t& dstAgent,
+                             const_address src, hsa_agent_t& srcAgent, size_t size,
+                             amd::CopyMetadata& copyMetadata) const;
+
+  const size_t PinXferSize;                   //!< Copy size for Pinned Copy
+  const size_t MinSizeForPinnedXfer;          //!< Mininum copy size for Pinned Copy
+  const size_t StagingXferSize;               //!< Copy size for Staging Buffer Copy
 
-  const size_t MinSizeForPinnedTransfer;
   bool completeOperation_;                    //!< DMA blit manager must complete operation
   amd::Context* context_;                     //!< A dummy context
+  uint32_t sdmaEngineReadMask_;               //!< SDMA Engine Read Mask
+  uint32_t sdmaEngineWriteMask_;              //!< SDMA Engine Write Mask
 
  private:
   //! Disable copy constructor
@@ -243,34 +252,30 @@ class DmaBlitManager : public device::HostBlitManager {
   //! Disable operator=
   DmaBlitManager& operator=(const DmaBlitManager&);
 
-  //! Reads video memory, using a staged buffer
-  bool readMemoryStaged(Memory& srcMemory,  //!< Source memory object
-                        void* dstHost,      //!< Destination host memory
-                        Memory& xferBuf,    //!< Staged buffer for read
-                        size_t origin,      //!< Original offset in the source memory
-                        size_t& offset,     //!< Offset for the current copy pointer
-                        size_t& totalSize,  //!< Total size for copy region
-                        size_t xferSize     //!< Transfer size
+  bool hsaCopyStagedOrPinned(const_address hostSrc,             //!< Src buffer address
+                             address hostDst,                   //!< Dst Buffer address
+                             size_t size,                       //!< Size of copy data in bytes
+                             bool hostToDev,                    //!< True for H2D copy
+                             amd::CopyMetadata& copyMetadata,   //!< copy MetaData
+                             bool enPinning = false             //!< True if pinning required
+                             ) const;
+  struct BufferState{
+    address buffer_;         //!< Staging Buffer or Pinned Host Mem Address
+    amd::Memory* pinnedMem_; //!< Pinned Memory
+    size_t copySize_;        //!< last copy size
+  };
+
+  // Get Pinned Host Memory or Staging Buffer
+  void getBuffer(const_address hostMem,         //!< Host Mem Address
+                        size_t size,            //!< Transfer Size
+                        bool enablePin,         //!< True when Pinning is enabled
+                        bool first_tx,          //!< True for first copy
+                        BufferState &buffer     //!< State of Buffer
                         ) const;
 
-  //! Write into video memory, using a staged buffer
-  bool writeMemoryStaged(const void* srcHost,  //!< Source host memory
-                         Memory& dstMemory,    //!< Destination memory object
-                         Memory& xferBuf,      //!< Staged buffer for write
-                         size_t origin,        //!< Original offset in the destination memory
-                         size_t& offset,       //!< Offset for the current copy pointer
-                         size_t& totalSize,    //!< Total size for the copy region
-                         size_t xferSize       //!< Transfer size
-                         ) const;
-
-  //! Assits in transferring data from Host to Local or vice versa
-  //! taking into account the Hsail profile supported by Hsa Agent
-  bool hsaCopyStaged(const_address hostSrc,  //!< Contains source data to be copied
-                     address hostDst,        //!< Destination buffer address for copying
-                     size_t size,            //!< Size of data to copy in bytes
-                     address staging,        //!< Staging resource
-                     bool hostToDev          //!< True if data is copied from Host To Device
-                     ) const;
+  // Release Pinned host memory
+  void releaseBuffer(BufferState &buff //!< True if last copy used Pinned resource
+                    ) const;
 };
 
 //! Kernel Blit Manager
@@ -288,6 +293,7 @@ class KernelBlitManager : public DmaBlitManager {
     Scheduler,
     GwsInit,
     InitHeap,
+    BatchMemOp,
     BlitLinearTotal,
     FillImage = BlitLinearTotal,
     BlitCopyImage,
@@ -512,6 +518,9 @@ class KernelBlitManager : public DmaBlitManager {
                              uint64_t mask
   ) const;
 
+  //! Batch memory ops- Submits batch of streamWaits and streamWrite operations.
+  virtual bool batchMemOps(const void* paramArray, size_t paramSize, uint32_t count) const;
+
   virtual amd::Monitor* lockXfer() const { return &lockXferOps_; }
 
   virtual bool initHeap(device::Memory* heap_to_initialize,
@@ -569,13 +578,19 @@ class KernelBlitManager : public DmaBlitManager {
                           const device::Memory* dev_mem = nullptr,
                           bool writeVAImmediate = false) const;
 
-  static constexpr uint32_t kCBSize = 0x80;
-  static constexpr size_t   kCBAlignment = 0x80;
+  static constexpr uint32_t kCBSize = 0x100;
+  static constexpr size_t   kCBAlignment = 0x100;
 
   inline uint32_t NumBlitKernels() {
     return (dev().info().imageSupport_) ? BlitTotal : BlitLinearTotal;
   }
 
+  //! Copies a buffer using the shader path
+  bool shaderCopyBuffer(address dst, address src,
+                        const amd::Coord3D& dstOrigin, const amd::Coord3D& srcOrigin,
+                        const amd::Coord3D& size, bool entire, const uint32_t blitWg,
+                        amd::CopyMetadata copyMetadata, bool attachSignal = false) const;
+
   //! Disable copy constructor
   KernelBlitManager(const KernelBlitManager&);
 
@@ -589,13 +604,15 @@ class KernelBlitManager : public DmaBlitManager {
 };
 
 static const char* BlitName[KernelBlitManager::BlitTotal] = {
-  "__amd_rocclr_fillBufferAligned", "__amd_rocclr_fillBufferAligned2D", "__amd_rocclr_copyBuffer",
-  "__amd_rocclr_copyBufferAligned", "__amd_rocclr_copyBufferRect",
-  "__amd_rocclr_copyBufferRectAligned", "__amd_rocclr_streamOpsWrite", "__amd_rocclr_streamOpsWait",
-  "__amd_rocclr_scheduler", "__amd_rocclr_gwsInit", "__amd_rocclr_initHeap",
-  "__amd_rocclr_fillImage", "__amd_rocclr_copyImage", "__amd_rocclr_copyImage1DA",
-  "__amd_rocclr_copyImageToBuffer", "__amd_rocclr_copyBufferToImage"
-};
+    "__amd_rocclr_fillBufferAligned", "__amd_rocclr_fillBufferAligned2D",
+    "__amd_rocclr_copyBuffer",        "__amd_rocclr_copyBufferAligned",
+    "__amd_rocclr_copyBufferRect",    "__amd_rocclr_copyBufferRectAligned",
+    "__amd_rocclr_streamOpsWrite",    "__amd_rocclr_streamOpsWait",
+    "__amd_rocclr_scheduler",         "__amd_rocclr_gwsInit",
+    "__amd_rocclr_initHeap",          "__amd_rocclr_batchMemOp",
+    "__amd_rocclr_fillImage",         "__amd_rocclr_copyImage",
+    "__amd_rocclr_copyImage1DA",      "__amd_rocclr_copyImageToBuffer",
+    "__amd_rocclr_copyBufferToImage"};
 
 inline void KernelBlitManager::setArgument(amd::Kernel* kernel, size_t index,
                                            size_t size, const void* value, size_t offset,
@@ -616,9 +633,10 @@ inline void KernelBlitManager::setArgument(amd::Kernel* kernel, size_t index,
       reinterpret_cast<Memory**>(kernel->parameters().values() +
         kernel->parameters().memoryObjOffset())[desc.info_.arrayIndex_] = nullptr;
     } else {
-      // convert cl_mem to amd::Memory*, return false if invalid.
-      amd::Memory* mem = as_amd(*static_cast<const cl_mem*>(value));
       if (!writeVAImmediate) {
+        // convert cl_mem to amd::Memory*, return false if invalid.
+        amd::Memory* mem = as_amd(*static_cast<const cl_mem*>(value));
+
         reinterpret_cast<amd::Memory**>(kernel->parameters().values() +
           kernel->parameters().memoryObjOffset())[desc.info_.arrayIndex_] = mem;
         if (dev_mem == nullptr) {
@@ -671,4 +689,4 @@ inline void KernelBlitManager::setArgument(amd::Kernel* kernel, size_t index,
 }
 
 
-/*@}*/} // namespace roc
+/*@}*/} // namespace amd::roc
