@@ -55,7 +55,7 @@
  */
 
 //! HSA Device Implementation
-namespace roc {
+namespace amd::roc {
 
 /**
  * @brief List of environment variables that could be used to
@@ -81,14 +81,29 @@ public:
   hsa_signal_t  signal_;  //!< HSA signal to track profiling information
   Timestamp*    ts_;      //!< Timestamp object associated with the signal
   HwQueueEngine engine_;  //!< Engine used with this signal
-  bool          done_;    //!< True if signal is done
   amd::Monitor  lock_;    //!< Signal lock for update
+
+  typedef union {
+    struct {
+      uint32_t  done_            :  1; //!< True if signal is done
+      uint32_t  isPacketDispatch_:  1; //!< True if the packet, used with the signal, is dispatch
+      uint32_t  interrupt_       :  1; //!< True if the signal will trigger an interrupt
+      uint32_t  reserved_        : 29;
+    };
+    uint32_t data_;
+  } Flags;
+
+  Flags flags_;
+
   ProfilingSignal()
     : ts_(nullptr)
     , engine_(HwQueueEngine::Compute)
-    , done_(true)
-    , lock_("Signal Ops Lock", true)
-    { signal_.handle = 0; }
+    , lock_(true) /* Signal Ops Lock */
+    {
+      signal_.handle = 0;
+      flags_.data_ = 0;
+      flags_.done_ = true;
+    }
 
   virtual ~ProfilingSignal();
   amd::Monitor& LockSignalOps() { return lock_; }
@@ -207,19 +222,36 @@ class NullDevice : public amd::Device {
     ShouldNotReachHere();
     return;
   }
-  void* virtualAlloc(void* addr, size_t size, size_t alignment) override {
+
+  void* virtualAlloc(void* req_addr, size_t size, size_t alignment) override {
     ShouldNotReachHere();
     return nullptr;
   }
 
-  void virtualFree(void* addr) override {
+  bool virtualFree(void* addr) override {
     ShouldNotReachHere();
-    return;
+    return true;
+  }
+
+  virtual bool SetMemAccess(void* va_addr, size_t va_size, VmmAccess access_flags)
+                            override {
+    ShouldNotReachHere();
+    return false;
+  }
+
+  virtual bool GetMemAccess(void* va_addr, VmmAccess* access_flags_ptr) const {
+    ShouldNotReachHere();
+    return false;
+  }
+
+  virtual bool ValidateMemAccess(amd::Memory& mem, bool read_write) const {
+    ShouldNotReachHere();
+    return true;
   }
 
   //! Determine if we can use device memory for SVM
   const bool forceFineGrain(amd::Memory* memory) const {
-    return !settings().enableCoarseGrainSVM_ || (memory->getContext().devices().size() > 1);
+    return (memory->getContext().devices().size() > 1);
   }
 
   virtual bool importExtSemaphore(void** extSemahore, const amd::Os::FileDesc& handle,
@@ -259,8 +291,11 @@ class NullDevice : public amd::Device {
     return true;
   }
 
-  bool IsHwEventReady(const amd::Event& event, bool wait = false) const override { return false; }
-  bool IsHwEventReadyForcedWait(const amd::Event& event) const override { return false; }
+  bool IsHwEventReady(const amd::Event& event, bool wait = false,
+                      uint32_t hip_event_flags = 0) const override {
+    return false;
+  }
+
   void getHwEventTime(const amd::Event& event, uint64_t* start, uint64_t* end) const override{};
   void ReleaseGlobalSignal(void* signal) const override {}
 
@@ -425,8 +460,10 @@ class Device : public NullDevice {
   bool deviceAllowAccess(void* dst) const;
 
   bool allowPeerAccess(device::Memory* memory) const;
-
-  void* deviceLocalAlloc(size_t size, bool atomics = false, bool pseudo_fine_grain=false) const;
+  void deviceVmemRelease(uint64_t mem_handle) const;
+  uint64_t deviceVmemAlloc(size_t size, uint64_t flags) const;
+  void* deviceLocalAlloc(size_t size, bool atomics = false, bool pseudo_fine_grain=false,
+                         bool contiguous = false) const;
 
   void memFree(void* ptr, size_t size) const;
 
@@ -440,14 +477,24 @@ class Device : public NullDevice {
   virtual bool GetSvmAttributes(void** data, size_t* data_sizes, int* attributes,
                                 size_t num_attributes, const void* dev_ptr, size_t count) const;
 
-  virtual void* virtualAlloc(void* addr, size_t size, size_t alignment);
-  virtual void virtualFree(void* addr);
+  virtual void* virtualAlloc(void* req_addr, size_t size, size_t alignment);
+  virtual bool virtualFree(void* addr);
+
+  virtual bool SetMemAccess(void* va_addr, size_t va_size, VmmAccess access_flags);
+  virtual bool GetMemAccess(void* va_addr, VmmAccess* access_flags_ptr) const;
+  virtual bool ValidateMemAccess(amd::Memory& mem, bool read_write) const { return true; }
+
+  virtual bool ExportShareableVMMHandle(amd::Memory& amd_mem_obj, int flags, void* shareableHandle);
+
+  bool ImportShareableHSAHandle(void* osHandle, uint64_t* hsa_handle_ptr) const;
+
+  virtual amd::Memory* ImportShareableVMMHandle(void* osHandle);
 
   virtual bool SetClockMode(const cl_set_device_clock_mode_input_amd setClockModeInput,
                             cl_set_device_clock_mode_output_amd* pSetClockModeOutput);
 
-  virtual bool IsHwEventReady(const amd::Event& event, bool wait = false) const;
-  virtual bool IsHwEventReadyForcedWait(const amd::Event& event) const;
+  virtual bool IsHwEventReady(const amd::Event& event, bool wait = false,
+                              uint32_t hip_event_flags = 0) const;
   virtual void getHwEventTime(const amd::Event& event, uint64_t* start, uint64_t* end) const;
   virtual void ReleaseGlobalSignal(void* signal) const;
 
@@ -470,9 +517,6 @@ class Device : public NullDevice {
   //! Adds a map target to the cache
   bool addMapTarget(amd::Memory* memory) const;
 
-  //! Returns transfer buffer object
-  XferBuffers& xferWrite() const { return *xferWrite_; }
-
   //! Returns transfer buffer object
   XferBuffers& xferRead() const { return *xferRead_; }
 
@@ -516,10 +560,15 @@ class Device : public NullDevice {
   //! share previously created
   hsa_queue_t* acquireQueue(uint32_t queue_size_hint, bool coop_queue = false,
                             const std::vector<uint32_t>& cuMask = {},
-                            amd::CommandQueue::Priority priority = amd::CommandQueue::Priority::Normal);
+                            amd::CommandQueue::Priority priority = amd::CommandQueue::Priority::Normal,
+                            bool managed = false);
 
   //! Release HSA queue
-  void releaseQueue(hsa_queue_t*, const std::vector<uint32_t>& cuMask = {});
+  void releaseQueue(hsa_queue_t*, const std::vector<uint32_t>& cuMask = {},
+                    bool coop_queue = false, bool managed = false);
+
+  hsa_queue_t* AcquireActiveNormalQueue();
+  bool ReleaseActiveNormalQueue(hsa_queue_t* queue);
 
   //! For the given HSA queue, return an existing hostcall buffer or create a
   //! new one. queuePool_ keeps a mapping from HSA queue to hostcall buffer.
@@ -552,9 +601,9 @@ class Device : public NullDevice {
 
   //! Allocates hidden heap for device memory allocations
   void HiddenHeapAlloc(const VirtualGPU& gpu);
-
-  uint32_t fetchSDMAMask(const device::BlitManager* handle, bool readEngine = true) const;
-  void resetSDMAMask(const device::BlitManager* handle) const ;
+  //! Init hidden heap for device memory allocations
+  void HiddenHeapInit(const VirtualGPU& gpu);
+  void getSdmaRWMasks(uint32_t* readMask, uint32_t* writeMask) const;
   bool isXgmi() const { return isXgmi_; }
 
  private:
@@ -583,7 +632,7 @@ class Device : public NullDevice {
   mutable std::mutex lock_allow_access_; //!< To serialize allow_access calls
   hsa_agent_t bkendDevice_;
   uint32_t pciDeviceId_;
-  hsa_agent_t* p2p_agents_list_;
+  hsa_agent_t* p2p_agents_list_ = nullptr;
   hsa_profile_t agent_profile_;
   hsa_amd_memory_pool_t group_segment_;
   hsa_amd_memory_pool_t system_segment_;
@@ -601,19 +650,30 @@ class Device : public NullDevice {
   VirtualGPU* xferQueue_;  //!< Transfer queue, created on demand
 
   XferBuffers* xferRead_;   //!< Transfer buffers read
-  XferBuffers* xferWrite_;  //!< Transfer buffers write
   std::atomic<size_t> freeMem_;   //!< Total of free memory available
   mutable amd::Monitor vgpusAccess_;     //!< Lock to serialise virtual gpu list access
   bool hsa_exclusive_gpu_access_;  //!< TRUE if current device was moved into exclusive GPU access mode
   static address mg_sync_;  //!< MGPU grid launch sync memory (SVM location)
 
   struct QueueInfo {
-    int refCount;
-    void* hostcallBuffer_;
+    int refCount;           //! Reference counter. Shows how many time the queue was shared
+    void* hostcallBuffer_;  //! Host call buffer for the HSA queue
   };
 
+  struct QueueCompare {
+    // Customized queue compare operator to make sure the queues are sorted in the creation order
+    bool operator()(hsa_queue_t* lhs, hsa_queue_t* rhs) const {
+      if (DEBUG_HIP_DYNAMIC_QUEUES) {
+        return (lhs->id < rhs->id) ? true : false;
+      } else {
+        return (lhs < rhs) ? true : false;
+      }
+    }
+  };
   //! a vector for keeping Pool of HSA queues with low, normal and high priorities for recycling
-  std::vector<std::map<hsa_queue_t*, QueueInfo>> queuePool_;
+  std::vector<std::map<hsa_queue_t*, QueueInfo, QueueCompare>> queuePool_;
+  amd::Monitor active_queue_access_;                //!< Lock to serialise virtual gpu list access
+  std::atomic<uint32_t> num_normal_queues_{0};      //!< The total number of allocated normal queues
 
   //! returns a hsa queue from queuePool with least refCount and updates the refCount as well
   hsa_queue_t* getQueueFromPool(const uint qIndex);
@@ -624,13 +684,11 @@ class Device : public NullDevice {
                             std::vector<LinkAttrType>* link_attr);
 
   //! Pool of HSA queues with custom CU masks
-  std::vector<std::map<hsa_queue_t*, QueueInfo>> queueWithCUMaskPool_;
+  std::vector<std::map<hsa_queue_t*, QueueInfo, QueueCompare>> queueWithCUMaskPool_;
 
   //! Read and Write mask for device<->host
   uint32_t maxSdmaReadMask_;
   uint32_t maxSdmaWriteMask_;
-  //! Map of SDMA engineId<->stream
-  mutable std::map<uint32_t, const device::BlitManager*> engineAssignMap_;
   bool isXgmi_; //!< Flag to indicate if there is XGMI between CPU<->GPU
 
  public:
@@ -645,7 +703,7 @@ class Device : public NullDevice {
 #endif
 #endif
 };                                // class roc::Device
-}  // namespace roc
+}  // namespace amd::roc
 
 /**
  * @}
