@@ -38,7 +38,7 @@
 #include "platform/interop_gl.hpp"
 #include "platform/external_memory.hpp"
 
-namespace roc {
+namespace amd::roc {
 
 // ======================================= roc::Memory ============================================
 Memory::Memory(const roc::Device& dev, amd::Memory& owner)
@@ -615,13 +615,13 @@ Buffer::Buffer(const roc::Device& dev, size_t size) : roc::Memory(dev, size) {}
 
 Buffer::~Buffer() {
   if (owner() == nullptr) {
-    dev().hostFree(deviceMemory_, size());
+    dev().memFree(deviceMemory_, size());
   } else {
     destroy();
 
     if (owner()->ipcShared()) {
       // Detach the memory from HSA
-      auto hsa_status = hsa_amd_ipc_memory_detach(owner()->getHostMem());
+      auto hsa_status = hsa_amd_ipc_memory_detach(owner()->getSvmPtr());
       if (hsa_status != HSA_STATUS_SUCCESS) {
         LogPrintfError("HSA failed to detach memory with status: %d \n", hsa_status);
       }
@@ -648,6 +648,12 @@ void Buffer::destroy() {
     }
     const bool isFineGrain = memFlags & CL_MEM_SVM_FINE_GRAIN_BUFFER;
 
+    if (memFlags & ROCCLR_MEM_PHYMEM) {
+      // If this is physical memory, dont call hsa free function, since device mem was never created
+      dev().deviceVmemRelease(owner()->getUserData().hsa_handle);
+      return;
+    }
+
     if (kind_ != MEMORY_KIND_PTRGIVEN) {
       if (isFineGrain) {
         if (memFlags & CL_MEM_ALLOC_HOST_PTR) {
@@ -657,7 +663,7 @@ void Buffer::destroy() {
               ClPrint(amd::LOG_DEBUG, amd::LOG_MEM, "[ROCClr] munmap failed \n");
             }
           } else {
-            dev().hostFree(deviceMemory_, size());
+            dev().memFree(deviceMemory_, size());
           }
         } else if (memFlags & ROCCLR_MEM_HSA_SIGNAL_MEMORY) {
           if (HSA_STATUS_SUCCESS != hsa_signal_destroy(signal_)) {
@@ -666,7 +672,7 @@ void Buffer::destroy() {
           }
           deviceMemory_ = nullptr;
         } else {
-          dev().hostFree(deviceMemory_, size());
+          dev().memFree(deviceMemory_, size());
         }
       } else {
         dev().memFree(deviceMemory_, size());
@@ -682,15 +688,12 @@ void Buffer::destroy() {
   }
 
   if (deviceMemory_ != nullptr) {
+    bool needUnlockHostMem = false;
     if (deviceMemory_ != owner()->getHostMem()) {
       // if they are identical, the host pointer will be
       // deallocated later on => avoid double deallocation
       if (isHostMemDirectAccess()) {
-        if (memFlags & (CL_MEM_USE_HOST_PTR | CL_MEM_ALLOC_HOST_PTR)) {
-          if (dev().agent_profile() != HSA_PROFILE_FULL) {
-            hsa_amd_memory_unlock(owner()->getHostMem());
-          }
-        }
+        needUnlockHostMem = true;
       } else {
         dev().memFree(deviceMemory_, size());
         const_cast<Device&>(dev()).updateFreeMemory(size(), true);
@@ -702,6 +705,18 @@ void Buffer::destroy() {
         if (dev().settings().apuSystem_) {
           const_cast<Device&>(dev()).updateFreeMemory(size(), true);
         }
+      } else if ((memFlags & CL_MEM_ALLOC_HOST_PTR) &&
+                 (owner()->getContext().devices().size() == 1)) {
+        dev().hostFree(deviceMemory_, size());
+      } else if (isHostMemDirectAccess()) {
+        needUnlockHostMem = true;
+      }
+    }
+
+    if (needUnlockHostMem) {
+      if (memFlags & (CL_MEM_USE_HOST_PTR | CL_MEM_ALLOC_HOST_PTR)) {
+        if (dev().agent_profile() != HSA_PROFILE_FULL)
+          hsa_amd_memory_unlock(owner()->getHostMem());
       }
     }
   }
@@ -748,9 +763,36 @@ bool Buffer::create(bool alloc_local) {
     owner()->setSvmPtr(orig_dev_ptr);
   }
 
+
+
   // Allocate backing storage in device local memory unless UHP or AHP are set
   cl_mem_flags memFlags = owner()->getMemFlags();
 
+  if (memFlags & ROCCLR_MEM_PHYMEM) {
+    if (memFlags & ROCCLR_MEM_INTERPROCESS) {
+      int dmabuf_fd = *(reinterpret_cast<int*>(owner()->getSvmPtr()));
+      // if interprocess flag is set, then the memory is importable.
+      if (!dev().ImportShareableHSAHandle(owner()->getSvmPtr(),
+            &owner()->getUserData().hsa_handle)) {
+        LogPrintfError("Importing Shareable Memory failed with os_handle: 0x%x \n",
+                        owner()->getSvmPtr());
+        return false;
+      }
+    } else {
+      // If this is physical memory request, then get an handle and store it in user data
+      owner()->getUserData().hsa_handle = dev().deviceVmemAlloc(owner()->getSize(), 0);
+    }
+
+    if (owner()->getUserData().hsa_handle == 0) {
+      LogError("HSA Opaque Handle returned was null");
+      return false;
+    }
+
+    owner()->setSvmPtr(reinterpret_cast<void*>(owner()->getUserData().hsa_handle));
+
+    return true;
+  }
+
   if ((owner()->parent() == nullptr) &&
       (owner()->getSvmPtr() != nullptr)) {
     if (dev().forceFineGrain(owner()) || dev().isFineGrainedSystem(true)) {
@@ -758,7 +800,7 @@ bool Buffer::create(bool alloc_local) {
     }
     const bool isFineGrain = memFlags & CL_MEM_SVM_FINE_GRAIN_BUFFER;
 
-    if (isFineGrain) {
+    if (isFineGrain && !(memFlags & CL_MEM_VA_RANGE_AMD)) {
       // Use CPU direct access for the fine grain buffer
       flags_ |= HostMemoryDirectAccess;
     }
@@ -814,7 +856,8 @@ bool Buffer::create(bool alloc_local) {
       } else {
         assert(!isHostMemDirectAccess() && "Runtime doesn't support direct access to GPU memory!");
         deviceMemory_ = dev().deviceLocalAlloc(size(), (memFlags & CL_MEM_SVM_ATOMICS) != 0,
-                                               (memFlags & ROCCLR_MEM_HSA_UNCACHED) != 0);
+                                               (memFlags & ROCCLR_MEM_HSA_UNCACHED) != 0,
+                                               (memFlags & ROCCLR_MEM_HSA_CONTIGUOUS) != 0);
       }
       owner()->setSvmPtr(deviceMemory_);
     } else {
@@ -941,7 +984,7 @@ bool Buffer::create(bool alloc_local) {
 
     return deviceMemory_ != nullptr;
   }
-  assert(owner()->getHostMem() != nullptr);
+  assert(owner()->getHostMem() != nullptr || (owner()->getContext().devices().size() == 1));
 
   flags_ |= HostMemoryDirectAccess;
 
@@ -955,14 +998,23 @@ bool Buffer::create(bool alloc_local) {
     return deviceMemory_ != nullptr;
   }
 
-  if (owner()->getSvmPtr() != owner()->getHostMem()) {
+  // Just one device and allocation must be done in the backend
+  if ((memFlags & CL_MEM_ALLOC_HOST_PTR) && (owner()->getContext().devices().size() == 1)) {
+    deviceMemory_ = dev().hostAlloc(size(), 1, Device::MemorySegment::kNoAtomics);
+    // Copy original data to the allocated host memory
+    if (memFlags & CL_MEM_COPY_HOST_PTR) {
+      memcpy(deviceMemory_, owner()->getHostMem(), owner()->getSize());
+    }
+    owner()->setHostMem(deviceMemory_);
+  } else if (owner()->getSvmPtr() != owner()->getHostMem()) {
     if (memFlags & (CL_MEM_USE_HOST_PTR | CL_MEM_ALLOC_HOST_PTR)) {
       hsa_amd_memory_pool_t pool = (memFlags & CL_MEM_SVM_ATOMICS) ?
                                     dev().SystemSegment() :
                                     (dev().SystemCoarseSegment().handle != 0 ?
                                         dev().SystemCoarseSegment() : dev().SystemSegment());
+      hsa_agent_t hsa_agent = dev().getBackendDevice();
       hsa_status_t status = hsa_amd_memory_lock_to_pool(owner()->getHostMem(),
-          owner()->getSize(), nullptr, 0, pool, 0, &deviceMemory_);
+          owner()->getSize(), &hsa_agent, 1, pool, 0, &deviceMemory_);
       ClPrint(amd::LOG_DEBUG, amd::LOG_MEM, "Locking to pool %p, size 0x%zx, HostPtr = %p,"
               " DevPtr = %p", pool, owner()->getSize(), owner()->getHostMem(), deviceMemory_ );
       if (status != HSA_STATUS_SUCCESS) {
@@ -1200,6 +1252,7 @@ bool Image::create(bool alloc_local) {
     permission_      = orgImage->permission_;
     deviceMemory_    = orgImage->deviceMemory_;
     hsaImageObject_  = orgImage->hsaImageObject_;
+    ownsHsaImageObject_ = false;
     return true;
   }
 
@@ -1438,7 +1491,7 @@ void Image::destroy() {
 
   delete copyImageBuffer_;
 
-  if (hsaImageObject_.handle != 0) {
+  if (hsaImageObject_.handle != 0 && ownsHsaImageObject_) {
     hsa_status_t status = hsa_ext_image_destroy(dev().getBackendDevice(), hsaImageObject_);
     assert(status == HSA_STATUS_SUCCESS);
   }
