@@ -1,4 +1,4 @@
-/* Copyright (c) 2008 - 2021 Advanced Micro Devices, Inc.
+/* Copyright (c) 2008 - 2023 Advanced Micro Devices, Inc.
 
  Permission is hereby granted, free of charge, to any person obtaining a copy
  of this software and associated documentation files (the "Software"), to deal
@@ -214,7 +214,7 @@ address Os::reserveMemory(address start, size_t size, size_t alignment, MemProt
                                 MAP_PRIVATE | MAP_NORESERVE | MAP_ANONYMOUS, 0, 0);
 
   // check for out of memory
-  if (mem == NULL) return NULL;
+  if (mem == MAP_FAILED) return NULL;
 
   address aligned = alignUp(mem, alignment);
 
@@ -233,6 +233,17 @@ address Os::reserveMemory(address start, size_t size, size_t alignment, MemProt
     }
   }
 
+  // Hint to enable THP for large host allocations which can help in performance gain
+  constexpr size_t kLargePageSize = 2 * Mi;
+  if (size >= kLargePageSize) {
+    int status = madvise(aligned, size, MADV_HUGEPAGE);
+    if (status) {
+      ClPrint(amd::LOG_DEBUG, amd::LOG_CODE, "madvise with advice MADV_HUGEPAGE"
+              " starting at address %p and page size 0x%zx, returned %d, errno: %s",
+              aligned, size, status, strerror(errno));
+    }
+  }
+
   return aligned;
 }
 
@@ -513,7 +524,7 @@ int Os::systemCall(const std::string& command) {
 #if 1
   size_t len = command.size();
   char* cmd = new char[len + 1];
-  fastMemcpy(cmd, command.c_str(), len);
+  std::memcpy(cmd, command.c_str(), len);
   cmd[len] = 0;
 
   // Split the command into arguments. This is a very
@@ -670,8 +681,6 @@ uint64_t Os::xgetbv(uint32_t ecx) {
 }
 #endif  // ATI_ARCH_X86
 
-void* Os::fastMemcpy(void* dest, const void* src, size_t n) { return memcpy(dest, src, n); }
-
 uint64_t Os::offsetToEpochNanos() {
   static uint64_t offset = 0;
 
@@ -842,7 +851,7 @@ bool Os::MemoryMapFileDesc(FileDesc fdesc, size_t fsize, size_t foffset, const v
   }
 
   *mmap_ptr = mmap(NULL, fsize, PROT_READ, MAP_SHARED, fdesc, foffset);
-  return true;
+  return (*mmap_ptr == MAP_FAILED) ? false : true;
 }
 
 bool Os::MemoryUnmapFile(const void* mmap_ptr, size_t mmap_size) {
@@ -874,7 +883,7 @@ bool Os::MemoryMapFile(const char* fname, const void** mmap_ptr, size_t* mmap_si
 
   close(fd);
 
-  if (*mmap_ptr == nullptr) {
+  if (*mmap_ptr == MAP_FAILED) {
     return false;
   }
 
@@ -899,7 +908,7 @@ bool Os::MemoryMapFileTruncated(const char* fname, const void** mmap_ptr, size_t
 
   close(fd);
 
-  if (*mmap_ptr == nullptr) {
+  if (*mmap_ptr == MAP_FAILED) {
     return false;
   }
 
