@@ -51,17 +51,19 @@
 bool amd::ClGlEvent::waitForFence() {
   GLenum ret;
   // get fence id associated with fence event
-  GLsync gs = reinterpret_cast<GLsync>(command().data());
+  GLsync gs = !command().data().empty() ? reinterpret_cast<GLsync>(command().data().back())
+                                          : nullptr;
   if (!gs) return false;
 
 // Try to use DC and GLRC of current thread, if it doesn't exist
 // create a new GL context on this thread, which is shared with the original context
 
 #ifdef _WIN32
-  HDC tempDC_ = wglGetCurrentDC();
-  HGLRC tempGLRC_ = wglGetCurrentContext();
+  HDC tempDC_ = context().glenv()->wglGetCurrentDC_();
+  HGLRC tempGLRC_ = context().glenv()->wglGetCurrentContext_();
   // Set DC and GLRC
   if (tempDC_ && tempGLRC_) {
+    amd::GLFunctions::Lock lock(context().glenv());
     ret = context().glenv()->glClientWaitSync_(gs, GL_SYNC_FLUSH_COMMANDS_BIT,
                                                static_cast<GLuint64>(-1));
     if (!(ret == GL_ALREADY_SIGNALED || ret == GL_CONDITION_SATISFIED)) return false;
@@ -73,14 +75,12 @@ bool amd::ClGlEvent::waitForFence() {
       return false;
 
     // Make the newly created GL context current to this thread
-    context().glenv()->setIntEnv();
+    amd::GLFunctions::SetIntEnv ie(context().glenv());
+
     // If fence has not yet executed, wait till it finishes
     ret = context().glenv()->glClientWaitSync_(gs, GL_SYNC_FLUSH_COMMANDS_BIT,
                                                static_cast<GLuint64>(-1));
     if (!(ret == GL_ALREADY_SIGNALED || ret == GL_CONDITION_SATISFIED)) return false;
-    // Since we're done making GL calls, restore whatever context was previously current to this
-    // thread
-    context().glenv()->restoreEnv();
   }
 #else  // Lnx
   Display* tempDpy_ = context().glenv()->glXGetCurrentDisplay_();
@@ -88,6 +88,7 @@ bool amd::ClGlEvent::waitForFence() {
   GLXContext tempCtx_ = context().glenv()->glXGetCurrentContext_();
   // Set internal Display and GLXContext
   if (tempDpy_ && tempCtx_) {
+    amd::GLFunctions::Lock lock(context().glenv());
     ret = context().glenv()->glClientWaitSync_(gs, GL_SYNC_FLUSH_COMMANDS_BIT,
                                                static_cast<GLuint64>(-1));
     if (!(ret == GL_ALREADY_SIGNALED || ret == GL_CONDITION_SATISFIED)) return false;
@@ -97,14 +98,12 @@ bool amd::ClGlEvent::waitForFence() {
       return false;
 
     // Make the newly created GL context current to this thread
-    context().glenv()->setIntEnv();
+    amd::GLFunctions::SetIntEnv ie(context().glenv());
+
     // If fence has not yet executed, wait till it finishes
     ret = context().glenv()->glClientWaitSync_(gs, GL_SYNC_FLUSH_COMMANDS_BIT,
                                                static_cast<GLuint64>(-1));
     if (!(ret == GL_ALREADY_SIGNALED || ret == GL_CONDITION_SATISFIED)) return false;
-    // Since we're done making GL calls, restore whatever context was previously current to this
-    // thread
-    context().glenv()->restoreEnv();
   }
 #endif
   // If we reach this point, fence should have completed
@@ -150,6 +149,14 @@ amd::GLFunctions::SetIntEnv::~SetIntEnv() {
   env_->getLock().unlock();
 }
 
+amd::GLFunctions::Lock::Lock(GLFunctions* env) : env_(env) {
+    env_->getLock().lock();
+}
+
+amd::GLFunctions::Lock::~Lock() {
+    env_->getLock().unlock();
+}
+
 amd::GLFunctions::GLFunctions(HMODULE h, bool isEGL)
     : libHandle_(h),
       missed_(0),
