@@ -43,11 +43,13 @@
 #define CL_MEM_VA_RANGE_AMD             (1u << 28)
 #define ROCCLR_MEM_HSA_UNCACHED         (1u << 27)
 #define ROCCLR_MEM_INTERPROCESS         (1u << 26)
+#define ROCCLR_MEM_PHYMEM               (1u << 25)
+#define ROCCLR_MEM_HSA_CONTIGUOUS       (1u << 24)
 
-namespace device {
+namespace amd::device {
 class Memory;
 class VirtualDevice;
-}  // namespace device
+}  // namespace amd::device
 
 namespace amd {
 
@@ -141,13 +143,16 @@ class Memory : public amd::RuntimeObject {
  public:
   enum MemoryType {
     kSvmMemoryPtr = 0x1,
-    kArenaMemoryPtr = 0x100
+    kArenaMemoryPtr = 0x100,
   };
 
   struct UserData
   {
      int deviceId = 0;     //!< Device ID memory is allocated on
      void* data = nullptr; //!< Opaque user data from CL or HIP or etc.
+     amd::Memory* phys_mem_obj = nullptr; //<! Physical mem obj, only set on virtual mem
+     amd::Memory* vaddr_mem_obj = nullptr; //<! Virtual address mem obj, only set on virtual mem
+     uint64_t hsa_handle = 0; //!<Opaque hsa handle saved for Virtual memories
      unsigned int flags = 0; //!< HIP memory flags
      //! hipMallocPitch allocates buffer using width & height and returns pitch & device pointer.
      //! Since device pointer is void*, It looses the values of width & height used for allocation.
@@ -194,6 +199,7 @@ class Memory : public amd::RuntimeObject {
   device::VirtualDevice* vDev_;  //!< Memory object belongs to a virtual device only
   std::atomic_uint mapCount_;    //!< Keep track of number of mappings for a memory object
   void* svmHostAddress_;         //!< svm host address;
+  size_t resOffset_;             //!< resource offset
   union {
     struct {
       uint32_t isParent_ : 1;          //!< This object is a parent
@@ -374,6 +380,10 @@ class Memory : public amd::RuntimeObject {
 
   void* getSvmPtr() const { return svmHostAddress_; }   //!< svm pointer accessor;
   void setSvmPtr(void* ptr) { svmHostAddress_ = ptr; }  //!< svm pointer setter;
+
+  size_t getOffset() const { return resOffset_; }         //!< resource offset accessor;
+  void setOffset(size_t offset) { resOffset_ = offset; }  //!< resource offset setter;
+
   bool isSvmPtrCommited() const {
     return svmPtrCommited_;
   }                        //!< svm host address committed accessor;
@@ -404,6 +414,12 @@ class Memory : public amd::RuntimeObject {
 
   //!find if memory object is Arena memory
   virtual bool isArena() { return false; }
+
+  //! get device by id when glb ctx is used.
+  Device* GetDeviceById();
+
+  //! Validate memory access for vmm memory
+  bool ValidateMemAccess(const Device& dev, bool read_write);
 };
 
 //! Buffers are a specialization of memory. Just a wrapper, really,
@@ -561,7 +577,7 @@ class Image : public Memory {
         size_t depth, size_t rowPitch, size_t slicePitch, uint mipLevels = 1);
 
   Image(Buffer& buffer, Type type, Flags flags, const Format& format, size_t width, size_t height,
-        size_t depth, size_t rowPitch, size_t slicePitch);
+        size_t depth, size_t rowPitch, size_t slicePitch, uint mipLevels = 1, size_t offset = 0);
 
   //! Validate image dimensions with supported sizes
   static bool validateDimensions(
@@ -682,8 +698,9 @@ public:
 class IpcBuffer : public Buffer {
  public:
   IpcBuffer(Context& context, Flags flags, size_t offset, size_t size, const void* handle)
-    : Buffer(context, flags, offset, size), handle_(handle) {
+    : Buffer(context, flags, size), handle_(handle) {
     setIpcShared(true);
+    setOffset(offset);
   }
 
   virtual void initDeviceMemory();
