@@ -541,13 +541,11 @@ int32_t Program::build(const std::vector<Device*>& devices, const char* options,
     parsedOptions.oVariables->AssumeAlias = true;
 
     if (language_ == Assembly) {
-      constexpr char asmLang[] = "asm";
-      parsedOptions.oVariables->XLang = asmLang;
+      parsedOptions.oVariables->XLang = "asm";
     }
 
     if (language_ == HIP) {
-      constexpr char hipLang[] = "HIP";
-      parsedOptions.oVariables->CLStd = hipLang;
+      parsedOptions.oVariables->CLStd = "HIP";
       parsedOptions.origOptionStr = options;
       parsedOptions.oVariables->DumpPrefix = "_hip_";
       parsedOptions.oVariables->OptLevel = '3';
@@ -614,11 +612,16 @@ bool Program::load(const std::vector<Device*>& devices) {
     }
 
     if (!devProgram.load()) {
+      if (!devProgram.buildLog().empty()) {
+        LogPrintfError("devProgram.load() failed with buildLog=%s\n",
+            devProgram.buildLog().c_str());
+      }
       return false;
     }
 
     // Run kernels marked with init
     if (!devProgram.runInitKernels()) {
+      LogError("runInitKernels() failed\n");
       return false;
     }
   }
