@@ -28,31 +28,23 @@ release(int, AMD_LOG_LEVEL, 0,                                                \
         "The default log level")                                              \
 release(uint, AMD_LOG_MASK, 0X7FFFFFFF,                                       \
         "The mask to enable specific kinds of logs")                          \
+release(cstring, AMD_LOG_LEVEL_FILE, "",                                      \
+        "Set output file for AMD_LOG_LEVEL, Default is stderr")               \
+release(size_t, AMD_LOG_LEVEL_SIZE, 2048,                                     \
+        "The max size of AMD_LOG generated in MB if printed to a file")       \
 debug(uint, DEBUG_GPU_FLAGS, 0,                                               \
         "The debug options for GPU device")                                   \
-release(uint, GPU_MAX_COMMAND_QUEUES, 300,                                    \
-        "The maximum number of concurrent Virtual GPUs")                      \
 release(size_t, CQ_THREAD_STACK_SIZE, 256*Ki, /* @todo: that much! */         \
         "The default command queue thread stack size")                        \
 release(int, GPU_MAX_WORKGROUP_SIZE, 0,                                       \
         "Maximum number of workitems in a workgroup for GPU, 0 -use default") \
-release(int, GPU_MAX_WORKGROUP_SIZE_2D_X, 0,                                  \
-        "Maximum number of workitems in a 2D workgroup for GPU, x component, 0 -use default") \
-release(int, GPU_MAX_WORKGROUP_SIZE_2D_Y, 0,                                  \
-        "Maximum number of workitems in a 2D workgroup for GPU, y component, 0 -use default") \
-release(int, GPU_MAX_WORKGROUP_SIZE_3D_X, 0,                                  \
-        "Maximum number of workitems in a 3D workgroup for GPU, x component, 0 -use default") \
-release(int, GPU_MAX_WORKGROUP_SIZE_3D_Y, 0,                                  \
-        "Maximum number of workitems in a 3D workgroup for GPU, y component, 0 -use default") \
-release(int, GPU_MAX_WORKGROUP_SIZE_3D_Z, 0,                                  \
-        "Maximum number of workitems in a 3D workgroup for GPU, z component, 0 -use default") \
 debug(bool, CPU_MEMORY_GUARD_PAGES, false,                                    \
         "Use guard pages for CPU memory")                                     \
 debug(size_t, CPU_MEMORY_GUARD_PAGE_SIZE, 64,                                 \
         "Size in KB of CPU memory guard page")                                \
 debug(size_t, CPU_MEMORY_ALIGNMENT_SIZE, 256,                                 \
         "Size in bytes for the default alignment for guarded memory on CPU")  \
-debug(size_t, PARAMETERS_MIN_ALIGNMENT, 16,                                   \
+debug(size_t, PARAMETERS_MIN_ALIGNMENT, NATIVE_ALIGNMENT_SIZE,                \
         "Minimum alignment required for the abstract parameters stack")       \
 debug(size_t, MEMOBJ_BASE_ADDR_ALIGN, 4*Ki,                                   \
         "Alignment of the base address of any allocate memory object")        \
@@ -74,8 +66,6 @@ release(uint, GPU_BLIT_ENGINE_TYPE, 0x0,                                      \
         "Blit engine type: 0 - Default, 1 - Host, 2 - CAL, 3 - Kernel")       \
 release(bool, GPU_FLUSH_ON_EXECUTION, false,                                  \
         "Submit commands to HW on every operation. 0 - Disable, 1 - Enable")  \
-release(bool, GPU_USE_SYNC_OBJECTS, true,                                     \
-        "If enabled, use sync objects instead of polling")                    \
 release(bool, CL_KHR_FP64, true,                                              \
         "Enable/Disable support for double precision")                        \
 release(cstring, AMD_OCL_BUILD_OPTIONS, 0,                                    \
@@ -86,12 +76,8 @@ release(cstring, AMD_OCL_LINK_OPTIONS, 0,                                     \
         "Set clLinkProgram()'s options (override)")                           \
 release(cstring, AMD_OCL_LINK_OPTIONS_APPEND, 0,                              \
         "Append clLinkProgram()'s options")                                   \
-release(cstring, AMD_OCL_SC_LIB, 0,                                           \
-        "Set shader compiler shared library name or path")                    \
 debug(cstring, AMD_OCL_SUBST_OBJFILE, 0,                                      \
         "Specify binary substitution config file for OpenCL")                 \
-debug(bool, AMD_OCL_ENABLE_MESSAGE_BOX, false,                                \
-        "Enable the error dialog on Windows")                                 \
 release(size_t, GPU_PINNED_XFER_SIZE, 32,                                     \
         "The pinned buffer size for pinning in read/write transfers in MiB")  \
 release(size_t, GPU_PINNED_MIN_XFER_SIZE, 128,                                \
@@ -99,49 +85,31 @@ release(size_t, GPU_PINNED_MIN_XFER_SIZE, 128,                                \
 release(size_t, GPU_RESOURCE_CACHE_SIZE, 64,                                  \
         "The resource cache size in MB")                                      \
 release(size_t, GPU_MAX_SUBALLOC_SIZE, 4096,                                  \
-        "The maximum size accepted for suballocaitons in KB")                 \
-release(bool, GPU_FORCE_64BIT_PTR, 0,                                         \
-        "Forces 64 bit pointers on GPU")                                      \
-release(bool, GPU_FORCE_OCL20_32BIT, 0,                                       \
-        "Forces 32 bit apps to take CLANG\HSAIL path")                        \
-release(bool, GPU_RAW_TIMESTAMP, 0,                                           \
-        "Reports GPU raw timestamps in GPU timeline")                         \
+        "The maximum size accepted for suballocations in KB")                 \
 release(size_t, GPU_NUM_MEM_DEPENDENCY, 256,                                  \
         "Number of memory objects for dependency tracking")                   \
 release(size_t, GPU_XFER_BUFFER_SIZE, 0,                                      \
         "Transfer buffer size for image copy optimization in KB")             \
 release(bool, GPU_IMAGE_DMA, true,                                            \
         "Enable DRM DMA for image transfers")                                 \
-release(uint, GPU_SINGLE_ALLOC_PERCENT, 85,                                   \
+release(uint, GPU_SINGLE_ALLOC_PERCENT, 100,                                  \
         "Maximum size of a single allocation as percentage of total")         \
 release(uint, GPU_NUM_COMPUTE_RINGS, 2,                                       \
         "GPU number of compute rings. 0 - disabled, 1 , 2,.. - the number of compute rings") \
-release(int, GPU_SELECT_COMPUTE_RINGS_ID, -1,                                 \
-        "GPU select the compute rings ID -1 - disabled, 0 , 1,.. - the forced compute rings ID for submission") \
-release(uint, GPU_WORKLOAD_SPLIT, 22,                                         \
-        "Workload split size")                                                \
-release(bool, GPU_USE_SINGLE_SCRATCH, false,                                  \
-        "Use single scratch buffer per device instead of per HW ring")        \
 release(bool, AMD_OCL_WAIT_COMMAND, false,                                    \
         "1 = Enable a wait for every submitted command")                      \
 release(uint, GPU_PRINT_CHILD_KERNEL, 0,                                      \
         "Prints the specified number of the child kernels")                   \
 release(bool, GPU_USE_DEVICE_QUEUE, false,                                    \
         "Use a dedicated device queue for the actual submissions")            \
-release(bool, GPU_ENABLE_LARGE_ALLOCATION, true,                              \
-        "Enable >4GB single allocations")                                     \
 release(bool, AMD_THREAD_TRACE_ENABLE, true,                                  \
         "Enable thread trace extension")                                      \
-release(uint, OPENCL_VERSION, (IS_BRAHMA ? 120 : 200),                        \
-        "Force GPU opencl verison")                                           \
+release(uint, OPENCL_VERSION, 200,                                            \
+        "Force GPU opencl version")                                           \
 release(bool, HSA_LOCAL_MEMORY_ENABLE, true,                                  \
         "Enable HSA device local memory usage")                               \
 release(uint, HSA_KERNARG_POOL_SIZE, 1024 * 1024,                             \
         "Kernarg pool size")                                                  \
-release(bool, HSA_ENABLE_COARSE_GRAIN_SVM, true,                              \
-        "Enable device memory for coarse grain SVM allocations")              \
-release(bool, GPU_IFH_MODE, false,                                            \
-        "1 = Enable GPU IFH (infinitely fast hardware) mode. Any other value keeps setting disabled.") \
 release(bool, GPU_MIPMAP, true,                                               \
         "Enables GPU mipmap extension")                                       \
 release(uint, GPU_ENABLE_PAL, 2,                                              \
@@ -152,12 +120,8 @@ release(int, AMD_GPU_FORCE_SINGLE_FP_DENORM, -1,                              \
         "Force denorm for single precision: -1 - don't force, 0 - disable, 1 - enable") \
 release(uint, OCL_SET_SVM_SIZE, 4*16384,                                      \
         "set SVM space size for discrete GPU")                                \
-debug(uint, OCL_SYSMEM_REQUIREMENT, 2,                                        \
-        "Use flag to change the minimum requirement of system memory not to downgrade")        \
 release(uint, GPU_WAVES_PER_SIMD, 0,                                          \
         "Force the number of waves per SIMD (1-10)")                          \
-release(bool, GPU_WAVE_LIMIT_ENABLE, false,                                   \
-        "1 = Enable adaptive wave limiter")                                   \
 release(bool, OCL_STUB_PROGRAMS, false,                                       \
         "1 = Enables OCL programs stubing")                                   \
 release(bool, GPU_ANALYZE_HANG, false,                                        \
@@ -166,20 +130,6 @@ release(uint, GPU_MAX_REMOTE_MEM_SIZE, 2,                                     \
         "Maximum size (in Ki) that allows device memory substitution with system") \
 release(bool, GPU_ADD_HBCC_SIZE, false,                                        \
         "Add HBCC size to the reported device memory")                        \
-release_on_stg(uint, GPU_WAVE_LIMIT_CU_PER_SH, 0,                             \
-        "Assume the number of CU per SH for wave limiter")                    \
-release_on_stg(uint, GPU_WAVE_LIMIT_MAX_WAVE, 10,                             \
-        "Set maximum waves per SIMD to try for wave limiter")                 \
-release_on_stg(uint, GPU_WAVE_LIMIT_RUN, 20,                                  \
-        "Set running factor for wave limiter")                                \
-release_on_stg(cstring, GPU_WAVE_LIMIT_DUMP, "",                              \
-        "File path prefix for dumping wave limiter output")                   \
-release_on_stg(cstring, GPU_WAVE_LIMIT_TRACE, "",                             \
-        "File path prefix for tracing wave limiter")                          \
-release(bool, OCL_CODE_CACHE_ENABLE, false,                                   \
-        "1 = Enable compiler code cache")                                     \
-release(bool, OCL_CODE_CACHE_RESET, false,                                    \
-        "1 =  Reset the compiler code cache storage")                         \
 release(bool, PAL_DISABLE_SDMA, false,                                        \
         "1 = Disable SDMA for PAL")                                           \
 release(uint, PAL_RGP_DISP_COUNT, 10000,                                      \
@@ -220,6 +170,9 @@ release(uint, AMD_SERIALIZE_KERNEL, 0,                                        \
 release(uint, AMD_SERIALIZE_COPY, 0,                                          \
         "Serialize copies, 0x1 = Wait for completion before enqueue"          \
         "0x2 = Wait for completion after enqueue 0x3 = both")                 \
+release(uint, HIP_LAUNCH_BLOCKING, 0,                                         \
+        "Serialize kernel enqueue 0x1 = Wait for completion after enqueue,"   \
+        "same as AMD_SERIALIZE_KERNEL=2")                                     \
 release(bool, PAL_ALWAYS_RESIDENT, false,                                     \
         "Force memory resources to become resident at allocation time")       \
 release(uint, HIP_HOST_COHERENT, 0,                                           \
@@ -233,8 +186,8 @@ release(bool, AMD_DIRECT_DISPATCH, false,                                     \
 release(uint, HIP_HIDDEN_FREE_MEM, 0,                                         \
         "Reserve free mem reporting in Mb"                                    \
         "0 = Disable")                                                        \
-release(size_t, GPU_FORCE_BLIT_COPY_SIZE, 0,                                  \
-        "Size in KB of the threshold below which to force blit instead for sdma") \
+release(size_t, GPU_FORCE_BLIT_COPY_SIZE, 16,                                 \
+        "Use Blit until this size(in KB) for copies")                         \
 release(uint, ROC_ACTIVE_WAIT_TIMEOUT, 0,                                     \
         "Forces active wait of GPU interrup for the timeout(us)")             \
 release(bool, ROC_ENABLE_LARGE_BAR, true,                                     \
@@ -243,16 +196,14 @@ release(bool, ROC_CPU_WAIT_FOR_SIGNAL, true,                                  \
         "Enable CPU wait for dependent HSA signals.")                         \
 release(bool, ROC_SYSTEM_SCOPE_SIGNAL, true,                                  \
         "Enable system scope for signals (uses interrupts).")                 \
-release(bool, ROC_SKIP_COPY_SYNC, false,                                      \
-        "Skips copy syncs if runtime can predict the same engine.")           \
-release(bool, ROC_ENABLE_PRE_VEGA, false,                                     \
-        "Enable support of pre-vega ASICs in ROCm path")                      \
 release(bool, GPU_FORCE_QUEUE_PROFILING, false,                               \
         "Force command queue profiling by default")                           \
-release(bool, HIP_MEM_POOL_SUPPORT, false,                                    \
+release(bool, HIP_MEM_POOL_SUPPORT, true,                                     \
         "Enables memory pool support in HIP")                                 \
-release(bool, HIP_MEM_POOL_USE_VM, IS_WINDOWS,                                \
+release(bool, HIP_MEM_POOL_USE_VM, true,                                      \
         "Enables memory pool support in HIP")                                 \
+release(bool, PAL_HIP_IPC_FLAG, true,                                         \
+        "Enable interprocess flag for device allocation in PAL HIP")          \
 release(uint, PAL_FORCE_ASIC_REVISION, 0,                                     \
         "Force a specific asic revision for all devices")                     \
 release(bool, PAL_EMBED_KERNEL_MD, false,                                     \
@@ -260,8 +211,6 @@ release(bool, PAL_EMBED_KERNEL_MD, false,                                     \
 release(cstring, ROC_GLOBAL_CU_MASK, "",                                      \
         "Sets a global CU mask (entered as hex value) for all queues,"        \
         "Each active bit represents using one CU (e.g., 0xf enables only 4 CUs)") \
-release(cstring, AMD_LOG_LEVEL_FILE, "",                                      \
-        "Set output file for AMD_LOG_LEVEL, Default is stderr")               \
 release(size_t, PAL_PREPINNED_MEMORY_SIZE, 64,                                \
         "Size in KBytes of prepinned memory")                                 \
 release(bool, AMD_CPU_AFFINITY, false,                                        \
@@ -272,22 +221,61 @@ release(uint, ROC_P2P_SDMA_SIZE, 1024,                                        \
         "The minimum size in KB for P2P transfer with SDMA")                  \
 release(uint, ROC_AQL_QUEUE_SIZE, 16384,                                      \
         "AQL queue size in AQL packets")                                      \
-release(uint, ROC_SIGNAL_POOL_SIZE, 32,                                       \
+release(uint, ROC_SIGNAL_POOL_SIZE, 64,                                       \
         "Initial size of HSA signal pool")                                    \
+release(uint, DEBUG_CLR_LIMIT_BLIT_WG, 16,                                    \
+        "Limit the number of workgroups in blit operations")                  \
+release(bool, DEBUG_CLR_BLIT_KERNARG_OPT, false,                              \
+        "Enable blit kernel arguments optimization")                          \
 release(bool, ROC_SKIP_KERNEL_ARG_COPY, false,                                \
         "If true, then runtime can skip kernel arg copy")                     \
 release(bool, GPU_STREAMOPS_CP_WAIT, false,                                   \
         "Force the stream wait memory operation to wait on CP.")              \
-release(bool, HIP_USE_RUNTIME_UNBUNDLER, true,                                \
+release(bool, HIP_USE_RUNTIME_UNBUNDLER, false,                               \
         "Force this to use Runtime code object unbundler.")                   \
 release(bool, HIPRTC_USE_RUNTIME_UNBUNDLER, false,                            \
         "Set this to true to force runtime unbundler in hiprtc.")             \
 release(size_t, HIP_INITIAL_DM_SIZE, 8 * Mi,                                  \
         "Set initial heap size for device malloc.")                           \
-release(bool, HIP_FORCE_DEV_KERNARG, 0,                                       \
+release(bool, HIP_FORCE_DEV_KERNARG, true,                                    \
          "Force device mem for kernel args.")                                 \
-release(bool, DEBUG_CLR_USE_SDMA_QUERY, 0,                                    \
-         "Use SDMA query API to make copy decisions.")
+release(bool, DEBUG_CLR_GRAPH_PACKET_CAPTURE, true,                           \
+         "Enable/Disable graph packet capturing")                             \
+release(bool, GPU_DEBUG_ENABLE, false,                                        \
+        "Enables collection of extra info for debugger at some perf cost")    \
+release(cstring, HIPRTC_COMPILE_OPTIONS_APPEND, "",                           \
+        "Set compile options needed for hiprtc compilation")                  \
+release(cstring, HIPRTC_LINK_OPTIONS_APPEND, "",                              \
+        "Set link options needed for hiprtc compilation")                     \
+release(bool, HIP_VMEM_MANAGE_SUPPORT, true,                                  \
+        "Virtual Memory Management Support")                                  \
+release(bool, DEBUG_HIP_GRAPH_DOT_PRINT, false,                               \
+         "Enable/Disable graph debug dot print dump")                         \
+release(bool, DEBUG_HIP_FORCE_ASYNC_QUEUE, false,                             \
+        "Forces grpahs into async queue mode. DEBUG_HIP_FORCE_GRAPH_QUEUES must be 1") \
+release(uint, DEBUG_HIP_FORCE_GRAPH_QUEUES, 4,                                \
+        "Forces the number of streams for the graph parallel execution")      \
+release(bool, HIP_ALWAYS_USE_NEW_COMGR_UNBUNDLING_ACTION, false,              \
+        "Force to always use new comgr unbundling action")                    \
+release(uint, DEBUG_HIP_BLOCK_SYNC, 50,                                       \
+        "Blocks synchronization on CPU until the callback processing is done")\
+release(uint, DEBUG_CLR_MAX_BATCH_SIZE, 1000,                                 \
+        "Forces the callback to clean-up CPU submission queue")               \
+release(bool, DEBUG_CLR_SYSMEM_POOL, false,                                   \
+        "Use sysmem pool implementation in runtime for amd commands")         \
+release(bool, DEBUG_HIP_KERNARG_COPY_OPT, true,                               \
+        "Enable/Disable multiple kern arg copies")                            \
+release(bool, DEBUG_CLR_USE_STDMUTEX_IN_AMD_MONITOR, false,                   \
+        "Use std::mutex in amd::monitor")                                     \
+release(bool, DEBUG_CLR_KERNARG_HDP_FLUSH_WA, false,                          \
+        "Toggle kernel arg copy workaround")                                  \
+release(bool, DEBUG_HIP_DYNAMIC_QUEUES, false,                                 \
+        "Forces dynamic queue management")                                    \
+release(uint, DEBUG_HIP_7_PREVIEW, 0,                                         \
+        "Enables specific backward incompatible changes support before 7.0,"  \
+        "using the mask. By default the changes are disabled and is set to 0")\
+release(bool, HIP_FORCE_SPIRV_CODEOBJECT, false,                              \
+         "Force use of SPIRV instead of device specific code object.")        \
 
 namespace amd {
 
@@ -363,7 +351,12 @@ struct Flag {
 #ifdef _WIN32
 # define EXPORT_FLAG extern "C" __declspec(dllexport)
 #else // !_WIN32
+#ifdef BUILD_STATIC_LIBS
+# define EXPORT_FLAG extern
+#else
 # define EXPORT_FLAG extern "C"
+#endif
+namespace amd::flags {
 #endif // !_WIN32
 
 #define DECLARE_RELEASE_FLAG(type, name, value, help) EXPORT_FLAG type name;
@@ -377,5 +370,8 @@ RUNTIME_FLAGS(DECLARE_DEBUG_FLAG, DECLARE_RELEASE_FLAG, DECLARE_DEBUG_FLAG);
 
 #undef DECLARE_DEBUG_FLAG
 #undef DECLARE_RELEASE_FLAG
-
+#ifndef _WIN32
+}
+using namespace amd::flags;
+#endif // !_WIN32
 #endif /*FLAGS_HPP_*/
